/*
TNKernel LP real-time kernel - examples

Copyright ╘ 2004,2007 Yuri Tiomkin
All rights reserved.

Permission to use, copy, modify, and distribute this software in source
and binary forms and its documentation for any purpose and without fee
is hereby granted, provided that the above copyright notice appear
in all copies and that both that copyright notice and this permission
notice appear in supporting documentation.

THIS SOFTWARE IS PROVIDED BY THE YURI TIOMKIN AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL YURI TIOMKIN OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

#include "../MC9S08GT60A.h"
#include "../../tnkernel_lp/tn.h"
#include "utils.h"

//-- Specific Freescale CodeWarrior warnings disabling

#pragma MESSAGE DISABLE C1420 /* C1420: Result of function-call is ignored */
#pragma MESSAGE DISABLE C5703 /* C5703: Parameter declared in function but not referenced */


extern TN_SEM semKBI;


extern tBTNSTATE button1Status;
extern tBTNSTATE button2Status;
extern tBTNSTATE emergStatus;
extern tMOTIONSTATE motionStatus;

//extern unsigned char gKbdIntReg;




static void processingKbdInt(void);

//----------------------------------------------------------------------------
void tn_cpu_int_handler(void)
{
   unsigned char dummy;

   if(tn_int_source == 1) //-- Timer
   {
      //-- Clear Interrupt Source
      //-- Reading flag, then write a zero to the bit.
      dummy = TPM1SC;
      _TPM1SC.Bits.TOF = 0;

      //-- Payload

      tn_tick_int_processing();
   }

   if(tn_int_source == 2)   //-- RTI
   {
      ICGC1_CLKS = 0b01;   //-- Bits 4..3(CLKS)=01 FLL engaged, internal reference

      if(SRTISC_RTIF)       //-- Bit 7(RTIF)- if 1, Periodic wakeup timer timed out
         SRTISC_RTIACK = 1; //-- Bit 6(RTIACK) - write 1 to clear RTIF bit
      if(SPMSC2_PPDF)       //-- Bit 3(PPDF) - if 1, Stop2 mode recovery.
         SPMSC2_PPDACK = 1; //-- Bit 2(PPDACK) write 1 to clear PPDF bit.

      tn_tick_after_stop = TRUE;
      tn_tick_int_processing();
   }

   if(tn_int_source == 3)   //--KBI
   {
      processingKbdInt();

   /*
  //-- Disable KBI pins as Int source

      KBI1PE &=  ~((1<<7) | //-- Bit 7 KBIPE7=0  Bit of KBI port enabled as a keyboard interrupt input
                   (1<<6) | //-- Bit 6 KBIPE6=0
                   (1<<5) | //-- Bit 5 KBIPE5=0
                   (1<<4)); //-- Bit 4 KBIPE4=0

      KBI1SC_KBACK = 1;  //-- Clear Int

  //-- Enable KBI pins as Int source

      KBI1PE  =   ((1<<7) | //-- Bit 7 KBIPE7=1  Bit of KBI port enabled as a keyboard interrupt input
                   (1<<6) | //-- Bit 6 KBIPE6=1
                   (1<<5) | //-- Bit 5 KBIPE5=1
                   (1<<4)); //-- Bit 4 KBIPE4=1
k++;
if(k==128) PTCD&=~LED2_MASK;
      tn_sem_isignal(&semKBI);
*/
   }
}

//----------------------------------------------------------------------------
// Processor specific routine
//
//----------------------------------------------------------------------------
void tn_cpu_int_enable()
{
   tn_enable_interrupt();
}


//----------------------------------------------------------------------------
static void processingKbdInt(void)
{
   unsigned char int_reg;


   KBI1SC_KBIE = 0;  //-- Disable kbd int

   int_reg = KBI1PE; //-- Save curr int enable reg
   KBI1PE = 0;  //-- All pins - input

   KBI1SC_KBACK = 1;  //-- Clear Int

   int_reg &= MASK_MOTION | MASK_BUTTON1 | MASK_BUTTON2 | MASK_EMERG;

   //---- Motion ------

   if(int_reg & MASK_MOTION) //-- Int enabled
   {
      if(PTAD & MASK_MOTION) //-- Motion exists
      {
         PTBD |= LED1_MASK;     //-- LED on  
          
         int_reg &= ~MASK_MOTION; //--  Disable motion int
         motionStatus.state = MTN_STATE_INT_DETECTED;
      }
   }

   //---- Button 1 ------

   if(int_reg & MASK_BUTTON1) //-- Int enabled
   {
      if((PTAD & MASK_BUTTON1) == 0) //-- Btn pressed
      {
         int_reg &= ~MASK_BUTTON1; //--  Disable btn int
         button1Status.state = BTN_STATE_CHK_PRESSED;
       //  button1Status.cnt   = 0;
      }
   }

   //---- Button 2
   if(int_reg & MASK_BUTTON2) //-- Int enabled
   {
      if((PTAD & MASK_BUTTON2) == 0) //-- Btn pressed
      {
         int_reg &= ~MASK_BUTTON2; //--  Disable btn int
         button2Status.state = BTN_STATE_CHK_PRESSED;
        // button2Status.cnt   = 0;
      }
   }

   //---- Emergency
   if(int_reg & MASK_EMERG) //-- Int enabled
   {
      if(PTAD & MASK_EMERG) //-- Emerg Button released
      {
         int_reg &= ~MASK_EMERG; //--  Disable btn int
         emergStatus.state = BTN_STATE_CHK_PRESSED;
         emergStatus.cnt   = 0;
      }
   }
 /*
   //---- ProGen
   if(int_reg & MASK_BUTTON1) //-- Int enabled
   {
      if((PTAD & MASK_BUTTON1) == 0) //-- Btn pressed
      {
         int_reg &= ~MASK_BUTTON1; //--  Disable btn int
         button1Status.state = BTN_STATE_CHK_PRESSED;
      }
   }
 */
 
   PTBD &= ~LED1_MASK;    

   KBI1PE = int_reg; //-- Enable current state int from kbd
   KBI1SC_KBIE = 1;  //-- Enable kbd int
}




