/*
TNKernel LP real-time kernel - examples

Copyright  2004,2007 Yuri Tiomkin
All rights reserved.

Permission to use, copy, modify, and distribute this software in source
and binary forms and its documentation for any purpose and without fee
is hereby granted, provided that the above copyright notice appear
in all copies and that both that copyright notice and this permission
notice appear in supporting documentation.

THIS SOFTWARE IS PROVIDED BY THE YURI TIOMKIN AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL YURI TIOMKIN OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

#include "../MC9S08GT60A.h"
#include "../../tnkernel_lp/tn.h"
#include "utils.h"


//-- Specific Freescale CodeWarrior warnings disabling

#pragma MESSAGE DISABLE C1420 /* C1420: Result of function-call is ignored */
#pragma MESSAGE DISABLE C5703 /* C5703: Parameter declared in function but not referenced */


extern tBTNSTATE button1Status;
extern tBTNSTATE button2Status;
extern tBTNSTATE emergStatus;
extern tMOTIONSTATE motionStatus;

extern unsigned char gKbdIntReg;

extern TN_DQUE queueEvents;

//----------- Prototypes ---

void tn_cpu_isr(void);

static void InitTimer1(void);
static void initButtons(void);


//----------------------------------------------------------------------------
void HardwareInit(void)
{



  //-- Set Port B for LEDs
   PTBDD |= (LED1_MASK | LED2_MASK);   //-- Output
   PTBPE &= ~(LED1_MASK | LED2_MASK);  //-- Disable pull-ups

   initButtons();

   InitTimer1();
}

//----------------------------------------------------------------------------
static void InitTimer1(void)
{
  //-- Bit 7 TOF = 0   ~
  //-- Bit 6 TOIE = 1  -  TOF interrupts enabled
  //-- Bit 5 CPWMS = 0 - counting from $0000 at the next clock
  //-- Bits 4..3 CLKSB:CLKSA = 01 - Bus rate clock (BUSCLK)
  //-- Bits 2..0 PS[2:0] = 110 - Prescaler is 64

   TPM1SC = (1<<6) | (1<<3) | 0x06;

   TPM1CNTH = 0;   //-- Reset counter

   TPM1MODH = 0x08;  //-- Set Int period ~8 mS
   TPM1MODL = 0x3C;
}

//----------------------------------------------------------------------------
static void initButtons(void)
{
   KBI1SC_KBIE = 0; //-- Disable KBD int

   emergStatus.btn_mask   = MASK_EMERG;
   emergStatus.state      = BTN_STATE_WAIT_INT;
   emergStatus.cnt        = 0;

   button1Status.btn_mask  = MASK_BUTTON1;
   button1Status.state     = BTN_STATE_WAIT_INT;
   button1Status.cnt       = 0;

   button2Status.btn_mask  = MASK_BUTTON2;
   button2Status.state     = BTN_STATE_WAIT_INT;
   button2Status.cnt       = 0;

   motionStatus.btn_mask   = MASK_MOTION;
   motionStatus.state      = MTN_STATE_WAIT_INT;
   motionStatus.cnt        = 0;
   motionStatus.integrator = 0;
   motionStatus.status     = MTN_NO_MOTION;

   //--- Hardware settings

   PTAPE   =  0x7F; //  All  Internal pullup device enabled,except motion


   KBI1SC = (1<<7)| //-- KBEDG[7] Motion - Rising edges/high levels
                1;  //-- KBIMOD   Edge-and-level detection

   gKbdIntReg =  MASK_BUTTON1 | MASK_BUTTON2 | MASK_MOTION;
   KBI1PE = gKbdIntReg;

   KBI1SC_KBACK = 1;  //-- Clear Int
   KBI1SC_KBIE  = 1;  //-- Enable KBD int
}

//----------------------------------------------------------------------------
void processingButtons(tBTNSTATE * pbs)
{
   unsigned char par;
   unsigned int data_ptr;

   if(pbs->state == BTN_STATE_WAIT_INT)
      return;

   if(pbs->cnt < 255)
      pbs->cnt++;

   if(pbs->state == BTN_STATE_CHK_PRESSED)
   {
      if((PTAD & pbs->btn_mask) == 0) //-- Btn pressed
      {

         pbs->state = BTN_STATE_PRESSED;
         pbs->cnt = 0;

      }
      else  //-- False pressing
      {
         pbs->state = BTN_STATE_WAIT_INT;

         gKbdIntReg |= pbs->btn_mask; //-- Enable int from kbd
         KBI1PE = gKbdIntReg;
      }
   }
   else if(pbs->state == BTN_STATE_PRESSED)
   {
      if(PTAD & pbs->btn_mask)      //-- Btn released
         pbs->state = BTN_CHK_RELEASED1;
   }
   else if(pbs->state == BTN_CHK_RELEASED1)
   {
      if(PTAD & pbs->btn_mask)      //-- Btn released
         pbs->state = BTN_CHK_RELEASED;
   }
   else if(pbs->state == BTN_CHK_RELEASED)
   {
      if(PTAD & pbs->btn_mask)      //-- Btn released
      {
         if(pbs->cnt < BTN_CNT_2S)
            par = BTN_BELOW_2S;
         else
            par = BTN_ABOVE_2S;

         data_ptr = pbs->btn_mask;
         data_ptr <<= 8;
         data_ptr |= par;
         tn_queue_send_polling(&queueEvents, (void *) data_ptr);

         pbs->state = BTN_STATE_WAIT_INT;

         gKbdIntReg |= pbs->btn_mask; //-- Enable int from kbd
         KBI1PE = gKbdIntReg;
      }
      else //-- False release
         pbs->state = BTN_STATE_PRESSED;
   }
}

//----------------------------------------------------------------------------
void processingEmergBtn(tBTNSTATE * pbs)
{
   if(pbs->state == BTN_STATE_WAIT_INT)
      return;

   if(pbs->cnt < 255)
      pbs->cnt++;

   if(pbs->state == BTN_STATE_CHK_PRESSED)
   {
      if(PTAD & pbs->btn_mask) //-- Emergency btn released
      {
         pbs->state = BTN_STATE_PRESSED;
      }
      else //-- false int
      {
         pbs->state = BTN_STATE_WAIT_INT;

         gKbdIntReg |= pbs->btn_mask; //-- Enable int from kbd
         KBI1PE = gKbdIntReg;
      }
   }
   else if(pbs->state == BTN_STATE_PRESSED)
   {
      if((PTAD & pbs->btn_mask)== 0) //-- Emergency btn pressed
         pbs->state = BTN_CHK_RELEASED;
   }
   else if(pbs->state == BTN_CHK_RELEASED)
   {
      if((PTAD & pbs->btn_mask)== 0) //-- Emergency btn pressed
      {
         pbs->state = BTN_STATE_WAIT_INT;

         gKbdIntReg |= pbs->btn_mask; //-- Enable int from kbd
         KBI1PE = gKbdIntReg;
      }
      else //-- False press
         pbs->state = BTN_STATE_PRESSED;
   }
}

//----------------------------------------------------------------------------
void  processingMotion(tMOTIONSTATE * pms)
{
   if(pms->state == MTN_STATE_INT_DETECTED)
   {
      pms->state = MTN_STATE_PAUSE;
      pms->cnt = 0;
      if(pms->integrator < MTN_TRESHOLD)
      {
         pms->integrator += MTN_STEP_UP;
         if(pms->integrator >= MTN_TRESHOLD)
         {
            pms->status = MTN_MOTION;
         }
      }
   }
   else
   {
      pms->cnt++;
      if(pms->cnt >= MTN_PAUSE_VALUE) //-- Pause or int waiting was expiried
      {
         pms->cnt = 0;

         if(pms->state == MTN_STATE_PAUSE)    //-- Pause was expiried
         {
            pms->state = MTN_STATE_WAIT_INT;

            gKbdIntReg |= pms->btn_mask; //-- Enable int from motion
            KBI1PE = gKbdIntReg;
         }
         else if(pms->state == MTN_STATE_WAIT_INT) // int waiting was expiried
         {
            if(pms->integrator > 0)
            {
               pms->integrator -= MTN_STEP_DOWN;
               if(pms->integrator <= 0)
               {
                  pms->status = MTN_NO_MOTION;
               }
            }
         }
      }
   }
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------




