/******************************************************************************
  FILE        : rtshc08.c
  PURPOSE     : run-time support.
  LANGUAGE    : Inline Assembler / ANSI C.
  ----------------------------------------------------------------------------
  HISTORY
    08/19/98   Created.
    10/06/99   IEEE32 & IEEE64 not using softfloat.
 ******************************************************************************/

#include "non_bank.sgm"

#pragma MESSAGE DISABLE C5703 /* INFORMATION C5703: w: parameter declared in function _LCMP_k_rel_j but not referenced */
#pragma NO_STRING_CONSTR

#define _PUL_RA                  \
  }; asm PULH                  ; \
     asm PULX                  ; \
     asm {

#define _PUSH_REGS   \
  }; asm PSHA      ; \
     asm PSHX      ; \
     asm PSHH      ; \
     asm {

/* opcodes for copy routines on TOS */
#define _LDA_HX  0xD6
#define _STA_HX  0xD7
#define _DECX    0x5A
#define _TSTX    0x5D
#define _BNE     0x26
#define _RTS     0x81
#define _TAX     0x97
#define _CLRH    0x8C
#define _CPHX    0x65
#define _LDHX    0x45
#define _AIX     0xAF

/************ parameter lists for symbolic access in HLI (first element is nearest SP) ************/
#if 0

stack layout 1 for unary operations with return value (k = <op> k):
-------------------------------------------------------------------
    < k:4 .. k:7> only for 64 bit return value
    | k:3  | 11
    | k:2  | 10
    | k:1  |  9
    | k:0  |  8            long k
    | Ra:1 |  7
    | Ra:0 |  6            int Ra
    | A    |  5            char saved_A
    | X    |  4
    | H    |  3            int saved_HX
    | ra:1 |  2            int ra (only if work routine is called)
    | ra:0 |  1


stack layout 2 for binary operations with return value (k = k <op> j):
----------------------------------------------------------------------

    | k:3  | 19
    | k:2  | 18
    | k:1  | 17
    | k:0  | 16            long k
    | Ra:1 | 15
    | Ra:0 | 14            int Ra
    | A    | 13            char saved_A
    | X    | 12
    | H    | 11            int saved_HX
    | w:3  | 10
    | w:2  |  9
    | w:1  |  8
    | w:0  |  7            long w  (additional workspace for * / % and float ops)
    | j:3  |  6
    | j:2  |  5
    | j:1  |  4
    | j:0  |  3            long j
    | ra:1 |  2            int ra (only if work routine is called)
    | ra:0 |  1
#endif

/* stack layout 1 */
#define _PARAM_UNARY_L         short saved_HX, char saved_A, short Ra, long k, ...
#define _PARAM_UNARY_L_INLINE  char saved_A, short Ra, long k, ...
#define _PARAM_UNARY_L64_INLINE  char saved_A, short Ra, double k, ...
#define _PARAM_UNARY_F         char temp, char Kspec, short Kexp, short saved_HX, char saved_A, short Ra, FloatReg K, ...

/* stack layout 2 */
#define _PARAM_BINARY_L         long j, long w, short saved_HX, char saved_A, short Ra, long k, ...
#define _PARAM_BINARY_L64       double j, double k, short saved_HX, char saved_A, short Ra, double res, ...


/*-------------------- tabled functions used with options -ot -oilib=g ----------------*/

/* 1 << char */
const unsigned char _PowOfTwo_8[] = {1u << 0, 1u << 1, 1u << 2, 1u << 3, 1u << 4, 1u << 5, 1u << 6, 1u << 7};
/* 1 << int */
const unsigned short _PowOfTwo_16[] =
{1u << 0, 1u << 1, 1u << 2, 1u << 3, 1u << 4, 1u << 5, 1u << 6, 1u << 7, 1u << 8, 1u << 9, 1u << 10, 1u << 11, 1u << 12, 1u << 13, 1u << 14 , 1u << 15};
/* 1L << long */
const unsigned long _PowOfTwo_32[] =
{1UL <<  0, 1UL <<  1, 1UL <<  2, 1UL <<  3, 1UL <<  4, 1UL <<  5, 1UL <<  6, 1UL << 7,  1UL <<  8, 1UL <<  9, 1UL << 10, 1UL << 11, 1UL << 12, 1UL << 13, 1UL << 14, 1UL << 15,
 1UL << 16, 1UL << 17, 1UL << 18, 1UL << 19, 1UL << 20, 1UL << 21, 1UL << 22, 1UL << 23, 1UL << 24, 1UL << 25, 1UL << 26, 1UL << 27, 1UL << 28, 1UL << 29, 1UL << 30, 1UL << 31};



/************************** enter and exit functions or macros *******************************/

#define _ADDR_1ST 7
#define _ADDR_RA  5
#define _ADDR_RES 14
#define _ALLOC_IEEE32_WSP (-13)
#define _ALLOC_IEEE64_WSP (-22)
#define _FREE_IEEE32_WSP  23
#define _FREE_IEEE64_WSP  40
#define _ALLOC_IEEE32_WSP_UNARY (-4)
#define _ALLOC_IEEE64_WSP_UNARY (-4)
#define _FREE_IEEE32_WSP_UNARY  6
#define _FREE_IEEE64_WSP_UNARY  6

/* entries */

static void _PUSH_ARGS_L(void) {
/* push 2 4 byte arguments on stack (stack layout 2) */
#if 0
preliminary H:X contents: @j

preliminary stack layout:

     |@i:1 |
     |@i:0 |
     |ra:1 |
     |ra:0 |

#endif
  asm {
        LDA  3,X                 ; // push 2nd argument j (addr in H:X)
        PSHA
        LDA  2,X
        PSHA
#ifdef __HCS08__
        LDHX 0,X
        PSHX
        PSHH
        LDHX _ADDR_1ST,SP        ; // insert 1st argument i (address of it must be on fix stack location)
#else
        LDA  1,X
        PSHA
        LDA  0,X
        PSHA
        LDX  _ADDR_1ST,SP        ; // insert 1st argument i (address of it must be on fix stack location)
        PSHX
        PULH
        LDX  _ADDR_1ST+1,SP
#endif
        LDA  3,X
        STA  _ADDR_RES+3,SP
        LDA  2,X
        STA  _ADDR_RES+2,SP
#ifdef __HCS08__
        LDHX 0,X
        STHX _ADDR_RES,SP
        LDHX _ADDR_RA,SP           ; // push return address of "_ENTER_BINARY_L(_LC|_RC)"
        JMP  0,X
#else
        LDA  1,X
        STA  _ADDR_RES+1,SP
        LDA  0,X
        STA  _ADDR_RES,SP
        LDA  _ADDR_RA+1,SP          ; // push return address of "_ENTER_BINARY_L(_LC|_RC)"
        PSHA
        LDA  _ADDR_RA+1,SP
        PSHA
        RTS ! {}, {HX+A+SR}
#endif
  }
}

#define _ADDR_2ND_D 20
#define _ADDR_RA_D  26

static void _PUSH_ARGS_D(void) {
/* push 2 8 byte arguments on stack (stack layout 2) */
#if 0
preliminary H:X contents: @j

preliminary stack layout:

     |@i:1 |
     |@i:0 |
     |ra:1 |
     |ra:0 |

#endif
  asm {
        LDA  7,X                 ; // push 1st argument i (addr in H:X)
        PSHA
        LDA  6,X
        PSHA
        LDA  5,X
        PSHA
        LDA  4,X
        PSHA
        LDA  3,X
        PSHA
        LDA  2,X
        PSHA
#ifdef __HCS08__
        LDHX 0,X
        PSHX
        PSHH
        LDHX _ADDR_2ND_D,SP        ; // push 1st argument i (address of it must be on fix stack location)
#else
        LDA  1,X
        PSHA
        LDA  0,X
        PSHA
        LDA  _ADDR_2ND_D,SP        ; // push 1st argument i (address of it must be on fix stack location)
        PSHA
        LDX  _ADDR_2ND_D+1+1,SP
        PULH                       ; // and now in H:X
#endif
        LDA  7,X
        PSHA
        LDA  6,X
        PSHA
        LDA  5,X
        PSHA
        LDA  4,X
        PSHA
        LDA  3,X
        PSHA
        LDA  2,X
        PSHA
#ifdef __HCS08__
        LDHX 0,X
        PSHX
        PSHH
        LDHX _ADDR_RA_D,SP          ; // push return address of "_ENTER_BINARY_D(_LC|_RC)"
        JMP  0,X
#else
        LDA  1,X
        PSHA
        LDA  0,X
        PSHA
        LDA  _ADDR_RA_D+1,SP          ; // push return address of "_ENTER_BINARY_D(_LC|_RC)"
        PSHA
        LDA  _ADDR_RA_D+1,SP
        PSHA
        RTS ! {}, {HX+A+SR}
#endif
  }
}


static void _ENTER_UNARY_L(void) {
  asm {
        AIS  #-2
        _PUSH_REGS
        LDA  7,SP          ; // push ra
        PSHA
        LDA  7,SP
        PSHA
        LDA  10,SP         ; // put return address of interface routine to required place
        STA  6,SP
        LDA  11,SP
        STA  7,SP
        LDA  3,X           ; // insert argument into result space
        STA  11,SP
        LDA  2,X
        STA  10,SP
#ifdef __HCS08__
        LDHX 0,X
        STHX 8,SP
#else
        LDA  1,X
        STA  9,SP
        LDA  0,X
        STA  8,SP
#endif
        RTS  ! {}, {HX+A+SR}
  }
}

#define _ENTER_UNARY_F() \
 }; asm JSR _ENTER_UNARY_L           ; \
    asm AIS #_ALLOC_IEEE32_WSP_UNARY ; \
    asm {


static void _ENTER_UNARY_L64(void) {
  asm {
        AIS  #-6
        _PUSH_REGS
        LDA  11,SP         ; // push ra
        PSHA
        LDA  11,SP
        PSHA
        LDA  14,SP         ; // put return address of interface routine to required place
        STA  6,SP
        LDA  15,SP
        STA  7,SP
        LDA  7,X           ; // insert argument into result space
        STA  15,SP
        LDA  6,X
        STA  14,SP
        LDA  5,X
        STA  13,SP
        LDA  4,X
        STA  12,SP
        LDA  3,X
        STA  11,SP
        LDA  2,X
        STA  10,SP
#ifdef __HCS08__
        LDHX 0,X
        STHX 8,SP
#else
        LDA  1,X
        STA  9,SP
        LDA  0,X
        STA  8,SP
#endif
        RTS  ! {}, {HX+A+SR}
  }
}

static void _ENTER_UNARY_L64_4(void) {  /* copy only 4 bytes */
  asm {
        AIS  #-6
        _PUSH_REGS
        LDA  11,SP         ; // push ra
        PSHA
        LDA  11,SP
        PSHA
        LDA  14,SP         ; // put return address of interface routine to required place
        STA  6,SP
        LDA  15,SP
        STA  7,SP
        LDA  3,X           ; // insert 4 byte large argument into result space
        STA  15,SP
        LDA  2,X
        STA  14,SP
#ifdef __HCS08__
        LDHX 0,X
        STHX 12,SP
#else
        LDA  1,X
        STA  13,SP
        LDA  0,X
        STA  12,SP
#endif
        RTS  ! {}, {HX+A+SR}
  }
}


#define _ENTER_UNARY_D() \
 }; asm JSR _ENTER_UNARY_L64         ; \
    asm AIS #_ALLOC_IEEE64_WSP_UNARY ; \
    asm {

#define _ENTER_UNARY_D4() \
 }; asm JSR _ENTER_UNARY_L64_4       ; \
    asm AIS #_ALLOC_IEEE64_WSP_UNARY ; \
    asm {


static void _ENTER_BINARY_L(void) {
  asm {
        _PUSH_REGS
        PSHX              ; // @i
        PSHH
#ifdef __HCS08__
        LDHX 6,SP         ; // ra
        PSHX
        PSHH
        LDHX 10,SP        ; // put return address of interface routine to required place
        STHX 8,SP
        LDHX 12,SP        ; // @j in H:X
#else
        TSX
        LDA  6,X          ; // ra
        PSHA
        LDA  5,X
        PSHA
        LDA  7,X          ; // put return address of interface routine to required place
        STA  5,X
        LDA  8,X
        STA  6,X
        LDX  12,SP        ; // @j in H:X
        PSHX
        PULH
        LDX  13,SP
#endif
        JMP  _PUSH_ARGS_L
  }
}

#define _ENTER_BINARY_F() \
 }; asm JSR _ENTER_BINARY_L    ; \
    asm AIS #_ALLOC_IEEE32_WSP ; \
    asm {


static void _ENTER_BINARY_L_RC(void) {
  asm {
        AIS  #-2
        _PUSH_REGS
        PSHX              ; // @i
        PSHH
#ifdef __HCS08__
        LDHX 8,SP         ; // ra
        PSHX
        PSHH
        LDHX 12,SP        ; // put return address of interface routine to required place and advance it by 4
        AIX  #4
        STHX 8,SP
        AIX  #-4          ; // @j (= return address of interface) in H:X
#else
        TSX
        LDA  8,X          ; // ra
        PSHA
        LDA  7,X
        PSHA
        LDA  10,X         ; // put return address of interface routine to required place and advance it by 4
        PSHA
        ADD  #4
        STA  6,X
        LDA  9,X
        PSHA
        ADC  #0
        STA  5,X
        PULH              ; // @j (= return address of interface) in H:X
        PULX
#endif
        JMP  _PUSH_ARGS_L
  }
}

#define _ENTER_BINARY_F_RC() \
 }; asm JSR _ENTER_BINARY_L_RC ; \
    asm AIS #_ALLOC_IEEE32_WSP ; \
    asm {


static void _ENTER_BINARY_L_LC(void) {
  asm {
        AIS  #-2
        _PUSH_REGS
#ifdef __HCS08__
        LDHX 8,SP          ; // put return address of interface routine to required place and advance it by 4
        PSHX
        PSHH
        AIX  #4
        STHX 6,SP          ; // @i (= return address of interface) is TOS
        LDHX 8,SP          ; // ra
        PSHX
        PSHH
        LDHX 5,SP          ; // @j in H:X
#else
        TSX
        LDA  8,X           ; // put return address of interface routine to required place and advance it by 4
        PSHA
        ADD  #4
        STA  4,X
        LDA  7,X
        PSHA
        ADC  #0
        STA  3,X           ; // @i (= return address of interface) is TOS
        LDA  6,X           ; // ra
        PSHA
        LDA  5,X
        PSHA
        LDX  5,SP          ; // @j in H:X
        PSHX
        PULH
        LDX  6,SP
#endif
        JMP  _PUSH_ARGS_L
  }
}

#define _ENTER_BINARY_F_LC() \
 }; asm JSR _ENTER_BINARY_L_LC ; \
    asm AIS #_ALLOC_IEEE32_WSP ; \
    asm {


static void _ENTER_BINARY_L64(void) {
  asm {
        AIS  #-4
        _PUSH_REGS
#ifdef __HCS08__
        LDHX 10,SP         ; // put return address of interface routine to required place
        STHX 4,SP
        LDHX 8,SP          ; // return address of this routine to to required place
        STHX 10,SP
        LDHX 1,SP          ; // @i in H:X
#else
        TSX
        LDA  9,X           ; // put return address of interface routine to required place
        STA  3,X
        LDA  10,X
        STA  4,X
        LDA  7,X           ; // return address of this routine to to required place
        STA  9,X
        LDA  8,X
        STA  10,X
        LDX  0,X           ; // @i in H:X
        PSHX
        PULH
        LDX  2,SP
#endif
        JMP  _PUSH_ARGS_D
  }
}

#define _ENTER_BINARY_D() \
 }; asm JSR _ENTER_BINARY_L64  ; \
    asm AIS #_ALLOC_IEEE64_WSP ; \
    asm {


static void _ENTER_BINARY_L64_LC(void) {
  asm {
        AIS  #-6
        _PUSH_REGS
#ifdef __HCS08__
        LDHX 12,SP         ; // put return address of interface routine to required place and advance it by 8
        PSHX
        PSHH
        AIX  #8
        STHX 6,SP
        LDHX 3,SP
        STHX 14,SP
        PULH               ; // @i (= return address of interface) in H:X
        PULX
#else
        TSX
        LDA  12,X          ; // put return address of interface routine to required place and advance it by 8
        PSHA
        ADD  #8
        STA  4,X
        LDA  11,X
        PSHA
        ADC  #0
        STA  3,X
        LDA  0,X           ; // @j to location expected by _PUSH_ARGS_D
        STA  11,X
        LDA  1,X
        STA  12,X
        PULH               ; // @i (= return address of interface) in H:X
        PULX
#endif
        JMP  _PUSH_ARGS_D
  }
}

#define _ENTER_BINARY_D_LC() \
 }; asm JSR _ENTER_BINARY_L64_LC ; \
    asm AIS #_ALLOC_IEEE64_WSP   ; \
    asm {


static void _ENTER_BINARY_L64_RC(void) {
  asm {
        AIS  #-6
        _PUSH_REGS
#ifdef __HCS08__
        LDHX 12,SP          ; // put return address of interface routine (= @j) to required place and advance it by 8
        AIX  #8
        STHX 4,SP
        LDHX 1,SP           ; // @i in H:X
#else
        TSX
        LDA  12,X           ; // put return address of interface routine (= @j) to required place and advance it by 8
        ADD  #8
        STA  4,X
        LDA  11,X
        ADC  #0
        STA  3,X
        LDX  0,X            ; // @i in H:X
        PSHX
        PULH
        LDX  2,SP
#endif
        JMP  _PUSH_ARGS_D
  }
}

#define _ENTER_BINARY_D_RC() \
 }; asm JSR _ENTER_BINARY_L64_RC ; \
    asm AIS #_ALLOC_IEEE64_WSP   ; \
    asm {


/* exits: no code of runtime support will be executed after ! */

#define _EXIT_UNARY_L \
 }; asm PULH           ; \
    asm PULX           ; \
    asm PULA           ; \
    asm _SRET          ; \
    asm {

#define _EXIT_UNARY_F \
 }; asm AIS #_FREE_IEEE32_WSP_UNARY ; \
    asm PULH           ; \
    asm PULX           ; \
    asm PULA           ; \
    asm _SRET          ; \
    asm {

#define _EXIT_UNARY_L64 _EXIT_UNARY_L

#define _EXIT_UNARY_D \
 }; asm AIS #_FREE_IEEE64_WSP_UNARY ; \
    asm PULH           ; \
    asm PULX           ; \
    asm PULA           ; \
    asm _SRET          ; \
    asm {

#ifdef __HCS08__
#define _EXIT_UNARY_D4 \
 }; asm AIS #_FREE_IEEE64_WSP_UNARY ; \
    asm LDHX   4,SP    ; \
    asm STHX   8,SP    ; \
    asm PULH           ; \
    asm PULX           ; \
    asm PULA           ; \
    asm AIS    #4      ; \
    asm _SRET          ; \
    asm {
#else
#define _EXIT_UNARY_D4 \
 }; asm AIS #_FREE_IEEE64_WSP_UNARY ; \
    asm LDA    4,SP    ; \
    asm STA    8,SP    ; \
    asm LDA    5,SP    ; \
    asm STA    9,SP    ; \
    asm PULH           ; \
    asm PULX           ; \
    asm PULA           ; \
    asm AIS    #4      ; \
    asm _SRET          ; \
    asm {
#endif

#define _EXIT_BINARY_L \
 }; asm AIS #10        ; \
    asm PULH           ; \
    asm PULX           ; \
    asm PULA           ; \
    asm _SRET          ; \
    asm {

#define _EXIT_BINARY_F \
 }; asm AIS #_FREE_IEEE32_WSP ; \
    asm PULH                  ; \
    asm PULX                  ; \
    asm PULA                  ; \
    asm _SRET                 ; \
    asm {

#define _EXIT_BINARY_D \
 }; asm AIS #_FREE_IEEE64_WSP ; \
    asm PULH                  ; \
    asm PULX                  ; \
    asm PULA                  ; \
    asm _SRET                 ; \
    asm {


/************************** integer work routines ************************/

int errno;


#pragma NO_EXIT
static void _IDIVMOD (char dummy_sgn, int j, int dummy, int i, ...) {
/**************************************************************************
    a := a % b,
    b := a / b,
------------------------------------
  algorithm: b == b1:b0, a == a1:a0
------------------------------------
  if (b1 == 0) {
    <simple algo>
  } else if (a < b) {
    <common shift algo (8 shifts suffice)>
  }
***************************************************************************/
  asm {
                TST     j:0
                BNE     _IDIVMOD_0    ; // b >= 256
                ; // {
                ; //    j < 256
                ; // }
                LDA     i:0
                LDX     j:1
                CLRH
                DIV
                STA     j:0
                LDA     i:1
                DIV
                STA     j:1
                CLR     i:0
                PSHH
                PULA
                STA     i:1
                RTS             ; // end
    _IDIVMOD_0: ; // {
                ; //    j >= 256
                ; // }
                CLRA
                PSHA
                ; //
#define r i:-8 /** the value pushed on the stack **/
                ; //
                LDX     #8
                CLC
                ;
    _IDIVMOD_1: ;
                ROL     i:1
                ROL     i:0
                ROL     r
                LDA     j:0
                CMP     r
                BHI     _IDIVMOD_3   ; // carry cleared !
                BNE     _IDIVMOD_2
                LDA     j:1
                CMP     i:0
                BHI     _IDIVMOD_3   ; // carry cleared !
    _IDIVMOD_2: ;
                LDA     i:0
                SUB     j:1
                STA     i:0
                LDA     r
                SBC     j:0
                STA     r
                SEC
                ;
    _IDIVMOD_3: ;
                DBNZX   _IDIVMOD_1
                ;
                LDA     i:1
                ROLA
                STA     j:1
                LDA     i:0
                STA     i:1
                PULA            ; // that is 'r'
                STA     i:0
                CLR     j:0
                ;
                RTS
  }
}


#pragma NO_EXIT
static void _SPLITSIGN (char dummy1, int j,  int dummy, int i, ...) {
  asm {
                TST     i:0
                BPL     _SPLIT_0
                COM     i:0
                NEG     i:1
                BNE     _SPLIT_0
                INC     i:0
    _SPLIT_0:   ;
                TST     j:0
                BPL     _SPLIT_1
                COM     j:0
                NEG     j:1
                BNE     _SPLIT_1
                INC     j:0
    _SPLIT_1:   ;
; //               RTS ! {A}, {HX+A+SR}
                JMP     _IDIVMOD  ! {}, {HX+A+SR}
  }
}

#pragma NO_EXIT
static void _LADD_k_is_k_plus_j(_PARAM_BINARY_L) {
/* k = k + j */
  asm {
                LDA     k:3
                ADD     j:3
                STA     k:3
                LDA     k:2
                ADC     j:2
                STA     k:2
                LDA     k:1
                ADC     j:1
                STA     k:1
                LDA     k:0
                ADC     j:0
                STA     k:0
                _EXIT_BINARY_L
                RTS
  }
}

#pragma NO_EXIT
static void _k_is_k_plus_j_l(long j, long k, ...) {
/* k = k + j */
  asm {
                LDA     k:3
                ADD     j:3
                STA     k:3
                LDA     k:2
                ADC     j:2
                STA     k:2
                LDA     k:1
                ADC     j:1
                STA     k:1
                LDA     k:0
                ADC     j:0
                STA     k:0
                _PUL_RA
                AIS     #4
                JMP     ,X
  }
}

#pragma NO_EXIT
static void _k_is_k_plus_j_i(int j, int k, ...) {
/* k = k + j */
  asm {
                LDA     k:1
                ADC     j:1
                STA     k:1
                LDA     k:0
                ADC     j:0
                STA     k:0
                _PUL_RA
                AIS     #2
                JMP     ,X
  }
}

#pragma NO_EXIT
static void _LSUB_k_is_k_minus_j(_PARAM_BINARY_L) {
/* k = k - j */
  asm {
                LDA     k:3
                SUB     j:3
                STA     k:3
                LDA     k:2
                SBC     j:2
                STA     k:2
                LDA     k:1
                SBC     j:1
                STA     k:1
                LDA     k:0
                SBC     j:0
                STA     k:0
                _EXIT_BINARY_L
                RTS
  }
}

#pragma NO_EXIT
static void _LAND_k_is_k_and_j(_PARAM_BINARY_L) {
/* k = k & j */
  asm {
                LDA     k:3
                AND     j:3
                STA     k:3
                LDA     k:2
                AND     j:2
                STA     k:2
                LDA     k:1
                AND     j:1
                STA     k:1
                LDA     k:0
                AND     j:0
                STA     k:0
                _EXIT_BINARY_L
                RTS
  }
}

#pragma NO_EXIT
static void _LOR_k_is_k_or_j(_PARAM_BINARY_L) {
/* k = k | j */
  asm {
                LDA     k:3
                ORA     j:3
                STA     k:3
                LDA     k:2
                ORA     j:2
                STA     k:2
                LDA     k:1
                ORA     j:1
                STA     k:1
                LDA     k:0
                ORA     j:0
                STA     k:0
                _EXIT_BINARY_L
                RTS
  }
}

#pragma NO_EXIT
static void _LXOR_k_is_k_xor_j(_PARAM_BINARY_L) {
/* k = k ^ j */
  asm {
                LDA     k:3
                EOR     j:3
                STA     k:3
                LDA     k:2
                EOR     j:2
                STA     k:2
                LDA     k:1
                EOR     j:1
                STA     k:1
                LDA     k:0
                EOR     j:0
                STA     k:0
                _EXIT_BINARY_L
                RTS
  }
}

#pragma NO_EXIT
static void _LMUL_k_is_k_mul_j(_PARAM_BINARY_L) {
/* k = k * j */
  asm {
                LDA      k:3
                LDX      j:3
                MUL
                PSHA
                PSHX
                CLRH
                PSHH
                PSHH
                LDA      k:3
                LDX      j:2
                MUL
                PSHH
                PSHA
                PSHX
                PSHH
                LDA      k:2
                LDX      j:3
                MUL
                PSHH
                PSHA
                PSHX
                PSHH
                JSR      _k_is_k_plus_j_l
                _STACK   #4                           ; // --> new HLI must support this to enable symbolic access
                JSR      _k_is_k_plus_j_l
                _STACK   #4
                LDA      k:3
                LDX      j:1
                MUL
                PSHA
                PSHX
                JSR      _k_is_k_plus_j_i
                _STACK   #2
                LDA      k:2
                LDX      j:2
                MUL
                PSHA
                PSHX
                JSR      _k_is_k_plus_j_i
                _STACK   #2
                LDA      k:1
                LDX      j:3
                MUL
                PSHA
                PSHX
                JSR      _k_is_k_plus_j_i
                _STACK   #2
    _MUL_1:     ; // reference point for adjust
                LDA      k:3
                LDX      j:0
                MUL
                PSHA
                LDA      k:2
                LDX      j:1
                MUL
                PSHA
                LDA      k:1
                LDX      j:2
                MUL
                PSHA
                LDA      k:0
                LDX      j:3
                MUL
                ADD      1,SP
                ADD      2,SP
                ADD      3,SP
                ADD      4,SP
                STA      k:0
                AIS #4
                PULA
                STA      k:1
                PULA
                STA      k:2
                PULA
                STA      k:3
                _EXIT_BINARY_L
                RTS
  }
}

static void _LDIVMOD (char dummy_sgn, int ret, _PARAM_BINARY_L) {
/**************************************************************************
    k := k % j,
    j := k / j,
---------------------------------------
  algorithm: j == j0..j3, k == k0..k3
---------------------------------------
  if (j < 256) {
    j0, k0 =  0:k0 / j3,  0:k0 % j3;
    j1, k1 = k0:k1 / j3, k0:k1 % j3;
    j2, k2 = k1:k2 / j3, k1:k2 % j3;
    j3, k3 = k2:k3 / j3, k2:k3 % j3;
    k0..k2 = 0..0;
  } else {
    for (i = 0, r = k, q = 0; i < 32; i++) {
      r, q = 2*r, 2*q;
      if (r >= exp(2, 32)*j) {
        r -= exp(2, 32)*j;
        q += 1;
      }
    }
  }
***************************************************************************/
  long t;

  asm {
                LDA     j:0
                ORA     j:1
                ORA     j:2
                BNE     _LDIVMOD_0
                ; // {
                ; //   j < 256
                ; // }
                LDA     k:0
                LDX     j:3
                CLRH
                DIV
                STA     j:0
                LDA     k:1
                DIV
                STA     j:1
                LDA     k:2
                DIV
                STA     j:2
                LDA     k:3
                DIV
                STA     j:3
                PSHH
                PULA
                STA     k:3
                CLR     k:2
                CLR     k:1
                CLR     k:0
  }
  return;

  /*------------------------------------*/
  asm {
    _LDIVMOD_0: ; // {
                ; //   j >= 256;
                ; // }
                /** t = j; j = k; k = 0
                    may not write C code since global vars
                    would be destroyed!
                **/
#ifdef __HCS08__
                LDHX    j:2
                STHX    t:2
                LDHX    j:0
                STHX    t:0
                LDHX    k:2
                STHX    j:2
                LDHX    k:0
                STHX    j:0
                CLRX
                CLRH
                STHX    k:2
                STHX    k:0
#else
                LDA     j:3
                STA     t:3
                LDA     j:2
                STA     t:2
                LDA     j:1
                STA     t:1
                LDA     j:0
                STA     t:0
                LDA     k:3
                STA     j:3
                LDA     k:2
                STA     j:2
                LDA     k:1
                STA     j:1
                LDA     k:0
                STA     j:0
                CLR     k:3
                CLR     k:2
                CLR     k:1
                CLR     k:0
#endif
                ;
                LDX     #32
                ;
    _LDIVMOD_1: ; // {
                ; //   loop starts here
                ; // }
                ASL     j:3
                ROL     j:2
                ROL     j:1
                ROL     j:0
                ROL     k:3
                ROL     k:2
                ROL     k:1
                ROL     k:0
#ifdef __HCS08__
                PSHX
                LDHX    k:0
                CPHX    t:0
                BHI     _LDIVMOD_2
                BLO     _LDIVMOD_3
                LDHX    k:2
                CPHX    t:2
                BLO     _LDIVMOD_3
#else
                LDA     k:0
                CMP     t:0
                BHI     _LDIVMOD_2
                BLO     _LDIVMOD_3
                LDA     k:1
                CMP     t:1
                BHI     _LDIVMOD_2
                BLO     _LDIVMOD_3
                LDA     k:2
                CMP     t:2
                BHI     _LDIVMOD_2
                BLO     _LDIVMOD_3
                LDA     k:3
                CMP     t:3
                BLO     _LDIVMOD_3
#endif
    _LDIVMOD_2: ; // {
                ; //   r >= exp*j
                ; // }
                LDA     k:3
                SUB     t:3
                STA     k:3
                LDA     k:2
                SBC     t:2
                STA     k:2
                LDA     k:1
                SBC     t:1
                STA     k:1
                LDA     k:0
                SBC     t:0
                STA     k:0
                INC     j:3
    _LDIVMOD_3: ; // {
                ; //   [k:j] < exp(32) * t;
                ; // }
#ifdef __HCS08__
                PULX
#endif
                DBNZX   _LDIVMOD_1
                ;
  }
}

#pragma NO_EXIT
static void _NEG_L_HX (void) {
/* negate [HX] */
  asm {
                COM      0,X
                COM      1,X
                COM      2,X
                NEG      3,X
                BNE      _NEG_L_HX_0
                INC      2,X
                BNE      _NEG_L_HX_0
                INC      1,X
                BNE      _NEG_L_HX_0
                INC      0,X
  _NEG_L_HX_0:     ;
                RTS
  }
}

#pragma NO_EXIT
static void _ABS_L_HX (void) {
/* labs([HX]) */
  asm {
                TST      0,X
                BPL      _ABS_L_HX_0
                JSR      _NEG_L_HX
  _ABS_L_HX_0:     ;
                RTS
  }
}

#pragma NO_EXIT
static void _SPLITSIGN_L (char resSign, int ret, _PARAM_BINARY_L) {
/* labs(j), labs(k) */
  asm {
                LDHX    @j
                JSR     _ABS_L_HX
                LDHX    @k
                JMP     _ABS_L_HX
  }
}

#pragma NO_EXIT
static void _LMODU_k_is_k_mod_j(void) {
/* k = k % j */
  asm {
                AIS      #-1
                JSR      _LDIVMOD ! {}, {HX+A+SR}
                AIS      #11
                PULH
                PULX
                PULA
                RTS
  }
}

#pragma NO_EXIT
static void _LDIVU_k_is_k_div_j(_PARAM_BINARY_L) {
/* k = k / j */
  asm {
                AIS      #-1
                JSR      _LDIVMOD ! {}, {HX+A+SR}
#ifdef __HCS08__
                LDHX     j:0
                STHX     k:0
                LDHX     j:2
                STHX     k:2
#else
                LDA      j:0
                STA      k:0
                LDA      j:1
                STA      k:1
                LDA      j:2
                STA      k:2
                LDA      j:3
                STA      k:3
#endif
                AIS      #11
                PULH
                PULX
                PULA
                RTS
  }
}

#pragma NO_EXIT
static void _LMODS_k_is_k_mod_j(_PARAM_BINARY_L) {
/* k = k % j */
  asm {
                LDA     k:0
                PSHA
                JSR      _SPLITSIGN_L
                JSR      _LDIVMOD
                PULA
                TSTA
                BPL      _DVS_0     ; // done
                LDHX     @k
                JSR      _NEG_L_HX
     _DVS_0:    ;
                _EXIT_BINARY_L
                RTS
  }
}

#pragma NO_EXIT
static void _LDIVS_k_is_k_div_j(_PARAM_BINARY_L) {
/* k = k / j */
  asm {
                LDA     j:0
                EOR     k:0
                PSHA
                JSR      _SPLITSIGN_L
                JSR      _LDIVMOD
                PULA
                TSTA
                BPL      _MDS_0     ; // done
                LDHX     @j
                JSR      _NEG_L_HX
     _MDS_0:    ;
#ifdef __HCS08__
                LDHX     j:0
                STHX     k:0
                LDHX     j:2
                STHX     k:2
#else
                LDA      j:0
                STA      k:0
                LDA      j:1
                STA      k:1
                LDA      j:2
                STA      k:2
                LDA      j:3
                STA      k:3
#endif
                _EXIT_BINARY_L
                RTS
  }
}

#pragma NO_EXIT
static void _LCMP_k_rel_j(_PARAM_BINARY_L) {
/* CC = k <rel> j */
  asm {
#ifdef __HCS08__
                LDHX    k:0
                CPHX    j:0
                BNE     _CMP_0
                LDHX    k:2
                CPHX    j:2
                BEQ     _CMP_0
#else
                LDA     k:0
                CMP     j:0
                BNE     _CMP_0
                LDA     k:1
                CMP     j:1
                BNE     _CMP_1
                LDA     k:2
                CMP     j:2
                BNE     _CMP_1
                LDA     k:3
                CMP     j:3
                BEQ     _CMP_0
#endif
    _CMP_1:     ;
                CLRA
                BCS     _CMP_2
                LDA     #2
    _CMP_2:     ;
                CMP     #1
    _CMP_0:     ; // flags are set for a - b
                AIS     #12
                PULA
                PULH
                PULX
                AIS     #4
                _SRET
                JMP     ,X
  }
}

#if defined(__HCS08__)
#pragma NO_EXIT
static void _CMP24_k_rel_j(_PARAM_BINARY_L) {
/* CC = k <rel> j */
  asm {
                LDHX    k:0
                CPHX    j:0
                BNE     _CMP_0
                LDX    k:2
                CPX    j:2
                BEQ     _CMP_0
    _CMP_1:     ;
                CLRA
                BCS     _CMP_2
                LDA     #2
    _CMP_2:     ;
                CMP     #1
    _CMP_0:     ; // flags are set for a - b
                AIS     #12
                PULA
                PULH
                PULX
                AIS     #4
                _SRET
                JMP     ,X
  }
}
#endif


/******************************** interface runtime routines *****************************/

void _BMULS(void) {
  /* DESC:    signed byte multiplication 16=8x8 :  z = x*y
   IN:      A x, X y
   OUT:     A low(z), X high(z)
   WRITTEN: CCR */
  asm {
                PSHA
                PSHX
                MUL
                PSHA
                TXA
                TST 2,SP ; // test sign of x
                BPL xpos
                SUB 3,SP ; // subtract y from high byte of result
xpos:
                TST 3,SP ; // test sign of y
                BPL ypos
                SUB 2,SP ; // subtract x from high byte of result
ypos:
                TAX
                PULA
                AIS  #2
  }
}

#pragma NO_EXIT
void _BDIVS(void) {
/* DESC:    byte division signed  z = x/y
   IN:      A x, X y
   OUT:     A z
   WRITTEN: A 
   Result is negative if x and y have different sign
  */
  asm {
                PSHX
                PSHH
                CLRH
                TSTX
                BPL    _XPOS
_XNEG:                           ; // x is negative
                NEGX
                TSTA
                BPL    _APOS_XNEG
_ANEG_XNEG:
                NEGA
_APOS_XPOS:
                DIV      ; // { same sign }
                BRA done
_XPOS:
                TSTA
                BPL    _APOS_XPOS
                NEGA
_ANEG_XPOS:
_APOS_XNEG:
                DIV      ; // { different sign -> negate result}
                NEGA
done:
                PULH
                PULX
                RTS ! {A}, {HX+A+SR}
  }
}




#pragma NO_EXIT
void _BMODS(void) {
/* DESC:    byte modulo signed z = x%y
   IN:      A x, X y
   OUT:     A z
   WRITTEN: A
   Result has sign of X
  */
  asm {
                PSHX
                PSHH
                PSHA     ; // store it for later sign check
                CLRH
                TSTX
                BPL    _XPOS
                NEGX
_XPOS:
                TSTA
                BPL    _APOS
                NEGA
_APOS:
                DIV
                PSHH
                PULA
                PULX
                TSTX
                BPL _RES_POS
                NEGA
_RES_POS:
                PULH
                PULX
                RTS ! {A}, {HX+A+SR}
  }
}


/* binary integer routines */
#if !defined(__HCS08__)
#pragma NO_EXIT
#pragma NO_RETURN
void _IMUL(int i, ...) {
/* DESC:    16 bit multiplication k = i * j
   IN:      TOS i, X:A j
   OUT:     X:A k
   WRITTEN: A,X,H */
  int j;
  asm {
                STA 2,SP
                STX 1,SP
                LDA      i:1
                LDX      j:1
                MUL
                PSHA
                PSHX
                LDA      i:1
                LDX      j:0
                MUL
                ADD      1,SP
                PSHA
                LDA      i:0
                LDX      j:1
                MUL
                ADD      1,SP
                STA      j:0
                AIS      #2
                PULA
                STA      j:1
                PULX
                PULA
                RTS  ! {A + X}, {H + SR}
  }
}
#endif

#if defined(__HCS08__)
#pragma NO_EXIT
#pragma NO_RETURN
void _IMUL_STAR08(int i, ...) {
/* DESC:    16 bit multiplication k = i * j
   IN:      TOS i, H:X j
   OUT:     H:X k
   WRITTEN: A,X,H */
  int j;
  asm {
                STHX     1,SP
                LDA      i:1
                LDX      j:1
                MUL
                PSHA
                PSHX
                LDA      i:1
                LDX      j:0
                MUL
                ADD      1,SP
                PSHA
                LDA      i:0
                LDX      j:1
                MUL
                ADD      1,SP
                STA      j:0
                AIS      #2
                PULA
                STA      j:1
                PULH
                PULX
                RTS  ! {A + X}, {H + SR}
  }
}
#endif

#if !defined(__HCS08__)
#pragma NO_EXIT
#pragma NO_RETURN
void _IDIVS(int i, ...) {
/* DESC:    16 bit signed division k = i / j
   IN:      TOS i, X:A j
   OUT:     X:A k
   WRITTEN: A,X,H */
  int j;
  asm {
                STA j:1
                STX j
                LDA     i:0
                EOR     j:0
                PSHA
                JSR     _SPLITSIGN ! {}, {HX+A+SR}
; //              JSR     _IDIVMOD  ! {}, {HX+A+SR}
                PULA
                TSTA
                BPL     _IDIVS_0     ; // done
                COM     j:0
                NEG     j:1
                BNE     _IDIVS_0
                INC     j:0
     _IDIVS_0:    ;
                PULX
                PULA
                RTS ! {A + X}, {H + SR}
  }
}
#endif

#if defined(__HCS08__)
#pragma NO_EXIT
#pragma NO_RETURN
void _IDIVS_STAR08(int i, ...) {
/* DESC:    16 bit signed division k = i / j
   IN:      TOS i, H:X j
   OUT:     H:X k
   WRITTEN: A,X,H */
  int j;
  asm {
                STHX    j
                LDA     i:0
                EOR     j:0
                PSHA
                JSR     _SPLITSIGN ! {}, {HX+A+SR}
; //              JSR     _IDIVMOD  ! {}, {HX+A+SR}
                PULA
                TSTA
                BPL     _IDIVS_0     ; // done
                COM     j:0
                NEG     j:1
                BNE     _IDIVS_0
                INC     j:0
     _IDIVS_0:    ;
                PULH
                PULX
                RTS ! {A + X}, {H + SR}
  }
}
#endif

#if !defined(__HCS08__)
#pragma NO_EXIT
#pragma NO_RETURN
void _IDIVU(int i, ...) {
/* DESC:    16 bit unsigned division k = i / j
   IN:      TOS i, X:A j
   OUT:     X:A k
   WRITTEN: A,X,H */

  int j;
  asm {
                STA j:1
                STX j
                PSHA
                JSR      _IDIVMOD  ! {}, {HX+A+SR}
                PULA
                PULX
                PULA
                RTS ! {A + X}, {H + SR}
  }
}
#endif

#if defined(__HCS08__)
#pragma NO_EXIT
#pragma NO_RETURN
void _IDIVU_STAR08(int i, ...) {
/* DESC:    16 bit unsigned division k = i / j
   IN:      TOS i, H:X j
   OUT:     H:X k
   WRITTEN: A,X,H */

  int j;
  asm {
                STHX j
                PSHA
                JSR      _IDIVMOD  ! {}, {HX+A+SR}
                PULA
                PULH
                PULX
                RTS ! {A + X}, {H + SR}
  }
}
#endif

#if !defined(__HCS08__)
#pragma NO_EXIT
#pragma NO_RETURN
void _IMODS(int i, ...) {
/* DESC:    16 bit signed modulus k = i % j
   IN:      TOS i, X:A j
   OUT:     X:A k
   WRITTEN: A,X,H */
  int j;
  asm {
                STA j:1
                STX j
                LDA     i:0
                PSHA
                JSR     _SPLITSIGN ! {}, {HX+A+SR}
; //                JSR     _IDIVMOD   ! {}, {HX+A+SR}
                PULA
                TSTA
                BPL     _IMODS_0     ; // done
                LDA     i:0
                COMA
                STA     i:0
                LDA     i:1
                NEGA
                STA     i:1
                BNE     _IMODS_0
                INC     i:0
     _IMODS_0:    ;
                AIS     #2
                TSX
                LDA      3,X
                LDX      2,X
                RTS ! {A + X}, {H + SR}
  }
}
#endif

#if defined(__HCS08__)
#pragma NO_EXIT
#pragma NO_RETURN
void _IMODS_STAR08(int i, ...) {
/* DESC:    16 bit signed modulus k = i % j
   IN:      TOS i, H:X j
   OUT:     H:X k
   WRITTEN: A,X,H */
  int j;
  asm {
                STHX    j
                LDA     i:0
                PSHA
                JSR     _SPLITSIGN ! {}, {HX+A+SR}
; //               JSR     _IDIVMOD   ! {}, {HX+A+SR}
                PULA
                TSTA
                BPL     _IMODS_0     ; // done
                LDA     i:0
                COMA
                STA     i:0
                LDA     i:1
                NEGA
                STA     i:1
                BNE     _IMODS_0
                INC     i:0
     _IMODS_0:    ;
                AIS     #2
                LDHX    3,SP
                RTS ! {A + X}, {H + SR}
  }
}
#endif

#if !defined(__HCS08__)
#pragma NO_EXIT
#pragma NO_RETURN
void _IMODU(int i, ...) {
/* DESC:    16 bit unsigned modulus k = i % j
   IN:      TOS i, X:A j
   OUT:     X:A k
   WRITTEN: A,X,H */
  int j;
  asm {
                STA j:1
                STX j
                PSHA
                JSR      _IDIVMOD ! {}, {HX+A+SR}
                PULA
                AIS      #2
                TSX
                LDA      3,X
                LDX      2,X
                RTS ! {A + X}, {H + SR}
  }
}
#endif

#if defined(__HCS08__)
#pragma NO_EXIT
#pragma NO_RETURN
void _IMODU_STAR08(int i, ...) {
/* DESC:    16 bit unsigned modulus k = i % j
   IN:      TOS i, H:X j
   OUT:     H:X k
   WRITTEN: A,X,H */
  int j;
  asm {
                STHX     j
                PSHA
                JSR      _IDIVMOD ! {}, {HX+A+SR}
                PULA
                AIS      #2
                LDHX     3,SP
                RTS ! {A + X}, {H + SR}
  }
}
#endif

#pragma NO_EXIT
#pragma NO_RETURN
void _IDIVU_8(void) {
/* DESC:    16 by 8 bit unsigned division k = i / j, unsigned int i, char j 
   IN:      H:A i, X j
   OUT:     A k
   WRITTEN: A,X,H */
  asm {
                PSHA
                PSHH
                DIV                    ; // sets carry if overflow occured
                BCC      _IDIVU_8_0
#ifdef __HCS08__
                CLRH
                JSR      _IDIVU_STAR08
                TXA
                PSHH
                PULX
#else
                TXA
                CLRX
                JSR      _IDIVU
#endif
    _IDIVU_8_0:
                AIS      #2
                RTS
  }
}

#pragma NO_EXIT
#pragma NO_RETURN
void _IMODU_8(void) {
/* DESC:    16 by 8 bit unsigned modulus k = i % j, unsigned int i, char j 
   IN:      H:A i, X j
   OUT:     A k
   WRITTEN: A,X,H */
  asm {
                PSHA
                PSHH
                DIV                    ; // sets carry if overflow occured
                BCC      _IMODU_8_0
#ifdef __HCS08__
                CLRH
                JSR      _IMODU_STAR08
                TXA
                PSHH
                PULX
#else
                TXA
                CLRX
                JSR      _IMODU
#endif
                BRA      _IMODU_8_1
    _IMODU_8_0:
                PSHH
                PULA
    _IMODU_8_1:
                AIS      #2
                RTS
  }
}

void _IASR(char j, ...) {
/* DESC:    16 bit arithmetical shift right k = i >> j
   IN:      X:A i, TOS j
   OUT:     X:A k
   WRITTEN: A,X */

  asm {
                TST j
                BEQ  _IASR_2
  _IASR_1:
                ASRX
                RORA
                DBNZ j,_IASR_1
  _IASR_2:
  }
}

void _ILSR(char j, ...) {
/* DESC:    16 bit logical shift right k = i >> j
   IN:      X:A i, TOS j
   OUT:     X:A k
   WRITTEN: A,X */

  asm {
                TST j
                BEQ  _ILSR_2
  _ILSR_1:
                LSRX
                RORA
                DBNZ j,_ILSR_1
  _ILSR_2:
  }
}

void _ILSL(char j, ...) {
/* DESC:    16 bit shift left k = i << j
   IN:      X:A i, TOS j
   OUT:     X:A k
   WRITTEN: A,X */

  asm {
                TST j
                BEQ  _ILSL_2
  _ILSL_1:
                LSLA
                ROLX
                DBNZ j,_ILSL_1
  _ILSL_2:
  }
}

#pragma NO_EXIT
void _ICMP(int j, ...) {
/* DESC:     16 bit compare i <rel> j
   IN:       X:A i, TOS j
   OUT:
   WRITTEN:  A,H,X */

  asm {
                CPX     j:0
                BNE     _ICMP_0
                CMP     j:1
                BEQ     _ICMP_0
                CLRA
                BCS     _ICMP_1
                LDA     #2
  _ICMP_1:       ;
                CMP     #1
  _ICMP_0:       ; // flags are set for i - j
                _PUL_RA
                AIS     #2                      ; // free pushed parameter
                JMP     0,X ! {HX+SR}, {HX+A}
  }
}

/* unary and binary long integer routines */

#pragma NO_EXIT
void _LINC(_PARAM_UNARY_L_INLINE) {
/* DESC:    32 bit increment by 1  k = k+1
   IN:      H:X @i
   OUT:     TOS j
   WRITTEN:  */

  _ENTER_UNARY_L();
  asm {
                TSX
                INC     k:3
                BNE     _LINC_0
                INC     k:2
                BNE     _LINC_0
                INC     k:1
                BNE     _LINC_0
                INC     k:0
  _LINC_0:      ;
                _EXIT_UNARY_L
                RTS
  }
}

#pragma NO_EXIT
void _LDEC(_PARAM_UNARY_L_INLINE) {
/* DESC:    32 bit decrement by 1  k = k-1
   IN:      H:X @k
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_UNARY_L();
  asm {
                TSX
                TST   k:3
                BNE   _LDEC_0
                TST   k:2
                BNE   _LDEC_1
                TST   k:1
                BNE   _LDEC_2
                DEC   k:0
  _LDEC_2:      DEC   k:1
  _LDEC_1:      DEC   k:2
  _LDEC_0:      DEC   k:3
                _EXIT_UNARY_L
                RTS
  }
}

#pragma NO_EXIT
void _LNEG(_PARAM_UNARY_L_INLINE) {
/* DESC:    32 bit negation by 1  k = -k
   IN:      H:X @k
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_UNARY_L();
  asm {
                COM      k:0
                COM      k:1
                COM      k:2
                NEG      k:3
                BNE      _LNEG_0
                INC      k:2
                BNE      _LNEG_0
                INC      k:1
                BNE      _LNEG_0
                INC      k:0
    _LNEG_0:    ;
                _EXIT_UNARY_L
                RTS
  }
}

#pragma NO_EXIT
void _LNOT(_PARAM_UNARY_L_INLINE) {
/* DESC:    32 bit complement by 1  k = ~k
   IN:      H:X @k
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_UNARY_L();
  asm {
                COM      k:0
                COM      k:1
                COM      k:2
                COM      k:3
                _EXIT_UNARY_L
                RTS
  }
}

#pragma NO_EXIT
void _LADD(void) {
/* DESC:    32 bit addition k = i + j
   IN:      H:X @i, TOS @j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_BINARY_L();
  asm JSR _LADD_k_is_k_plus_j;
}

#pragma NO_EXIT
void _LADD_RC(void) {
/* DESC:    32 bit addition k = i + j (j constant)
   IN:      H:X @i, 0,RA j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_BINARY_L_RC();
  asm _SRET;
  asm JSR _LADD_k_is_k_plus_j;
}

#pragma NO_EXIT
void _LSUB(void) {
/* DESC:    32 bit subtraction k = i - j
   IN:      H:X @i, TOS @j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_BINARY_L();
  asm JSR _LSUB_k_is_k_minus_j;
}

#pragma NO_EXIT
void _LSUB_LC(void) {
/* DESC:    32 bit subtraction k = i - j (i constant)
   IN:      0,RA i, H:X @j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_BINARY_L_LC();
  asm _SRET;
  asm JSR _LSUB_k_is_k_minus_j;
}

#pragma NO_EXIT
void _LSUB_RC(void) {
/* DESC:    32 bit subtraction k = i - j (j constant)
   IN:      H:X @i, 0,RA j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_BINARY_L_RC();
  asm _SRET;
  asm JSR _LSUB_k_is_k_minus_j;
}

#pragma NO_EXIT
void _LAND(void) {
/* DESC:    32 bit binary and k = i & j
   IN:      H:X @i, TOS @j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_BINARY_L();
  asm JSR _LAND_k_is_k_and_j;
}

#pragma NO_EXIT
void _LAND_RC(void) {
/* DESC:    32 bit binary and k = i & j (j constant)
   IN:      H:X @i, 0,RA @j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_BINARY_L_RC();
  asm _SRET;
  asm JSR _LAND_k_is_k_and_j;
}

#pragma NO_EXIT
void _LOR(void) {
/* DESC:    32 bit binary or k = i | j
   IN:      H:X @i, TOS @j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_BINARY_L();
  asm JSR _LOR_k_is_k_or_j;
}

#pragma NO_EXIT
void _LOR_RC(void) {
/* DESC:    32 bit binary or k = i | j (j constant)
   IN:      H:X @i, 0,RA @j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_BINARY_L_RC();
  asm _SRET;
  asm JSR _LOR_k_is_k_or_j;
}

#pragma NO_EXIT
void _LXOR(void) {
/* DESC:    32 bit binary xor k = i ^ j
   IN:      H:X @i, TOS @j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_BINARY_L();
  asm JSR _LXOR_k_is_k_xor_j;
}

#pragma NO_EXIT
void _LXOR_RC(void) {
/* DESC:    32 bit binary xor k = i ^ j (j constant)
   IN:      H:X @i, 0,RA @j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_BINARY_L_RC();
  asm _SRET;
  asm JSR _LXOR_k_is_k_xor_j;
}

#pragma NO_EXIT
void _LMUL(void) {
/* DESC:    32 bit multiplication k = i * j
   IN:      H:X @i, TOS @j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_BINARY_L();
  asm JSR _LMUL_k_is_k_mul_j;
}

#pragma NO_EXIT
void _LMUL_RC(void) {
/* DESC:    32 bit multiplication k = i * j (j constant)
   IN:      H:X @i, 0,RA @j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_BINARY_L_RC();
  asm _SRET;
  asm JSR _LMUL_k_is_k_mul_j;
}

#pragma NO_EXIT
void _LDIVS(void) {
/* DESC:    32 bit signed division k = i / j
   IN:      H:X @i, TOS @j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_BINARY_L();
  asm JSR _LDIVS_k_is_k_div_j;
}

#pragma NO_EXIT
void _LDIVS_LC(void) {
/* DESC:    32 bit signed division k = i / j (i constant)
   IN:      0,RA i, H:X @j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_BINARY_L_LC();
  asm _SRET;
  asm JSR _LDIVS_k_is_k_div_j;
}

#pragma NO_EXIT
void _LDIVS_RC(void) {
/* DESC:    32 bit signed division k = i / j (j constant)
   IN:      H:X @i, 0,RA j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_BINARY_L_RC();
  asm _SRET;
  asm JSR _LDIVS_k_is_k_div_j;
}

#pragma NO_EXIT
void _LDIVU(void) {
/* DESC:    32 bit unsigned division k = i / j
   IN:      H:X @i, TOS @j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_BINARY_L();
  asm JSR _LDIVU_k_is_k_div_j;
}

#pragma NO_EXIT
void _LDIVU_LC(void) {
/* DESC:    32 bit unsigned division k = i / j (i constant)
   IN:      0,RA i, H:X @j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_BINARY_L_LC();
  asm _SRET;
  asm JSR _LDIVU_k_is_k_div_j;
}

#pragma NO_EXIT
void _LDIVU_RC(void) {
/* DESC:    32 bit unsigned division k = i / j (j constant)
   IN:      H:X @i, 0,RA j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_BINARY_L_RC();
  asm _SRET;
  asm JSR _LDIVU_k_is_k_div_j;
}

#pragma NO_EXIT
void _LMODS(void) {
/* DESC:    32 bit signed modulus k = i % j
   IN:      H:X @i, TOS @j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_BINARY_L();
  asm JSR _LMODS_k_is_k_mod_j;
}

#pragma NO_EXIT
void _LMODS_LC(void) {
/* DESC:    32 bit signed modulus k = i % j (i constant)
   IN:      0,RA i, H:X @j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_BINARY_L_LC();
  asm _SRET;
  asm JSR _LMODS_k_is_k_mod_j;
}

#pragma NO_EXIT
void _LMODS_RC(void) {
/* DESC:    32 bit signed modulus k = i % j (j constant)
   IN:      H:X @i, 0,RA j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_BINARY_L_RC();
  asm _SRET;
  asm JSR _LMODS_k_is_k_mod_j;
}

#pragma NO_EXIT
void _LMODU(void) {
/* DESC:    32 bit unsigned modulus k = i % j
   IN:      H:X @i, TOS @j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_BINARY_L();
  asm JSR _LMODU_k_is_k_mod_j;
}

#pragma NO_EXIT
void _LMODU_LC(void) {
/* DESC:    32 bit unsigned modulus k = i % j (i constant)
   IN:      0,RA i, H:X @j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_BINARY_L_LC();
  asm _SRET;
  asm JSR _LMODU_k_is_k_mod_j;
}

#pragma NO_EXIT
void _LMODU_RC(void) {
/* DESC:    32 bit unsigned modulus k = i % j (j constant)
   IN:      H:X @i, 0,RA j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_BINARY_L_RC();
  asm _SRET;
  asm JSR _LMODU_k_is_k_mod_j;
}

#pragma NO_EXIT
void _LASR(_PARAM_UNARY_L_INLINE) {
/* DESC:    32 bit arithmetical shift right k = i >> j (j > 0)
   IN:      H:X @i, A j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_UNARY_L();
  asm {
                LDA   saved_A
                BEQ   _LASR_1
  _LASR_0:      ;
                ASR   k:0
                ROR   k:1
                ROR   k:2
                ROR   k:3
                DBNZA _LASR_0
  _LASR_1:
                _EXIT_UNARY_L
                RTS
  }
}

#pragma NO_EXIT
void _LLSR(_PARAM_UNARY_L_INLINE) {
/* DESC:    32 bit logical shift right k = i >> j (j > 0)
   IN:      H:X @i, A j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_UNARY_L();
  asm {
                LDA   saved_A
                BEQ   _LLSR_1
  _LLSR_0:      ;
                LSR   k:0
                ROR   k:1
                ROR   k:2
                ROR   k:3
                DBNZA _LLSR_0
  _LLSR_1:
                _EXIT_UNARY_L
                RTS
  }
}

#pragma NO_EXIT
void _LLSL(_PARAM_UNARY_L_INLINE) {
/* DESC:    32 bit logical shift left k = i << j (j > 0)
   IN:      H:X @i, A j
   OUT:     TOS k
   WRITTEN:  */

  _ENTER_UNARY_L();
  asm {
                LDA   saved_A
                BEQ   _LLSL_1
  _LLSL_0:      ;
                LSL   k:3
                ROL   k:2
                ROL   k:1
                ROL   k:0
                DBNZA _LLSL_0
  _LLSL_1:
                _EXIT_UNARY_L
                RTS
  }
}

#pragma NO_EXIT
void _LCMP(void) {
/* DESC:    32 bit compare CC = i rel j
   IN:      TOS @i, H:X @j
   OUT:     CCR condition code
   WRITTEN: X,H */

  _ENTER_BINARY_L();
  asm JSR _LCMP_k_rel_j;
}

#pragma NO_EXIT
void _LCMP_RC(void) {
/* DESC:    32 bit compare CC = i rel j (j constant)
   IN:      H:X @i, 0,RA j
   OUT:     CCR condition code
   WRITTEN: X,H */

  _ENTER_BINARY_L_RC();
  asm _SRET;
  asm JSR _LCMP_k_rel_j;
}

#if defined(__HCS08__)
/*
 _CMP24 and _CMP24_RC are only used when compiling with -MMU.
*/
#pragma NO_EXIT
void _CMP24(void) {
/* DESC:    24 bit compare CC = i rel j
   IN:      TOS @i, H:X @j
   OUT:     CCR condition code
   WRITTEN: X,H */

  _ENTER_BINARY_L();
  asm JSR _CMP24_k_rel_j;
}

#pragma NO_EXIT
void _CMP24_RC(void) {
/* DESC:    24 bit compare CC = i rel j (j constant)
   IN:      H:X @i, 0,RA j
   OUT:     CCR condition code
   WRITTEN: X,H */

  _ENTER_BINARY_L_RC();
  asm _SRET;
  asm JSR _CMP24_k_rel_j;
}
#endif


    /* copy routines */
#pragma NO_EXIT
void _COPY(void) {
/* DESC:    copy in 16 bit address space (stack incl.)
   IN:      TOS @dest, H:X @src, A count
   OUT:
   WRITTEN: X,H */

   asm {
            PSHA                          ; // save nof bytes to copy
            LDA    #_RTS
            PSHA
            LDA    #-10
            PSHA
            LDA    #_BNE
            PSHA
            LDA    #_TSTX
            PSHA
            LDA    9,SP                   ; // dest addr
            PSHA
            LDA    9,SP
            PSHA
            LDA    #_STA_HX
            PSHA
            PSHX                          ; // src addr
            PSHH
            LDHX    #_LDA_HX+(_DECX*0x100)
            PSHX
            PSHH
            LDHX    #_TAX+(_CLRH*0x100)
            PSHX
            PSHH
            TSX
            LDA    13,X                   ; // restore nof bytes to copy

            JSR    ,X                     ; // call pushed copy loop on TOS

            AIS    #13
            PULA
            _PUL_RA                       ; // pull return address
            AIS    #2                     ; // free parameter (src@)
            JMP    ,X
   }
}

#pragma NO_EXIT
void _COPY_L(void) {
/* DESC:    copy very large structures (>= 256 bytes) in 16 bit address space (stack incl.)
   IN:      TOS count, TOS(2) @dest, H:X @src
   OUT:
   WRITTEN: X,H */

   asm {
            PSHA
            LDA    #_RTS
            PSHA
            LDA    #-13
            PSHA
            LDA    #_BNE
            PSHA
            CLRA
            PSHA
            PSHA
            LDA    #_CPHX                 ; // CPHX #0
            PSHA
            LDA    13,SP                  ; // dest addr
            PSHA
            LDA    13,SP
            PSHA
            LDA    #_STA_HX
            PSHA
            PSHX                          ; // src addr
            PSHH
            LDHX   #_LDA_HX+0xff00
            PSHX
            PSHH
            LDA    #_AIX                  ; // AIX #-1
            PSHA
            TSX
            LDA    18,X
            PSHA
            LDA    17,X
            PSHA
            LDA    #_LDHX
            PSHA

            TSX
            JSR 0,X                       ; // call pushed copy loop on TOS

            AIS #17
            PULA
            _PUL_RA                       ; // pull return address
            AIS #4                        ; // free parameter (src@)
            JMP 0,X
   }
}

#pragma NO_EXIT
void _POP32(void) {
/* DESC:    pop 32 bits from TOS to [(H:X)]
   IN:      H:X @dest, TOS src
   OUT:
   WRITTEN: A */

   asm {
             PSHA
             LDA  4,SP
             STA  0,X
             LDA  5,SP
             STA  1,X
             LDA  6,SP
             STA  2,X
             LDA  7,SP
             STA  3,X
             PULA
             _PUL_RA
             AIS  #4
             JMP  ,X
   }
}

#pragma NO_EXIT
void _POP64(void) {
/* DESC:     pop 64 bits from TOS to [(H:X)]
   IN:       H:X @dest, TOS src
   OUT:
   WRITTEN:  A */

   asm {
             PSHA
             LDA  4,SP
             STA  0,X
             LDA  5,SP
             STA  1,X
             LDA  6,SP
             STA  2,X
             LDA  7,SP
             STA  3,X
             LDA  8,SP
             STA  4,X
             LDA  9,SP
             STA  5,X
             LDA  10,SP
             STA  6,X
             LDA  11,SP
             STA  7,X
             PULA
             _PUL_RA
             AIS  #8
             JMP  ,X
   }
}

void _STORE32(void) {
/* DESC:     store 32 bits from TOS to [(H:X)]
   IN:       H:X @dest, TOS src
   OUT:
   WRITTEN:  */

   asm {
             PSHA
             LDA  4,SP
             STA  0,X
             LDA  5,SP
             STA  1,X
             LDA  6,SP
             STA  2,X
             LDA  7,SP
             STA  3,X
             PULA
  }
}

void _STORE64(void) {
/* DESC:     store 64 bits from TOS to [(H:X)]
   IN:       H:X @dest, TOS src
   OUT:
   WRITTEN:  */

   asm {
             PSHA
             LDA  4,SP
             STA  0,X
             LDA  5,SP
             STA  1,X
             LDA  6,SP
             STA  2,X
             LDA  7,SP
             STA  3,X
             LDA  8,SP
             STA  4,X
             LDA  9,SP
             STA  5,X
             LDA  10,SP
             STA  6,X
             LDA  11,SP
             STA  7,X
             PULA
  }
}

    /* sign extentions */
#pragma NO_EXIT
void _SEXT8_32(void) {
/* DESC:    8 to 32 bit sign extention k = sext(i)
   IN:      A i
   OUT:     TOS k
   WRITTEN: X,H */

  asm {
                _PUL_RA
                PSHA
                CMP     #0x80
                CLRA
                BCS     _SEXT8_32_0
                COMA
    _SEXT8_32_0:    ;
                PSHA
                PSHA
                PSHA
                LDA     4,SP
                JMP     0,X
  }
}

#pragma NO_EXIT
void _SEXT16_32(void) {
/* DESC:    16 to 32 bit sign extention k = sext(i)
   IN:      X i.hi TOS i.lo
   OUT:     TOS k
   WRITTEN: A,X,H */

  asm {
                TXA
                _PUL_RA
                PSHA
                CMP     #0x80
                CLRA
                BCS     _SEXT16_32_0
                COMA
    _SEXT16_32_0:    ;
                PSHA
                PSHA
                JMP     0,X
  }
}

    /* function call utilities */
#if !defined(__HCS08__)
#pragma NO_EXIT
void _CALL(void) {
/* DESC:    calls a function (dest as pointer) with 2 register parameters (complex due to lack of registers!!!)
   IN:      X:A function pointer, TOS 2 parameters
   OUT:
   WRITTEN: A,X,H */

   asm {
     PSHA
     PSHX
     TSX
     LDA  5,X    ; // push pushed register params
     PSHA
     LDA  4,X
     PSHA
     LDA  3,X    ; // move ret@:lo
     STA  5,X
     LDA  2,X    ; // move ret@:hi
     STA  4,X
     LDA  1,X    ; // move function pointer:lo
     STA  3,X
     LDA  0,X    ; // move function pointer:hi
     STA  2,X
     PULX        ; // pull register params to their register
     PULA
     AIS  #2     ; // free resulting gap. now the function pointer is TOS and the RTS calls the function.
     RTS  ! {A+X} , {H+SR}
   }
}
#endif

#if defined(__HCS08__)
#pragma NO_EXIT
void _CALL_STAR08(void) {
/* DESC:    calls a function (dest as pointer) with 2 register parameters (complex due to lack of registers!!!)
   IN:      H:X function pointer, TOS parameter assigned to X or H:X
   OUT:
   WRITTEN: A,X,H */

   asm {
     PSHX        ; // push function pointer
     PSHH
     PSHA        ; // push non pushed register param
     TSX
     LDA  6,X    ; // push pushed register params
     PSHA
     LDA  5,X
     PSHA
     LDA  4,X    ; // move ret@:lo
     STA  6,X
     LDA  3,X    ; // move ret@:hi
     STA  5,X
     LDA  2,X    ; // move function pointer:lo
     STA  4,X
     LDA  1,X    ; // move function pointer:hi
     STA  3,X
     PULH        ; // pull register params to their register
     PULX
     PULA
     AIS  #2     ; // free resulting gap. now the function pointer is TOS and the RTS calls the function.
     RTS  ! {A+X} , {H+SR}
   }
}
#endif

#if defined(__HCS08__)
#pragma NO_EXIT
/* even if it might be __BANKED__ general, this function is HCS08/MMU specific since it uses the parameter passing policy of the STAR08 */
void _CALL_STAR08_FAR(void) {
/* DESC:    calls a far function
   IN:      A:H:X function pointer (PAGE:HI:LO)
   OUT:
   WRITTEN: A,X,H */

   asm {
;    // push function pointer 3 bytes (LO:HI:PAGE)
     PSHX        ; // push function pointer:LO
     PSHH        ; // push function pointer:HI
     PSHA        ; // push function pointer:PAGE
     TSX

;    // push pushed register params (all three of them A, X, H ... pushed in this order in the caller)
     LDA  8,X
     PSHA
     LDA  7,X
     PSHA
     LDA  6,X
     PSHA
     
;    // move the caller return address (LO:HI:PAGE) in the needed position on the stack
     LDA  5,X    ; // move ret@:LO
     STA  8,X
     LDA  4,X    ; // move ret@:HI
     STA  7,X
     LDA  3,X    ; // move ret@:PAGE
     STA  6,X                 

;    // move the called function pointer address (LO:HI:PAGE), on the stack, in order to perform call through RTC
     LDA  2,X    ; // move function pointer:LO
     STA  5,X
     LDA  1,X    ; // move function pointer:HI
     STA  4,X
     LDA  ,X     ; // move function pointer:PAGE
     STA  3,X
                 
;    // pull register params to their register (in the reversed order they where previously pushed: H, X, A)
     PULA
     PULH
     PULX
     
;    // free resulting gap
     AIS  #3
     
;    // perform function call through RTC (while having on the stack the function pointer address (LO:HI:PAGE))
     DCB 0x8D    ; //RTC
   }
}
#endif


/******************* SWITCH TABLES HANDLING ***********************************/

#pragma NO_EXIT
void _Jump_Table_Addr(void) {
/* DESC:      Configure H:X with the jump address of the matching case statement
   IN:        A:X contains offset in jump table    TOS contains the return address(jump table address)
   OUT:       H:X contains the address of the matching case statement.
   WRITTEN:   A H:X SP */
/* ...
   JSR _Jump_Table_Addr
   2 byte: Addr Label0
   2 byte: Addr Label1
   ...
   2 byte: Addr Labeln

   All Addresses are absolute 16 bit.
*/
   asm {
                ASLX
                ROLA
                PSHA
                TXA
                ADD  3,SP  ; // add low(index) + low(retadr)
                TAX
                PULA
                ADC 1,SP   ; // add high(index) + high(retadr) + carry
                PSHA
                PULH
                AIS   #2    ; // Release stack
                LDA   ,X    ; // Load address of matching case statement
                LDX   1,X
                PSHA
                PULH
                _SRET
                JMP   ,X
   }
}


#pragma NO_EXIT
void _Jump_Table_Offset(void) {
/* DESC:      Configure H:X with the jump address of the matching case statement
   IN:        A:X contains offset in jump table    TOS contains the return address(jump table address)
   OUT:       H:X contains the address of the matching case statement.
   WRITTEN:   A H:X SP */
/* ...
   JSR _Jump_Table_Offset
   1 byte: Offset Label0
   1 byte: Offset Label1
   ...
   1 byte: Offset Labeln

   All Addresses are absolute 8 bit relative from the next address.
     Example : 0x1234 DCB 0x12 points to address 0x1234+0x12+1 == 0x1247.

*/
   asm {
                  PSHA                ; // save high of index
                  TXA
                  ADD   3,SP          ; // add low of return address to low of index
                  TAX
                  PULA                ; // restore high of index
                  ADC   1,SP          ; // add high of return address to high of index
                  PSHA                ; // move high to h register
                  PULH
                  AIS  #2             ; // free stack from return address
                  TXA
                  ADD   0,X           ; // A <- (H:X) 
                  TAX
                  PSHH                ; // H <- H + (1,SP)
                  PULA
                  ADC   #0
                  PSHA
                  PULH
                  _SRET
                  JMP  1,X            ; // 1: 1 for sizeof (offset)
   }
}

#pragma NO_EXIT
void _Jump_Table_Header_Addr(void) {
/* DESC:      Configure H:X with the jump address of the matching case statement
   IN:        A:X contains offset in jump table    TOS contains the return address(jump table address)
   OUT:       H:X contains the address of the matching case statement.
   WRITTEN:   A H:X SP */
/* ...
   JSR _Jump_Table_Header_Addr
   2 byte: N
   2 byte: Addr Default
   2 byte: Addr Label0
   2 byte: Addr Label1
   ...
   2 byte: Addr Labeln

   All Addresses are absolute 16 bit.

*/
   asm {
                  PULH                ; // load high of return address
                  PSHX                ; // save offset in jump table on TOS
                  PSHA
                  LDX   3,SP          ; // load low of return address
                  CMP   ,X            ; // compare high part

                  BHI   defaul
                  BNE   normalCase

                  LDA   1,X           ; // load low part of N
                  CMP   2,SP          ; // compare low part
                  BLS   defaul

   normalCase:    AIX   #2            ; // skip (N & Default) entries

                  ASL   2,SP          ; // offset scaling
                  ROL   1,SP

                  TXA                 ; // X <- X + (0,SP)
                  ADD   2,SP
                  TAX

                  PSHH                ; // H <- H + (1,SP)
                  PULA
                  ADC   1,SP
                  PSHA
                  PULH
   defaul:
                  AIS   #3            ; // Release the stack
                  LDA   2,X           ; // Load address of matching case statement
                  LDX   3,X
                  PSHA
                  PULH
                  _SRET
                  JMP   ,X
   }
}

#pragma NO_EXIT
void _Jump_Table_Header_Offset(void) {
/* DESC:      Configure H:X with the jump address of the matching case statement
   IN:        A:X contains offset in jump table    TOS contains the return address(jump table address)
   OUT:       H:X contains the address of the matching case statement.
   WRITTEN:   A H:X SP */
/* ...
   JSR _Jump_Table_Header_Offset
   1 byte: N
   1 byte: Offset Default
   1 byte: Offset Label0
   1 byte: Offset Label1
   ...
   1 byte: Offset Labeln


   All Addresses are absolute 8 bit relative from the next address.
     Example : 0x1234 DCB 0x12 points to address 0x1234+0x12+1 == 0x1247.

*/
   asm {
                  TSTA
                  TXA
                  PULH                ; // load return address
                  PULX
                  BNE   defaul        ; // if high(A:X) != 0, use default
                  CMP   0,X           ; // Compare low part
                  BHS   defaul        ; // if it contains less entries than table, goto nondefaul
                  AIX   #1            ; // skip default
                  PSHX
                  ADD   1,SP
                  PSHA
                  PSHH
                  PULA
                  ADC   #0
                  PSHA
                  PULH
                  PULX
                  PULA                ; // free stack
defaul:
                  TXA
                  ADD   1,X           ; // A <- (H:X) 
                  TAX
                  PSHH                ; // H <- H + (1,SP)
                  PULA
                  ADC   #0
                  PSHA
                  PULH
                  _SRET
                  JMP 2,X             ; // 2: 1 for number of labels, 1 sizeof (offset)
   }
}

#pragma NO_EXIT
void _Search_Table_16_Addr(void) {
/* DESC:      Configure H:X with the jump address of the matching case statement
   IN:        A:X contains offset in jump table    TOS contains the return address(jump table address)
   OUT:       H:X contains the address of the matching case statement.
   WRITTEN:   A H:X SP */
/* ...
   JSR _Search_Table_16_Addr
   2 byte: Max (== Valuen)
   2 byte: Addr Default
   2 byte: Value0
   2 byte: Addr Label0
   2 byte: Value1
   2 byte: Addr Label1
   ...
   2 byte: Valuen
   2 byte: Addr Labeln

   All Values are sorted unsigned.
   All Addresses are absolute 16 bit.
*/
   asm {
                  PULH
                  PSHH
                  PSHX                ; // Save offset in jump table on TOS
                  PSHA

                  LDX  4,SP

                  CMP   ,X            ; // Compare high part
                  BHI   found
                  BNE   search

   checkLowPartN: LDA   1,X           ; // Load low part of N
                  CMP   2,SP          ; // Compare low part
                  BCS   found

   search:        AIX   #4            ; // Skip (N & Default) entries
                  LDA   ,X            ; // Compare high part of Valuex
                  CMP   1,SP
                  BCS   search
                  BNE   reload

                  LDA   1,X           ; // Load low part of valuex
                  CMP   2,SP          ; // Compare low part of valuex
                  BCS   search
                  BEQ   found

reload:           AIS   #2            ; // Reload jump table address
                  PULH
                  PULX
                  BRA   end

   found:         AIS   #4            ; // free stack 
   end:
                  LDA   2,X           ; // Load address of matching case statement
                  LDX   3,X
                  PSHA
                  PULH
                  _SRET
                  JMP   ,X


   }
}

#pragma NO_EXIT
void _Search_Table_16_Offset(void) {
/* DESC:      Configure H:X with the jump address of the matching case statement
   IN:        A:X contains offset in jump table    TOS contains the return address(jump table address)
   OUT:       H:X contains the address of the matching case statement.
   WRITTEN:   A H:X SP */
/* ...
   JSR _Search_Table_16_Offset
   2 byte: Max (== Valuen)
   1 byte: Offset Default
   2 byte: Value0
   1 byte: Offset Label0
   2 byte: Value1
   1 byte: Offset Label1
   ...
   2 byte: Valuen
   1 byte: Offset Labeln

   All Values are sorted unsigned.
   All Addresses are absolute 8 bit relative from the next address.
     Example : 0x1234 DCB 0x12 points to address 0x1234+0x12+1 == 0x1247.
*/
   asm {
                  PULH
                  PSHH
                  PSHX                ; // Save offset in jump table on TOS
                  PSHA
                  LDX  4,SP

                  CMP   ,X            ; // Compare high part
                  BHI   found
                  BNE   search

                  LDA   1,X           ; // Load low part of N
                  CMP   2,SP          ; // Compare low part
                  BCS   found

   search:        AIX   #3            ; // Skip (N & Default) entries

                  LDA   ,X            ; // Compare high part of Valuex
                  CMP   1,SP
                  BCS   search
                  BNE   reload

                  LDA   1,X           ; // Load low part of valuex
                  CMP   2,SP          ; // Compare low part of valuex
                  BCS   search
                  BEQ   found

reload:           AIS   #2            ; // Reload jump table address
                  PULH
                  PULX
                  BRA   end

   found:         AIS   #4            ; // Update SP pointer
   end:
                  TXA
                  ADD   2,X           ; // H:X <- H:X + (offset)
                  TAX
                  PSHH
                  PULA
                  ADC   #0
                  PSHA
                  PULH
                  _SRET
                  JMP   3,X           ; //  3: 2 : from (2,X), 1 for sizeof (offset)


   }
}

#pragma NO_EXIT
void _Search_Table_8_Addr(void) {
/* DESC:      Configure H:X with the jump address of the matching case statement
   IN:        A:X contains offset in jump table    TOS contains the return address(jump table address)
   OUT:       H:X contains the address of the matching case statement.
   WRITTEN:   A H:X SP */
/* ...
   JSR _Search_Table_8_Addr
   1 byte: Max (== Valuen)
   2 byte: Addr Default
   1 byte: Value0
   2 byte: Addr Label0
   1 byte: Value1
   2 byte: Addr Label1
   ...
   1 byte: Valuen
   2 byte: Addr Labeln

   All Values are sorted unsigned.
   All Addresses are absolute 16 bit.
*/
   asm {
                  PULH
                  PSHH
                  PSHX                ; // load low part of valuex on TOS
                  LDX   3,SP
                  TSTA                ; // If high part of valuex != 0
                  PULA                ; // load low part of valuex into A
                  BNE   found         ; // use default
                  CMP   0,X           ; // > max?
                  BHI   found         ; // if >max, branch to default

   search:        AIX   #3            ; // Skip (N & Default) entries
                  CMP   ,X            ; // Compare low part of valuex
                  BHI   search
                  BEQ   found

                  PULH                ; // use default
                  PULX                ; // Reload jump table address
                  BRA   end           ; // this may be optimized to a SKIP2 !
   found:
                  AIS   #2            ; // Update SP pointer
   end:
                  LDA   1,X           ; // Load address of matching case statement
                  LDX   2,X
                  PSHA
                  PULH
                  _SRET
                  JMP   ,X
   }
}

#pragma NO_EXIT
void _Search_Table_8_Offset(void) {
/* DESC:      Configure H:X with the jump address of the matching case statement
   IN:        A:X contains offset in jump table    TOS contains the return address(jump table address)
   OUT:       H:X contains the address of the matching case statement.
   WRITTEN:   A H:X SP */
/* ...
   JSR _Search_Table_8_Offset
   1 byte: Max (== Valuen)
   1 byte: Offset Default
   1 byte: Value0
   1 byte: Offset Label0
   1 byte: Value1
   1 byte: Offset Label1
   ...
   1 byte: Valuen
   1 byte: Offset Labeln

   All Values are sorted unsigned.
   All Addresses are absolute 8 bit relative from the next address.
     Example : 0x1234 DCB 0x12 points to address 0x1234+0x12+1 == 0x1247

*/
   asm {
                  PULH
                  PSHH
                  PSHX                ; // Save offset in jump table on TOS
                  LDX  3,SP
                  TSTA                ; // Compare high part
                  PULA
                  BNE   found         ; // use default if != 0
                  CMP   0,X           ; //  > max?
                  BHI   found         ; // if >max, branch to default
   search:
                  AIX   #2            ; // Skip (N & Default) entries
                  CMP   ,X            ; // Compare low part of valuex
                  BHI   search
                  BEQ   found
                  PULH
                  PULX
                  BRA   end           ; // this may be optimized to a SKIP2 !
   found:
                  AIS   #2            ; // Update SP pointer
   end:
                  TXA
                  ADD   1,X           ; // Load offset of matching case statement
                  TAX
                  PSHH
                  PULA
                  ADC   #0
                  PSHA
                  PULH
                  _SRET
                  JMP   2,X           ; //  2 : 1 (1,X), 1 : sizeof(value) 
   }
}

    /* miscellaneous */

#pragma NO_EXIT
void _PUSH_CC(void) {
/* DESC:    push condition code without affecting any other register
   IN:      CC
   OUT:
   WRITTEN:  */

   asm {
                PSHA                ; // save A
                TPA                 ; // save CC on stack
                PSHA
                LDA    4,SP         ; // push return address
                PSHA
                LDA    4,SP
                PSHA
                LDA    4,SP         ; // reload A
                RTS ! {H+X+A}, {SR}
   }
}

#pragma NO_EXIT
void _POP_CC(void) {
/* DESC:    pop condition code without affecting any other register
   IN:
   OUT:     CC
   WRITTEN:  */

   asm {
                STA    4,SP        ; // save A
                PULA               ; // move return address to the place of '_PUSH_CC'
                STA    4,SP
                PULA
                STA    4,SP
                PULA               ; // restore CC
                TAP
                PULA               ; // restore A
                RTS ! {H+X+A+SR}, {}
   }
}

#pragma NO_EXIT
void _CONV_FAR_TO_NEAR(void) {
/* DESC:    converts a FAR (24-bit) pointer to a NEAR (16-bit) pointer
            (near_address = ((far_offset - 0x8000) + (far_page * 0x4000))) & 0xFFFF
   IN:      A (page), H(offset:0), X (offset:8)
   OUT:     H:X
   WRITTEN:
   Note:    in the comments bellow there will be used the notation in.* to refer to
            the value of the corresponding * register at the begining of the function,
            no matter its location at that moment */

   asm {
                /* save the A:H:X (all the modifications to H:X will be done directly to the stack locations they are placed in) */
                PSHA
                PSHH
                PSHX
                TSX
                
                /* the conversion is valid only if the address is in the Paging Window (i.e. 0x80 <= in.H < 0xC0) */
                /* if the address is NOT in the Paging Window, the exact in.H:X address will be returned, with NO modification */
                LDA    #128        ; // put 0x80 in A
                CMP    1,X         ; // check if 0x80 > in.H (i.e. NOT(0x80 <= in.H))
                BHI    CONV_RETURN
                LDA    #192        ; // put 0xC0 in A
                CMP    1,X         ; // check if 0xC0 <= in.H (i.e. NOT(in.H < 0xC0))
                BLS    CONV_RETURN
                
                /* compute (far_offset - 0x8000) */
                LDA    1,X         ; // put in.H in A
                SUB    #128        ; // subtract 0x80 from in.H (in order to have H:X <- in.H:X - 0x8000)
                STA    1,X

                /* the conversion is valid only if the PAGE (i.e. in.A) is either 0, 1, 2 or 3 */
                LDA    2,X
                CMP    #0
                BEQ    CONV_0
                CMP    #1
                BEQ    CONV_1
                CMP    #2
                BEQ    CONV_2
                CMP    #3
                BEQ    CONV_3
                
                /* the conversion is not valid for this PAGE ... just return H:X <- far_offset - 0x8000 */
                BRA    CONV_RETURN

                /* place in A the right value to be added to H (in order to have H:X <- H:X + 0x4000 * far_page) */
   CONV_0:
                LDA    #0          ; // put 0x00 in A
                BRA    CONV_RESUME
   CONV_1:
                LDA    #64         ; // put 0x40 in A
                BRA    CONV_RESUME   
   CONV_2:
                LDA    #128        ; // put 0x80 in A
                BRA    CONV_RESUME
   CONV_3:
                LDA    #192        ; // put 0xC0 in A
   
   CONV_RESUME:
                ADD    1,X
                STA    1,X
                
   CONV_RETURN:
                /* put in registers (H:X) the right computed values and restore the stack */
                PULX               ; // RETURN_VALUE:0
                PULH               ; // RETURN_VALUE:8
                PULA               ; // dummy PULL (just to restore the stack)
   
                RTS
   }
}

#pragma NO_EXIT
void _CONV_FAR_TO_LINEAR(void) {
/* DESC:    converts a FAR (24-bit) pointer (PAGED address) to a LINEAR (24-bit) pointer (LINEAR address)
            (linear_address = (far_offset - 0x8000) + (far_page * 0x4000)) & 0xFFFFFF
   IN:      A (page), H(offset:0), X (offset:8)
   OUT:     A:H:X
   WRITTEN:
   Note:    in the comments bellow there will be used the notation in.* to refer to
            the value of the corresponding * register at the begining of the function,
            no matter its location in that moment */

   asm {
                /* save the A:H:X (all the modifications to H:X will be done directly to the stack locations they are placed in) */
                PSHA
                PSHH
                PSHX
                TSX
                
                /* the conversion is valid only if the address is in the Paging Window (i.e. 0x80 <= in.H < 0xC0) */
                /* if the address is NOT in the Paging Window, the exact in.H:X address will be returned, with NO modification */
                LDA    #128        ; // put 0x80 in A
                CMP    1,X         ; // check if 0x80 > in.H (i.e. NOT(0x80 <= in.H))
                BHI    CONV_RETURN
                LDA    #192        ; // put 0xC0 in A
                CMP    1,X         ; // check if 0xC0 <= in.H (i.e. NOT(in.H < 0xC0))
                BLS    CONV_RETURN
                
                /* compute (far_offset - 0x8000) */
                LDA    1,X         ; // put in.H in A
                SUB    #128        ; // subtract 0x80 from in.H (in order to have H:X <- in.H:X - 0x8000)
                STA    1,X

                /* compute (far_page * 0x4000) */
                LDA    2,X          ; // put in.A (far_page) in A
                LDX    #64          ; // put 0x40 in X
                MUL
                
                /* compute (linear_address) */
                ADD    2,SP         ; // add LSB(far_page * 0x40) to in.H - 0x80 (i.e. out.H)
                STA    2,SP
                
                TXA                 ; // add carry from previous ADD to MSB(far_page * 0x40) and store it in out.A
                ADC    #0
                STA    3,SP
                
   CONV_RETURN:
                /* put in registers (A:H:X) the right computed values and restore the stack */
                PULX               ; // RETURN_VALUE:0
                PULH               ; // RETURN_VALUE:8
                PULA               ; // RETURN_VALUE:16
   
                RTS
   }
}

#pragma NO_EXIT
void _CONV_LINEAR_TO_FAR(void) {
/* DESC:    converts a LINEAR (24-bit) pointer (LINEAR address) to a FAR (24-bit) pointer (PAGED address)
            (
            	paged_address:PAGE = linear_address / 0x4000
            	paged_address:OFFSET = linear_address % 0x4000 + 0x8000
            )
   IN:      A (address:0), H(address:8), X (address:16)
   OUT:     A (page), H(offset:0), X (offset:8)
   WRITTEN:
   Note:    in the comments bellow there will be used the notation in.* to refer to
            the value of the corresponding * register at the begining of the function,
            no matter its location in that moment */

   asm {
                /* save the A:H:X (all the modifications to H:X will be done directly to the stack locations they are placed in) */
                PSHA
                PSHH
                PSHX

                /* devide in.A:in.H to 0x40 */
                LDHX   3,SP        ; // put in.A in H
                LDA    2,SP        ; // put in.H in A
                LDX    #64         ; // put 0x40 in X
                DIV                ; // H:A / X => quotient in A and remainder in H
                STA    3,SP        ; // put the QUOTIENT in out.A
                PSHH               ; // put the REMAINDER on stack

                /* add 0x80 to the REMAINDER */
                PULA
                ADD    #128
                STA    2,SP

                /* put in registers (A:H:X) the right computed values and restore the stack */
                PULX               ; // RETURN_VALUE:0
                PULH               ; // RETURN_VALUE:8
                PULA               ; // RETURN_VALUE:16
   
                RTS
   }
}

#ifndef __NO_FLOAT__
/******************************************************************/
/* floats and doubles */
/******************************************************************/

/*************************** IEEE32 *******************************/

/* float work routines (not using softfloat) */

#if 0
  work unpacked IEEE32 registers K (also used as result) and L:

  Name:  Type:      Size:
  -------------------------------
  sign   FloatReg.s 1 byte
  mant   FloatReg.m 3 bytes
  exp    int        2 bytes
  spec   char       1 byte


stack layout 1 for unary float operations with return value (K = <op> K):
------------------------------------------------------------------------
    < K:4 .. K:7> only for 64 bit return value
    | K:3    | 15
    | K:2    | 14
    | K:1    | 13
    | K:0    | 12          FloatReg K
    | Ra:1   | 11
    | Ra:0   | 10          int Ra
    | A      |  9          char saved_A
    | X      |  8
    | H      |  7          int saved_HX
    | Kexp:1 |  6          exp of K
    | Kexp:0 |  5
    | Kspec  |  4          IEEE32 state of K (!= 0, == 0, NAN, INV)
    | temp   |  3          used by work routines
    | ra:1   |  2          int ra (only if work routine is called)
    | ra:0   |  1


stack layout 2 for binary float operations with return value (K = K <op> L):
---------------------------------------------------------------------------

    | K:3    | 32
    | K:2    | 31
    | K:1    | 30
    | K:0    | 29          FloatReg K
    | Ra:1   | 28
    | Ra:0   | 27          int Ra
    | A      | 26          char saved_A
    | X      | 25
    | H      | 24          int saved_HX
    | w:3    | 23
    | w:2    | 22
    | w:1    | 21
    | w:0    | 20          long w  (additional workspace for * / % and float ops)
    | L:3    | 19
    | L:2    | 18
    | L:1    | 17
    | L:0    | 16          FloatReg L
    | Lexp:1 | 15          exp of L
    | Lexp:0 | 14
    | Lspec  | 13          IEEE32 state of L (!= 0, == 0, NAN, INV)
    | Kexp:1 | 12          exp of K
    | Kexp:0 | 11
    | Kspec  | 10          IEEE32 state of K (!= 0, == 0, NAN, INV)
    | M:5    |  9          additional wsp for shift and add algorithm
    | M:4    |  8
    | M:3    |  7
    | M:2    |  6
    | M:1    |  5
    | M:0    |  4
    | temp   |  3          used by work routines
    | ra:1   |  2          int ra (only if work routine is called)
    | ra:0   |  1

#endif

typedef struct {
  char s;
  char m[3];
} FloatReg;

typedef struct {
  char m[6];
} FloatMantExt;

/* stack layout 1 */
#define _PARAM_UNARY_F         char temp, char Kspec, short Kexp, short saved_HX, char saved_A, short Ra, FloatReg K, ...
#define _PARAM_UNARY_F_2       short ra, _PARAM_UNARY_F

/* stack layout 2 */
#define _PARAM_BINARY_F         char temp, FloatMantExt M, char Kspec, short Kexp, char Lspec, short Lexp, FloatReg L, long w, short saved_HX, char saved_A, short Ra, FloatReg K, ...
#define _PARAM_BINARY_F_2       short ra, _PARAM_BINARY_F

#define Kmant    K.m
#define Lmant    L.m
#define Ksign    K.s
#define Lsign    L.s

#define  SPECIAL_ZERO       2
#define  SPECIAL_NORMAL     0
#define  SPECIAL_NAN      (-1)
#define  SPECIAL_INFINITY (-2)    /* INF, -INF */

#define  F_WIDTH           24
#define  F_EXPLO          126
#define  F_EXPHI            0

#define Lmant_Kmant        13
#define M_Lmant          (-13)

/* utilities shared over IEEE32 routines */

#pragma NO_EXIT
static void F_CLRK (_PARAM_BINARY_F) {
/* set K to 0 */
  asm {
              LDA      #(SPECIAL_ZERO)
              STA      Kspec
              RTS ! {}, {HX+A+SR}
  }
}

#pragma NO_EXIT
static void F_CLRK_Unary (_PARAM_UNARY_F) {
/* set K to 0 */
  asm {
              LDA      #(SPECIAL_ZERO)
              STA      Kspec
              RTS ! {}, {HX+A+SR}
  }
}

#pragma NO_EXIT
static void F_NANK (_PARAM_BINARY_F) {
/* set K to SPECIAL_NAN */
  asm {
              LDA      #(SPECIAL_NAN)
              STA      Kspec
              RTS ! {}, {HX+A+SR}
  }
}

#pragma NO_EXIT
static void F_INFK (_PARAM_BINARY_F) {
/* set K to INV */
  asm {
              LDA      #(SPECIAL_INFINITY)
              STA      Kspec
              RTS ! {}, {HX+A+SR}
  }
}

#pragma NO_EXIT
static void F_INFK_Unary (_PARAM_UNARY_F) {
/* set K to INV */
  asm {
              LDA      #(SPECIAL_INFINITY)
              STA      Kspec
              RTS ! {}, {HX+A+SR}
  }
}

#pragma NO_EXIT
static void F_TLK (_PARAM_BINARY_F_2){
/* K := L */
  asm {
              LDA     Lsign
              STA     Ksign
              LDA     Lspec
              STA     Kspec
              BNE     TLK_0
#ifdef __HCS08__
              LDHX    Lexp:0
              STHX    Kexp:0
              LDHX    Lsign
              STHX    Ksign
              LDHX    Lmant:1
              STHX    Kmant:1
#else
              LDA     Lexp:0
              STA     Kexp:0
              LDA     Lexp:1
              STA     Kexp:1
              LDA     Lmant:0
              STA     Kmant:0
              LDA     Lmant:1
              STA     Kmant:1
              LDA     Lmant:2
              STA     Kmant:2
#endif
    TLK_0:    ;
              RTS ! {}, {HX+A+SR}
  }
}

#pragma NO_EXIT
static void F_XGKL (_PARAM_BINARY_F_2) {
/* K <=> L */
  asm {
              LDA     Kspec
              LDX     Lspec
              STX     Kspec
              STA     Lspec
              LDA     Kexp:0
              LDX     Lexp:0
              STX     Kexp:0
              STA     Lexp:0
              LDA     Kexp:1
              LDX     Lexp:1
              STX     Kexp:1
              STA     Lexp:1
              LDA     Ksign
              LDX     Lsign
              STX     Ksign
              STA     Lsign
              LDA     Kmant:0
              LDX     Lmant:0
              STX     Kmant:0
              STA     Lmant:0
              LDA     Kmant:1
              LDX     Lmant:1
              STX     Kmant:1
              STA     Lmant:1
              LDA     Kmant:2
              LDX     Lmant:2
              STX     Kmant:2
              STA     Lmant:2
              RTS ! {}, {HX+A+SR}
  }
}

#pragma NO_EXIT
static void _UNPACK_k_to_K(_PARAM_BINARY_F) {
/* IEEE32 k  --->  virtual register K */
  asm {
               CLR     Kspec
               ;
               LDA     K:0
               LSLA
               CMP     #0xFE
               BNE     GETK_6    ; // not special
               LDA     K:1
               BPL     GETK_5    ; // not special and not 0.
               ; // {
               ; //   number is special!
               ; // }
               DEC     Kspec
               LDA     K:1
               LSLA
               ORA     K:2
               ORA     K:3
               BNE     GETK_7   ; // SPECIAL_NAN
               DEC     Kspec    ; // SPECIAL_INFINITY
    GETK_7:    ;
               RTS ! {}, {HX+A+SR}
    GETK_6:    ; // is it 0?
               ORA     K:1
               ORA     K:2
               ORA     K:3
               BNE     GETK_5    ; // number != 0
               ;
               LDA     #SPECIAL_ZERO
               STA     Kspec     ; // SPECIAL_ZERO
               RTS ! {}, {HX+A+SR}
    GETK_5:    ;
               LDA     K:0
               CLR     K:0
               LDX     K:1
               LSLX
               ROLA
               BCC     GETK_0
               COM     K:0
               TSTA
    GETK_0:    ;
               BEQ     GETK_1
               SEC
    GETK_1:    ;
               RORX
               STX     K:1
               ; // {
               ; //   A contains exponent.
               ; //   Kmant & Ksign are set.
               ; // }
               CLR     Kexp:0
               STA     Kexp:1
               BNE     GETK_2  ; // normalized number
               ;
    GETK_3:    ;
               DEC     Kexp:1
               LSL     Kmant:2
               ROL     Kmant:1
               ROL     Kmant:0
               BPL     GETK_3
               ;
               INC     Kexp:1
               BEQ     GETK_2
               COM     Kexp:0
    GETK_2:    ; // { assert (K is normalized) }
               ;
               RTS ! {}, {HX+A+SR}
  } /* msa */
}

#pragma NO_EXIT
static void _UNPACK_k_to_K_Unary(_PARAM_UNARY_F) {
/* IEEE32 k  --->  virtual register K */
  asm {
               CLR     Kspec
               ;
               LDA     K:0
               LSLA
               CMP     #0xFE
               BNE     GETK_6    ; // not special
               LDA     K:1
               BPL     GETK_5    ; // not special and not 0.
               ; // {
               ; //   number is special!
               ; // }
               DEC     Kspec
               LDA     K:1
               LSLA
               ORA     K:2
               ORA     K:3
               BNE     GETK_7   ; // SPECIAL_NAN
               DEC     Kspec    ; // SPECIAL_INFINITY
    GETK_7:    ;
               RTS ! {}, {HX+A+SR}
    GETK_6:    ; // is it 0?
               ORA     K:1
               ORA     K:2
               ORA     K:3
               BNE     GETK_5    ; // number != 0
               ;
               LDA     #SPECIAL_ZERO
               STA     Kspec     ; // SPECIAL_ZERO
               RTS ! {}, {HX+A+SR}
    GETK_5:    ;
               LDA     K:0
               CLR     K:0
               LDX     K:1
               LSLX
               ROLA
               BCC     GETK_0
               COM     K:0
               TSTA
    GETK_0:    ;
               BEQ     GETK_1
               SEC
    GETK_1:    ;
               RORX
               STX     K:1
               ; // {
               ; //   A contains exponent.
               ; //   Kmant & Ksign are set.
               ; // }
               CLR     Kexp:0
               STA     Kexp:1
               BNE     GETK_2  ; // normalized number
               ;
    GETK_3:    ;
               DEC     Kexp:1
               LSL     Kmant:2
               ROL     Kmant:1
               ROL     Kmant:0
               BPL     GETK_3
               ;
               INC     Kexp:1
               BEQ     GETK_2
               COM     Kexp:0
    GETK_2:    ; // { assert (K is normalized) }
               ;
               RTS ! {}, {HX+A+SR}
  } /* msa */
}

#pragma NO_EXIT
static void _UNPACK_j_to_L(_PARAM_BINARY_F) {
/* IEEE32 j  --->  virtual register L */
  asm {
               CLR     Lspec
               ;
               LDA     L:0
               LSLA
               CMP     #0xFE
               BNE     GETL_6    ; // not special
               LDA     L:1
               BPL     GETL_5    ; // not special and not 0.
               ; // {
               ; //   number is special!
               ; // }
               DEC     Lspec
               LDA     L:1
               LSLA
               ORA     L:2
               ORA     L:3
               BNE     GETL_7   ; // SPECIAL_NAN
               DEC     Lspec    ; // SPECIAL_INFINITY
    GETL_7:    ;
               RTS ! {}, {HX+A+SR}
    GETL_6:    ; // is it 0?
               ORA     L:1
               ORA     L:2
               ORA     L:3
               BNE     GETL_5    ; // number != 0
               ;
               LDA     #SPECIAL_ZERO
               STA     Lspec     ; // SPECIAL_ZERO
               RTS ! {}, {HX+A+SR}
    GETL_5:    ;
               LDA     L:0
               CLR     L:0
               LDX     L:1
               LSLX
               ROLA
               BCC     GETL_0
               COM     L:0
               TSTA
    GETL_0:    ;
               BEQ     GETL_1
               SEC
    GETL_1:    ;
               RORX
               STX     L:1
               ; // {
               ; //   A contains exponent.
               ; //   Lmant & Lsign are set.
               ; // }
               CLR     Lexp:0
               STA     Lexp:1
               BNE     GETL_2  ; // normalized number
               ;
    GETL_3:    ;
               DEC     Lexp:1
               LSL     Lmant:2
               ROL     Lmant:1
               ROL     Lmant:0
               BPL     GETL_3
               ;
               INC     Lexp:1
               BEQ     GETL_2
               COM     Lexp:0
    GETL_2:    ; // { assert (L is normalized) }
               ;
               RTS ! {}, {HX+A+SR}
  } /* msa */
}

#pragma NO_EXIT
static void _PACK_K_to_k (_PARAM_BINARY_F) {
/* virtual register K  --->  IEEE32 k */
  asm {
               LDA     Kspec
               BEQ     PUTK_1
               ;
               BMI     PUTK_2
               ; // {
               ; //   number == 0.
               ; // }
               CLR     K:0
               CLR     K:1
               BRA     PUTK_5
    PUTK_2:    ;
               CMP     #(SPECIAL_NAN)
               BEQ     PUTK_3
               LDA     #0x80
               STA     K:1
               COMA            ; // 0x7F
               ORA     K:0     ; // + or - SPECIAL_INFINITY
               STA     K:0
    PUTK_5:
               CLR     K:2
               CLR     K:3
               BRA     PUTK_4
    PUTK_3:    ; // SPECIAL_NAN
               LDA     #0x7F
               STA     K:0
               LDA     #0xFF
               STA     K:1
               STA     K:2
               STA     K:3
               BRA     PUTK_4
    PUTK_1:    ; // {
               ; //   number is not special
               ; // }
               LDA     Kexp:1
               BNE     PUTK_0
               LSR     Kmant:0
               ROR     Kmant:1
               ROR     Kmant:2
               LSR     Ksign
               CLR     K:0
               ROR     K:0
               BRA     PUTK_4
    PUTK_0:    ;
               LSL     K:1
               LSL     Ksign
               RORA
               STA     K:0
               ROR     K:1
    PUTK_4:
               _EXIT_BINARY_F
               RTS
  } /* msa */
}

#pragma NO_EXIT
static void _PACK_K_to_k_Unary (_PARAM_UNARY_F) {
/* virtual register K  --->  IEEE32 k */
  asm {
               LDA     Kspec
               BEQ     PUTK_1
               ;
               BMI     PUTK_2
               ; // {
               ; //   number == 0.
               ; // }
               CLR     K:0
               CLR     K:1
               BRA     PUTK_5
    PUTK_2:    ;
               CMP     #(SPECIAL_NAN)
               BEQ     PUTK_3
               LDA     #0x80
               STA     K:1
               COMA            ; // 0x7F
               ORA     K:0     ; // + or - SPECIAL_INFINITY
               STA     K:0
    PUTK_5:
               CLR     K:2
               CLR     K:3
               BRA     PUTK_4
    PUTK_3:    ;
               LDA     #0x7F
               STA     K:0
               LDA     #0xFF
               STA     K:1
               STA     K:2
               STA     K:3
               BRA     PUTK_4
    PUTK_1:    ; // {
               ; //   number is not special
               ; // }
               LDA     Kexp:1
               BNE     PUTK_0
               LSR     Kmant:0
               ROR     Kmant:1
               ROR     Kmant:2
               LSR     Ksign
               CLR     K:0
               ROR     K:0
               BRA     PUTK_4
    PUTK_0:    ;
               LSL     K:1
               LSL     Ksign
               RORA
               STA     K:0
               ROR     K:1
    PUTK_4:
               _EXIT_UNARY_F
               RTS ! {}, {HX+A+SR}
  } /* msa */
}

#pragma NO_EXIT
static void F_NORMK (_PARAM_BINARY_F) {
/* normalize K */
  asm {
              TST     Kspec
              BEQ     NORMK_11
              ;
              RTS ! {}, {HX+A+SR}
    NORMK_11: ;
              LDA     Kmant:0
              ORA     Kmant:1
              ORA     Kmant:2
              BNE     NORMK_10
              JMP     F_CLRK ! {}, {HX+A+SR}
    NORMK_10: ;
              TST     Kexp:0
              BMI     NORMK_0    ; // underflow ->denormalize
              BEQ     NORMK_1    ; // in range
              ; // {
              ; //   OVERFLOW !
              ; // }
              JMP     F_INFK ! {}, {HX+A+SR}
    NORMK_1:  ;
              TST     Kmant:0
              BMI     NORMK_2    ; // leave as is
              LDA     Kexp:1
    NORMK_3:  ;
              TSTA
              BEQ     NORMK_6
              DECA
              LSL     Kmant:2
              ROL     Kmant:1
              ROL     Kmant:0
              BPL     NORMK_3
              ;
    NORMK_4:  ;
              STA     Kexp:1
    NORMK_2:  ;
              RTS ! {}, {HX+A+SR}
              ;
    NORMK_0:  ; // { 
              ; //   assert (expo < 0);
              ; //   denormalize!
              ; // }
              LDA     Kexp:1
              NEGA
    NORMK_5:  ;
              LSR     Kmant:0
              ROR     Kmant:1
              ROR     Kmant:2
              DBNZA   NORMK_5
    NORMK_6:  ; // {
              ; //   return non-normalized number
              ; // }
              CLR     Kexp:1
              CLR     Kexp:0
    NORMK_7:  ;
              RTS ! {}, {HX+A+SR}
  }
}


/* IEEE32 operations */

#pragma NO_EXIT
static void _FADD_K_is_K_plus_L(_PARAM_BINARY_F) {
/* K = K + L */
  asm {
              LDA     Kspec
              ORA     Lspec
              BEQ     ADDKL_Normal
              ; // {
              ; //   one number is special
              ; // }
              BPL     ADDKL_Zero     ; // SPECIAL_ZERO
              ; // NAN + x or x + NAN or INF + x or x + INF or INF + INF
              INCA
              BEQ     ADDKL_RET_NAN    ; // return K   : NAN + x or x + NAN -> NAN
              ;
              LDA     Lspec
              EOR     Kspec
              BPL     ADDKL_BothInf  ; // INF + INF case
              LDA     Kspec
              CMP     #(SPECIAL_INFINITY)
              BNE     ADDKL_2        ; // return L   : whatever + (+/-)INF -> (+/-)INF 
              RTS ! {}, {HX+A+SR}    ; // return K   : (+/-)INF + whatever -> (+/-)INF
ADDKL_Zero:                          ; // 0 + x or x + 0 (or 0 + 0) (assuming x is not special)
              LDA     Kspec
              BGT     ADDKL_2        ; // return L // 0 + L
              RTS ! {}, {HX+A+SR}    ; // return K // K + 0       
ADDKL_BothInf:                       ; // INF + INF case. compare signs. If != -> NAN
              LDA     K:0
              EOR     L:0
              BPL     ADDKL_3        ; // return K
ADDKL_RET_NAN:
              JMP     F_NANK ! {}, {HX+A+SR} ; // return NAN : (+/-)INF + (-/+)INF -> NAN

ADDKL_Normal:                        ; // L and K are not special (means not 0,NAN,inf)
              LDA     Kexp:1
              SUB     Lexp:1
              TAX
              LDA     Kexp:0
              SBC     Lexp:0
              BCS     ADDKL_0   ; // K:exp < L:exp
              TXA
              BEQ     ADDKL_1   ; // K:exp >= L:exp
              TST     Kexp:0    ; // R2780 negative Kexp not handled!
              BLT     ADDKL_0   ; // K:exp < L:exp
    ADDKL_3:  ;
              RTS ! {}, {HX+A+SR}               ; // return K
    ADDKL_2:  ;
              JSR     F_TLK ! {}, {HX+A+SR}     ; // return L
              RTS ! {}, {HX+A+SR}
    ADDKL_0:  ;
              COMA
              TXA
              BNE     ADDKL_2   ; // return L
              ; // { K:exp < L:exp ==> swap K,L }
              NEGA
              BEQ     ADDKL_2
              ;
              STA     M:0
              JSR     F_XGKL ! {}, {HX+A+SR}
              LDA     M:0
              ;
              ; // NOTE: One may not push-pull registers,
              ; //       since F_XGKL expects ... K, L ... at
              ; //       fixed offset !
              ;
    ADDKL_1:  ; // {
              ; //   assert (K:exp >= L:exp)
              ; //   A == K:exp - L:exp
              ; // }
              CMP     #(F_WIDTH)
              BHS     ADDKL_3   ; // return K
              TSTA
              BEQ     ADDKL_4
              ;
    ADDKL_5:  LSR     Lmant:0
              ROR     Lmant:1
              ROR     Lmant:2
              DBNZA   ADDKL_5
              ;
    ADDKL_4:  ; // {
              ; //   K, L mantissas with exponent == K:exp
              ; // }
              LDA     Ksign
              CMP     Lsign
              BEQ     ADDKL_6     ; // signs are the same
              ; // {
              ; //   assert (Ksign != Lsign)
              ; //   K, L mantissas with exponent == K:exp
              ; // }
              LDA     Kmant:2
              SUB     Lmant:2
              STA     Kmant:2
              LDA     Kmant:1
              SBC     Lmant:1
              STA     Kmant:1
              LDA     Kmant:0
              SBC     Lmant:0
              STA     Kmant:0
              BCC     ADDKL_8     ; // |K| was greater than |L|
              ; // {
              ; //   assert (K.mant == K.mant - L.mant);
              ; // }
              COM     Ksign
              COM     Kmant:0
              COM     Kmant:1
              NEG     Kmant:2
              BNE     ADDKL_8
              INC     Kmant:1
              BNE     ADDKL_8
              INC     Kmant:0

    ADDKL_8:  ; // {
              ; //   informal: K contains the correct result. 
              ; //             But K.mant may have leading zeros.
              ; // }
              JMP     F_NORMK ! {}, {HX+A+SR}   ; // normalize and return
    ADDKL_6:  ; // {
              ; //   assert (Ksign == Lsign)
              ; //   K, L mantissas with exponent == K:exp
              ; // }
              LDA     Kmant:2
              ADD     Lmant:2
              STA     Kmant:2
              LDA     Kmant:1
              ADC     Lmant:1
              STA     Kmant:1
              LDA     Kmant:0
              ADC     Lmant:0
              STA     Kmant:0
              BCC     ADDKL_11     ; // no overflow
              ROR     Kmant:0
              ROR     Kmant:1
              ROR     Kmant:2
              INC     Kexp:1
              BNE     ADDKL_11
              INC     Kexp:0
    ADDKL_11: ; // {
              ; //   assert (result in K)
              ; // }
              JMP     F_NORMK ! {}, {HX+A+SR}    ; // normalize and return 
  }
}

#define _TEMP0  w:3
#define _TEMP1  temp

#pragma NO_EXIT
static void _FMUL_K_is_K_times_L(_PARAM_BINARY_F) {
/* K = K * L */
  asm {
                LDA     Kspec
                ORA     Lspec
                BEQ     MULKL_12
                ; // {
                ; //   one number is special
                ; // }
                ; // NAN + x or x + NAN or INF + x or x + INF or INF + INF
                INCA
                BEQ     MULKL_RET_NAN    ; // return K   : NAN + x or x + NAN -> NAN
                BMI     MULKL_ONE_INF
                ; // None is NAN or INF, but one is special -> 0*0, 0*x or X*0 cases. All result in 0

                JMP     F_CLRK ! {}, {HX+A+SR} ; // return 0 for 0 * x
    MULKL_ONE_INF:
                LDA     Kspec
                EOR     Lspec
                CBEQA   #(SPECIAL_INFINITY^SPECIAL_ZERO), MULKL_RET_NAN ; // 0 * INF or INF * 0 case                
                BRA     MULKL_INF_RES ; // INF * INF, INF * X or X * INF
    MULKL_RET_NAN:
                JMP     F_NANK ! {}, {HX+A+SR} ; // return NAN for INF * 0
    MULKL_NOT_ZERO:   ; // L != 0 -> L is INF or normal
                TSTA
                BEQ     MULKL_INF_RES   ; // branch if L is normal
                LDA     Kspec  
                BGT     MULKL_RET_NAN ; // SPECIAL_ZERO)    ; return NAN for 0 * INF
    MULKL_INF_RES:             
                LDA      Lsign ; // INF * x or x * INF or INF * x
                EOR      Ksign
                STA      Ksign
                JMP      F_INFK ! {}, {HX+A+SR} ; // return NAN for INF * 0

    MULKL_12:   ;
                LDA      Lsign
                EOR      Ksign
                STA      Lsign
                ; // { sign done }
                LDA      Lexp:1
                ADD      Kexp:1
                STA      Lexp:1
                LDA      Lexp:0
                ADC      Kexp:0
                PSHA
                LDA      Lexp:1
                SUB      #(F_EXPLO)
                STA      Lexp:1
                PULA
                SBC      #(F_EXPHI)
                STA      Lexp:0
                ; // { exp done }
                CLR      M:0
                CLR      M:1
                CLR      M:2
                LDA      Kmant:0
                STA      M:3
                LDA      Kmant:1
                STA      M:4
                LDA      Kmant:2
                STA      M:5
                CLR      Lmant:3
                CLR      Lmant:4
                CLR      Lmant:5
                LDA      #(F_WIDTH)
                STA      _TEMP0
                ;
#if defined(__OPTIMIZE_FOR_TIME__)
    MULKL_0:    LSL      Lmant:5
                ROL      Lmant:4
                ROL      Lmant:3
                ROL      Lmant:2
                ROL      Lmant:1
                ROL      Lmant:0
#else
    MULKL_0:    LDHX     @Lmant:6
                LDA      #6
                CLC
                ;
    MULKL_1:    AIX      #-1
                ROL      0,X
                DBNZA    MULKL_1
#endif
                ;
                BCC      MULKL_2
#if defined( __OPTIMIZE_FOR_TIME__)
                LDHX     @Lmant:M_Lmant
                LDA      5-M_Lmant,X
                ADD      5,X
                STA      5-M_Lmant,X
                LDA      4-M_Lmant,X
                ADC      4,X
                STA      4-M_Lmant,X
                LDA      3-M_Lmant,X
                ADC      3,X
                STA      3-M_Lmant,X
                LDA      2-M_Lmant,X
                ADC      2,X
                STA      2-M_Lmant,X
                LDA      1-M_Lmant,X
                ADC      1,X
                STA      1-M_Lmant,X
                LDA      0-M_Lmant,X
                ADC      0,X
                STA      0-M_Lmant,X

#else
                LDHX     @Lmant:6
                LDA      #6
                STA      _TEMP1
                CLC
                ;
    MULKL_3:    AIX      #-1
                LDA      0,X
                ADC      M_Lmant,X
                STA      0,X
                DBNZ      _TEMP1,MULKL_3
#endif
                ;
    MULKL_2:    ;
                DBNZ      _TEMP0,MULKL_0
                ;
                ; // (result in L)
                ;
                JSR      F_TLK ! {}, {HX+A+SR}
                JMP      F_NORMK ! {}, {HX+A+SR}
  }
}


#undef  _TEMP0
#undef  _TEMP1
#define _TEMP0   w:3
#define _TEMP1   temp

#pragma NO_EXIT
static void _FDIV_K_is_L_div_K (_PARAM_BINARY_F) {
/* K = K / L */
  asm {
                LDA      Kspec
                ORA      Lspec
                BEQ      DIVLK_12
                ; // {
                ; //   one number is special
                ; // }
                INCA
                BEQ      DIVLK_RET_NAN
                LDA      Kspec
                BGT      DIVLK_RET_NAN       ; // SPECIAL_ZERO: branch for "L / 0" case, return NAN
                LDA      Lspec
                BGT      DIVLK_13            ; // SPECIAL_ZERO: branch for "0 / x" case, return 0
                CMP      #(SPECIAL_INFINITY) ; // INF / (X|INF) case?
                BNE      DIVLK_13            ; // branch for "x / INF" case. return 0
                                             ; // INF / X or INF / INF case
                LDA      Kspec
                CMP      #(SPECIAL_INFINITY) ; // INF / INF or x / INF?
                BNE      DIVLK_16            ; // branch for "INF / x" case. return INF with sign(INF^x)
                                             ; // for "INF/INF" case, return NAN 
    DIVLK_RET_NAN:   ;
                JMP      F_NANK ! {}, {HX+A+SR}
    DIVLK_16:                                ; // INF / x case
                LDA      Lsign
                EOR      Ksign
                STA      Ksign               ; // sign(res) = sign(K) ^ sign(L)
                JMP      F_INFK ! {}, {HX+A+SR}

    DIVLK_13:   ;
                JMP      F_CLRK ! {}, {HX+A+SR}
    DIVLK_12:   ;
                LDA      Lsign
                EOR      Ksign
                STA      Ksign
                ; // { sign done }
                LDA      Lexp:1
                SUB      Kexp:1
                STA      Kexp:1
                LDA      Lexp:0
                SBC      Kexp:0
                PSHA
                LDA      Kexp:1
                ADD      #(F_EXPLO + 1)
                ;
                ; // NOTE: add one more, since the virtual
                ; //       fractional point will be at bit <23>.<22>
                ;
                STA      Kexp:1
                PULA
                ADC      #(F_EXPHI)
                STA      Kexp:0
                ; // { exp done }
                CLR      M:0
                CLR      M:1
                CLR      M:2
                ;
                LDA      #(F_WIDTH)
                STA      _TEMP0
                CLC
    DIVLK_0:    ;
                ; // { carry set if > 0xFFFF }
                ;
                BCS      DIVLK_1
                LDA      Kmant:0
                CMP      Lmant:0
                BNE      DIVLK_2
                LDA      Kmant:1
                CMP      Lmant:1
                BNE      DIVLK_2
                LDA      Kmant:2
                CMP      Lmant:2
    DIVLK_2:    ;
                BHI      DIVLK_3   ; // carry cleared !
    DIVLK_1:    ;
#if 0 // loop is longer and slower
                LDHX     @Lmant:3
                LDA      #3
                STA      _TEMP1
                CLC
    DIVLK_4:    ;
                AIX      #-1
                LDA      0,X
                SBC      Lmant_Kmant,X
                STA      0,X
                DBNZ     _TEMP1,DIVLK_4
#else
                LDA      Lmant:2
                SUB      Kmant:2
                STA      Lmant:2
                LDA      Lmant:1
                SBC      Kmant:1
                STA      Lmant:1
                LDA      Lmant:0
                SBC      Kmant:0
                STA      Lmant:0
#endif
                SEC
    DIVLK_3:    ; // {
                ; //   Carry is set if q += 1
                ; // }
                ROL      M:2
                ROL      M:1
                ROL      M:0
                LSL      Lmant:2
                ROL      Lmant:1
                ROL      Lmant:0
                DBNZ     _TEMP0,DIVLK_0
                ;
                LDA      M:0
                STA      Kmant:0
                LDA      M:1
                STA      Kmant:1
                LDA      M:2
                STA      Kmant:2
                JMP      F_NORMK ! {}, {HX+A+SR}
  }
}

#pragma NO_EXIT
static void _F_LONGK (_PARAM_UNARY_F) {
/* K = (long) K */
  asm {
                LDA      Kspec
                BEQ      LONGK_12
                ;
                CMP      #(SPECIAL_ZERO)
                BNE      LONGK_13
                ;
    LONGK_0:    ;
                CLR      K:0
    LONGK_14:   ;
                CLR      K:1
                CLR      K:2
                CLR      K:3
                BRA      LONGK_5
    LONGK_13:   ;
                LDA      #0x80
                STA      K:0
                BRA      LONGK_14
    LONGK_12:   ;
                LDA      Ksign
                CLR      Ksign
                STA      temp
                LDA      Kexp:1
                SUB      #(F_EXPLO)
                PSHA
                LDA      Kexp:0
                SBC      #(F_EXPHI)
                PULA
                BCS      LONGK_0    ; // number < 1
                CMP      #(32)
                BHI      LONGK_0    ; // number too large
                ;
                SUB      #(F_WIDTH)
                BPL      LONGK_1    ; // shift left
                NEGA
                ;
    LONGK_2:    ;
                LSR      K:1
                ROR      K:2
                ROR      K:3
                DBNZA    LONGK_2
                ;
                BRA      LONGK_3
                ;
    LONGK_1:    ;
                BEQ      LONGK_3
    LONGK_4:    ;
                LSL      K:3
                ROL      K:2
                ROL      K:1
                ROL      K:0
                DBNZA    LONGK_4
                ;
    LONGK_3:    ;
                TST      temp
                BPL      LONGK_5     ; // number positive
                LDHX     @K
                JSR      _NEG_L_HX
                ;
    LONGK_5:
                _EXIT_UNARY_F
                RTS ! {}, {HX+A+SR}
                ;
  }
}

#pragma NO_EXIT
static void _F_FLOATK (_PARAM_UNARY_F) {
/* K = (float) K */
  asm {
#if defined(__OPTIMIZE_FOR_TIME__)
                LDA      #(24)
                TST      K:0
                BEQ      FLOAT_0  ; // shift left
    FLOAT_1:    ;
                LSR      K:0
                ROR      K:1
                ROR      K:2
                ROR      K:3
                INCA
                TST      K:0
                BNE      FLOAT_1
    FLOAT_2:    ;
                ADD      #(F_EXPLO)
                STA      Kexp:1
                CLR      Kexp:0
                CLR      Kspec
                RTS ! {}, {HX+A+SR}
                ;
    FLOAT_0:    TST      K:1
                BMI      FLOAT_2
                DECA
                LSL      K:3
                ROL      K:2
                ROL      K:1
    FLOAT_01:   
                BMI      FLOAT_2
                DECA
                LSL      K:3
                ROL      K:2
                ROL      K:1
    FLOAT_02:   
                BMI      FLOAT_2
                DECA
                LSL      K:3
                ROL      K:2
                ROL      K:1
    FLOAT_03:   
                BMI      FLOAT_2
                DECA
                LSL      K:3
                ROL      K:2
                ROL      K:1
    FLOAT_04:   
                BMI      FLOAT_2
                DECA
                LSL      K:3
                ROL      K:2
                ROL      K:1
    FLOAT_05:   
                BMI      FLOAT_2
                DECA
                LSL      K:3
                ROL      K:2
                ROL      K:1
    FLOAT_06:   
                BMI      FLOAT_2
                DECA
                LSL      K:3
                ROL      K:2
                ROL      K:1
    FLOAT_07:   
                BMI      FLOAT_2
                DECA
                LSL      K:3
                ROL      K:2
                ROL      K:1
    FLOAT_08:   
                BMI      FLOAT_2
                DECA
                LSL      K:3
                ROL      K:2
                ROL      K:1
    FLOAT_09:   
                BMI      FLOAT_2
                DECA
                LSL      K:3
                ROL      K:2
                ROL      K:1
    FLOAT_10:   
                BMI      FLOAT_2
                DECA
                LSL      K:3
                ROL      K:2
                ROL      K:1
    FLOAT_11:   
                BMI      FLOAT_2
                DECA
                LSL      K:3
                ROL      K:2
                ROL      K:1
    FLOAT_12:   
                BMI      FLOAT_2
                DECA
                LSL      K:3
                ROL      K:2
                ROL      K:1
    FLOAT_13:   
                BMI      FLOAT_2
                DECA
                LSL      K:3
                ROL      K:2
                ROL      K:1
    FLOAT_14:   
                BMI      FLOAT_2
                DECA
                LSL      K:3
                ROL      K:2
                ROL      K:1
    FLOAT_15:   
                BMI      FLOAT_2
                DECA
                LSL      K:3
                ROL      K:2
                ROL      K:1
    FLOAT_16:   
                BMI      FLOAT_2
                DECA
                LSL      K:3
                ROL      K:2
                ROL      K:1
    FLOAT_17:   
                BMI      FLOAT_2
                DECA
                LSL      K:3
                ROL      K:2
                ROL      K:1
    FLOAT_18:   
                BMI      FLOAT_2
                DECA
                LSL      K:3
                ROL      K:2
                ROL      K:1
    FLOAT_19:   
                BMI      FLOAT_2
                DECA
                LSL      K:3
                ROL      K:2
                ROL      K:1
    FLOAT_20:   
                BMI      FLOAT_2
                DECA
                LSL      K:3
                ROL      K:2
                ROL      K:1
    FLOAT_21:   
                BMI      FLOAT_2
                DECA
                LSL      K:3
                ROL      K:2
                ROL      K:1
    FLOAT_22:   
                BMI      FLOAT_2
                DECA
                LSL      K:3
                ROL      K:2
                ROL      K:1
    FLOAT_23:   
                BMI      FLOAT_2
                DECA
                LSL      K:3
                ROL      K:2
                ROL      K:1
                ;
                JMP      F_CLRK_Unary ! {}, {HX+A+SR} ; // number == 0
#else
                LDA      #(24)
                TST      K:0
                BEQ      FLOAT_0  ; // shift left
    FLOAT_1:    ;
                LSR      K:0
                ROR      K:1
                ROR      K:2
                ROR      K:3
                INCA
                TST      K:0
                BNE      FLOAT_1
    FLOAT_2:    ;
                ADD      #(F_EXPLO)
                STA      Kexp:1
                CLR      Kexp:0
                CLR      Kspec
                RTS ! {}, {HX+A+SR}
                ;
    FLOAT_0:    TST      K:1
                BMI      FLOAT_2
                LSL      K:3
                ROL      K:2
                ROL      K:1
                DBNZA    FLOAT_0
                ;
                JMP      F_CLRK_Unary ! {}, {HX+A+SR} ; // number == 0
#endif
  }
}

#pragma NO_EXIT
static void _FCMP_k_rel_j(_PARAM_BINARY_L) {
/* unsigned CC = k <rel_op> j */
  asm {
                LDHX    @j
                LDA     #4
                STA     w:0
                LDA     k:0
                ORA     j:0
                BPL     _CMP_0     ; // both positive
    _CMP_1:     ;
                LDA     0,X
                CMP     13,X
                BNE     _CMP_3
                AIX     #1
                DBNZ    w:0,_CMP_1
                BRA     _CMP_3
    _CMP_0:     ;
                LDA     13,X
                CMP     0,X
                BNE     _CMP_3
                AIX     #1
                DBNZ    w:0,_CMP_0
    _CMP_3:
                AIS     #12
                PULA
                PULH
                PULX
                AIS     #4
                _SRET
                JMP     ,X
  }
}


#pragma NO_EXIT
static void _FADD_Common(void) {
  asm {
                JSR _UNPACK_k_to_K;
                JSR _UNPACK_j_to_L;
                JSR _FADD_K_is_K_plus_L;
                _SRET
                JSR _PACK_K_to_k;     /* returns itself */
  }
}

#pragma NO_EXIT
static void _FSUB_Common(void) {
  asm {
                LDA  14,SP
                EOR  #0x80
                STA  14,SP
          is_zero:
                _SRET
                JMP _FADD_Common  ! {}, {HX+A+SR}
  }
}

#pragma NO_EXIT
static void _FMUL_Common(void) {
  asm {
                JSR _UNPACK_k_to_K;
                JSR _UNPACK_j_to_L;
                JSR _FMUL_K_is_K_times_L;
                _SRET
                JSR _PACK_K_to_k;    /* returns itself */
  }
}


#pragma NO_EXIT
static void _FDIV_Common(void) {
  asm {
                JSR _UNPACK_k_to_K;
                JSR _UNPACK_j_to_L;
                JSR _FDIV_K_is_L_div_K;
                _SRET
                JSR _PACK_K_to_k;    /* returns itself */
  }
}

/* IEEE32 interface routines */

#define RESERVE_WSP_UNARY  (-4)
#define RELEASE_WSP_UNARY  4


#pragma NO_EXIT
void _FADD(void) {
/* DESC:    adds two floating point (IEEE32) values
   IN:      &f1: on stack, &f2 in HX
   OUT:     result on top of stack
   WRITTEN: none */
  asm {
    _ENTER_BINARY_F();
    JMP _FADD_Common
  }
}

#pragma NO_EXIT
void _FADD_RC(void) {           /* RC means right operand will follow in code stream */
/* DESC:    IEEE32 addition f1 + f2 (f2 constant)
   IN:      H:X @f1, 0,RA f2
   OUT:     result on top of stack
   WRITTEN: none */
  asm {
    _ENTER_BINARY_F_RC();
    _SRET
    JMP _FADD_Common
  }
}

#pragma NO_EXIT
void _FSUB(void) {
/* DESC:    subtracts two floating point (IEEE32) values (f1 - f2)
   IN:      &f2: on stack, &f1 in HX
   OUT:     result on top of stack
   WRITTEN: none */
  asm {
    _ENTER_BINARY_F();
    JMP _FSUB_Common
  }
}

#pragma NO_EXIT
void _FSUB_RC(void) {           /* RC means right operand will follow in code stream */
/* DESC:    IEEE32 subtraction f1 - f2 (f2 constant)
   IN:      H:X @f1, 0,RA f2
   OUT:     result on top of stack
   WRITTEN: none */
  asm {
    _ENTER_BINARY_F_RC();
    _SRET
    JMP _FSUB_Common
  }
}

#pragma NO_EXIT
void _FSUB_LC(void) {           /* LC means left operand will follow in code stream */
/* DESC:    IEEE32 subtraction f1 - f2 (f1 constant)
   IN:      H:X @f2, 0,RA f1
   OUT:     result on top of stack
   WRITTEN: none */
  asm {
    _ENTER_BINARY_F_LC();
    _SRET
    JMP _FSUB_Common
  }
}

#pragma NO_EXIT
void _FMUL(void) {
/* DESC:    multiplies two floating point (IEEE32) values (f1 * f2)
   IN:      &f1: on stack, &f2 in HX
   OUT:     result on top of stack
   WRITTEN: none */
  asm {
    _ENTER_BINARY_F();
    JMP _FMUL_Common
  }
}

#pragma NO_EXIT
void _FMUL_RC(void) {           /* RC means right operand will follow in code stream */
/* DESC:    IEEE32 multiplication f1 * f2 (f2 constant)
   IN:      H:X @f1, 0,RA f2
   OUT:     result on top of stack
   WRITTEN: none */
  asm {
    _ENTER_BINARY_F_RC();
    _SRET
    JMP _FMUL_Common
  }
}

#pragma NO_EXIT
void _FDIV(void) {
/* DESC:    divides two floating point (IEEE32) values (f1 / f2)
   IN:      &f2: on stack, &f1 in HX
   OUT:     result on top of stack
   WRITTEN: none */
  asm {
    _ENTER_BINARY_F();
    JMP _FDIV_Common
  }
}

#pragma NO_EXIT
void _FDIV_RC(void) {           /* RC means right operand will follow in code stream */
/* DESC:    IEEE32 division f1 / f2 (f2 constant)
   IN:      H:X @f1, 0,RA f2
   OUT:     result on top of stack
   WRITTEN: none */
  asm {
    _ENTER_BINARY_F_LC();
    _SRET
    JMP _FDIV_Common
  }
}

#pragma NO_EXIT
void _FDIV_LC(void) {           /* LC means left operand will follow in code stream */
/* DESC:    IEEE32 division f1 / f2 (f1 constant)
   IN:      H:X @f2, 0,RA f1
   OUT:     result on top of stack
   WRITTEN: none */
  asm {
    _ENTER_BINARY_F_RC();
    _SRET
    JMP _FDIV_Common
  }
}



#pragma NO_EXIT
void _FSTRUNC(void) {
/* DESC:    returns the integral part of a floating point (IEEE32) value
   IN:      HX: address of floating point value
   OUT:     result is on stack (TOS, 32bit)
   WRITTEN: none */
  asm {
    _ENTER_UNARY_F();
    JSR _UNPACK_k_to_K_Unary;
    JSR _F_LONGK;     /* returns itself */
  }
}

#pragma NO_EXIT
void _FUTRUNC(void) {
/* DESC:    returns the integral part of a floating point (IEEE32) value (range [0 .. 2^32[)
   IN:      HX: address of floating point value
   OUT:     result is on stack (TOS, 32bit)
   WRITTEN: A, H, X using softfloat, else none */
  asm JMP _FSTRUNC;  /* NYI */
}

#pragma NO_EXIT
void _FNEG(_PARAM_UNARY_L_INLINE) {
/* DESC:     negates a IEEE32 floating point value (f1 => -f1)
   IN:       HX: &f1
   OUT:      result is on stack (TOS, 32bit)
   WRITTEN:  none */

  _ENTER_UNARY_L();
  asm {
    LDA   k:3
    ORA   k:2
    ORA   k:1
    ORA   k:0
    BEQ   isZero
    LDA   k:0
    EOR   #0x80
    STA   k:0
  isZero:
    _EXIT_UNARY_L
    RTS  ! {}, {HX+A+SR}
  }
}

#pragma NO_EXIT
void _FABS(_PARAM_UNARY_L_INLINE) {
/* DESC:     absolute value of a IEEE32 value (-f1 => f1, f1 => f1)
   IN:       HX: &f1
   OUT:      result is on stack (TOS, 32bit)
   WRITTEN:  none */

  _ENTER_UNARY_L();
  asm {
    LDA   k
    AND   #0x7F  ; // clear sign bit
    STA   k
  finish:
    _EXIT_UNARY_L
    RTS
  }
}

#pragma NO_EXIT
void _FCMP(void) {
/* DESC:    Compares two IEEE32 values (f1 cmp f2)
   IN:      Stack: &f2, HX: &f1
   OUT:     CCR
   WRITTEN: H, X */

  _ENTER_BINARY_L();
  asm JSR _FCMP_k_rel_j;
}

#pragma NO_EXIT
void _FCMP_RC(void) {
/* DESC:    IEEE32 comparison f1 cmp f2 (f2 constant)
   IN:      H:X @f1, 0,RA f2
   OUT:     CCR
   WRITTEN: H, X */
  _ENTER_BINARY_L_RC();
  asm _SRET;
  asm JSR _FCMP_k_rel_j;
}

static const float plus1=+1.0;
static const float minus1=-1.0;

#pragma NO_EXIT
void _FINC(void) {
/* DESC:    increments a IEEE32 value by 1.0 (f++)
   IN:      H:X contains &f
   OUT:     TOS: f+1.0
   WRITTEN: else none */

   asm {
     AIS #-2
     PSHH             ; // save H&X for frame optimization
     PSHX
     PSHA
#ifdef __HCS08__
     LDHX  6,SP       ; // move return address
     STHX  4,SP
     LDHX  @plus1     ; // store address of constant
     STHX  6,SP
#else
     LDA   6,SP       ; // move return address
     STA   4,SP
     LDA   7,SP
     STA   5,SP
     LDA  @plus1      ; // store address of constant
     STA   7,SP
     LDA  @plus1:MSB
     STA   6,SP
#endif
     PULA
     PULX
     PULH
     JMP _FADD
   }
}

#pragma NO_EXIT
void _FDEC(void) {
/* DESC:    subtracts 1.0 from a IEEE32 value (f--)
   IN:      H:X contains &f
   OUT:     TOS: f-1.0
   WRITTEN: A, H, X using softfloat, else none */

   asm {
     AIS #-2
     PSHH             ; // save H&X for frame optimization
     PSHX
     PSHA
#ifdef __HCS08__
     LDHX  6,SP       ; // move return address 
     STHX  4,SP
     LDHX  @minus1    ; // store address of constant 
     STHX  6,SP
#else
     LDA   6,SP       ; // move return address 
     STA   4,SP
     LDA   7,SP
     STA   5,SP
     LDA  @minus1     ; // store address of constant 
     STA   7,SP
     LDA  @minus1:MSB
     STA   6,SP
#endif
     PULA
     PULX
     PULH
     JMP _FADD
   }
}

#pragma NO_EXIT
void _FSFLOAT(_PARAM_UNARY_L_INLINE) {
/* DESC:    converts a 32bit signed integral value into a IEEE32 value (f = L)
   IN:      H:X contains &L
   OUT:     TOS: converted IEEE32 value
   WRITTEN: none */
  asm {
                _ENTER_UNARY_F()
                TST   k:0
                BMI   negative
                JSR   _F_FLOATK
                BRA   done
  negative:
                LDHX  @k
                JSR   _NEG_L_HX
                JSR   _F_FLOATK
                COM   k:0                      ; // invert sign
  done:
                JSR   _PACK_K_to_k_Unary;   /* returns itself */
  }
}

#pragma NO_EXIT
void _FUFLOAT(void) {
/* DESC:    converts a 32bit unsigned integral value into a IEEE32 value (f = uL)
   IN:      H:X contains &uL
   OUT:     TOS: converted IEEE32 value
   WRITTEN: none */
  asm {
                _ENTER_UNARY_F()
                JSR _F_FLOATK;
                JSR _PACK_K_to_k_Unary;      /* returns itself */
  }
}

/*************************** IEEE64 *******************************/

/* float work routines (not using softfloat) */

#if 0
  work unpacked IEEE64 registers K (also used as result) and L:

  Name:  Type:      Size:
  -------------------------------
  sign   FloatReg.s 1 byte
  mant   FloatReg.m 3 bytes
  exp    int        2 bytes
  spec   char       1 byte


stack layout 1 for unary double operations with return value (K = <op> K):
-------------------------------------------------------------------------

    | K:7    | 19
    | K:6    | 18
    | K:5    | 17
    | K:4    | 16
    | K:3    | 15
    | K:2    | 14
    | K:1    | 13
    | K:0    | 12          DoubleReg K
    | Ra:1   | 11
    | Ra:0   | 10          int Ra
    | A      |  9          char saved_A
    | X      |  8
    | H      |  7          int saved_HX
    | Kexp:1 |  6          exp of K
    | Kexp:0 |  5
    | Kspec  |  4          IEEE64 state of K (!= 0, == 0, NAN, INV)
    | temp   |  3          used by work routines
    | ra:1   |  2          int ra (only if work routine is called)
    | ra:0   |  1


stack layout 2 for binary double operations with return value (K = K <op> L):
----------------------------------------------------------------------------

    | res:7  | 53
    | res:6  | 52
    | res:5  | 51
    | res:4  | 50
    | res:3  | 49
    | res:2  | 48
    | res:1  | 47
    | res:0  | 46          double res (result location and additional workspace for * / % and double ops)
    | Ra:1   | 45
    | Ra:0   | 44          int Ra
    | A      | 43          char saved_A
    | X      | 42
    | H      | 41          int saved_HX
    | K:7    | 40
    | K:6    | 39
    | K:5    | 38
    | K:4    | 37
    | K:3    | 36
    | K:2    | 35
    | K:1    | 34
    | K:0    | 33          DoubleReg K
    | L:7    | 32
    | L:6    | 31
    | L:5    | 30
    | L:4    | 29
    | L:3    | 28
    | L:2    | 27
    | L:1    | 26
    | L:0    | 25          DoubleReg L
    | Lexp:1 | 24          exp of L
    | Lexp:0 | 23
    | Lspec  | 22          IEEE64 state of L (!= 0, == 0, NAN, INV)
    | Kexp:1 | 21          exp of K
    | Kexp:0 | 20
    | Kspec  | 19          IEEE64 state of K (!= 0, == 0, NAN, INV)
    | M:15   | 18
    | M:14   | 17
    | M:13   | 16
    | M:12   | 15
    | M:11   | 14
    | M:10   | 13
    | M:9    | 12
    | M:8    | 11
    | M:7    | 10
    | M:6    |  9
    | M:5    |  8
    | M:4    |  7
    | M:3    |  6
    | M:2    |  5
    | M:1    |  4
    | M:0    |  3          additional wsp for shift and add algorithm
    | ra:1   |  2          int ra (only if work routine is called)
    | ra:0   |  1

#endif

typedef struct {
  char s;
  char m[7];
} DoubleReg;

typedef struct {
  char m[16];
} DoubleMantExt;

/* stack layout 1 */
#define _PARAM_UNARY_D         char temp, char Kspec, short Kexp, short saved_HX, char saved_A, short Ra, DoubleReg K, ...
#define _PARAM_UNARY_D_2       short ra, _PARAM_UNARY_D

/* stack layout 2 */
#define _PARAM_BINARY_D         DoubleMantExt M, char Kspec, short Kexp, char Lspec, short Lexp, DoubleReg L, DoubleReg K, short saved_HX, char saved_A, short Ra, double res, ...
#define _PARAM_BINARY_D_2       short ra, _PARAM_BINARY_D

#define Kmant    K.m
#define Lmant    L.m
#define Ksign    K.s
#define Lsign    L.s

#define  D_WIDTH        53
#define  D_EXPLO        251
#define  D_EXPHI        3
#define  D_MBITS        56
#define  D_MBYTES       7

#undef  Lmant_Kmant
#undef  M_Lmant

#define Kmant_Lmant     8
#define Lmant_Kmant     (-Kmant_Lmant)
#define Kmant_M         (31)
#define Lmant_M         (23)
#define M_Kmant         (-Kmant_M)
#define M_Lmant         (-Lmant_M)
#define Res_K           13
#define Res_L           21


/* utilities shared over IEEE64 routines */

#undef  _TEMP0
#define _TEMP0 res:0

#pragma NO_EXIT
static void D_TLK (_PARAM_BINARY_D_2) {
/* K := L */
  asm {
              LDA     Lsign
              STA     Ksign
              LDA     Lspec
              STA     Kspec
              BNE     _TLK_1
              ;
              LDA     Lexp:0
              STA     Kexp:0
              LDA     Lexp:1
              STA     Kexp:1
              LDA     #8
              STA     _TEMP0
              LDHX    @L
    _TLK_0:   ;
              LDA     0,X
              STA     Kmant_Lmant,X
              AIX     #1
              DBNZ    _TEMP0,_TLK_0
              ;
    _TLK_1:   ;
              RTS ! {}, {HX+A+SR}
  }
}

#undef  _TEMP0
#define _TEMP0 M:15

#pragma NO_EXIT
static void D_TLRes (_PARAM_BINARY_D_2) {
/* res := L */
  asm {
              LDA     #8
              STA     _TEMP0
              LDHX    @L
    _TLK_0:   ;
              LDA     0,X
              STA     Res_L,X
              AIX     #1
              DBNZ    _TEMP0,_TLK_0
              ;
    _TLK_1:   ;
              RTS ! {}, {HX+A+SR}
  }
}

#undef  _TEMP0
#define _TEMP0 M:15

#pragma NO_EXIT
static void D_TKRes (_PARAM_BINARY_D_2) {
/* res := K */
  asm {
#ifdef __OPTIMZE_FOR_TIME__
#ifdef __HCS08__
              LDHX    K
              STHX    Res_K
              LDHX    K:2
              STHX    Res_K:2
              LDHX    K:4
              STHX    Res_K:4
              LDHX    K:6
              STHX    Res_K:6
#else
              LDHX    @K
              LDA     0,X
              STA     Res_K,X
              AIX     #1
              LDA     0,X
              STA     Res_K,X
              AIX     #1
              LDA     0,X
              STA     Res_K,X
              AIX     #1
              LDA     0,X
              STA     Res_K,X
              AIX     #1
              LDA     0,X
              STA     Res_K,X
              AIX     #1
              LDA     0,X
              STA     Res_K,X
              AIX     #1
              LDA     0,X
              STA     Res_K,X
              AIX     #1
              LDA     0,X
              STA     Res_K,X
              ;
#endif
#else
              LDA     #8
              STA     _TEMP0
              LDHX    @K
    _TLK_0:   ;
              LDA     0,X
              STA     Res_K,X
              AIX     #1
              DBNZ    _TEMP0,_TLK_0
              ;
#endif
    _TLK_1:   ;
              RTS ! {}, {HX+A+SR}
  }
}

#undef  _TEMP0
#define _TEMP0 res:0

#pragma NO_EXIT
static void D_XGKL (_PARAM_BINARY_D_2) {
/* K <=> L */
asm {
              LDA     Kspec
              LDX     Lspec
              STX     Kspec
              STA     Lspec
              LDA     Kexp:0
              LDX     Lexp:0
              STX     Kexp:0
              STA     Lexp:0
              LDA     Kexp:1
              LDX     Lexp:1
              STX     Kexp:1
              STA     Lexp:1
              ;
#ifdef __OPTIMZE_FOR_TIME__
              LDHX    @L
              LDA     0,X
              PSHA
              LDA     Kmant_Lmant,X
              STA     0,X
              PULA
              STA     Kmant_Lmant,X
              AIX     #1
              LDA     0,X
              PSHA
              LDA     Kmant_Lmant,X
              STA     0,X
              PULA
              STA     Kmant_Lmant,X
              AIX     #1
              LDA     0,X
              PSHA
              LDA     Kmant_Lmant,X
              STA     0,X
              PULA
              STA     Kmant_Lmant,X
              AIX     #1
              LDA     0,X
              PSHA
              LDA     Kmant_Lmant,X
              STA     0,X
              PULA
              STA     Kmant_Lmant,X
              AIX     #1
              LDA     0,X
              PSHA
              LDA     Kmant_Lmant,X
              STA     0,X
              PULA
              STA     Kmant_Lmant,X
              AIX     #1
              LDA     0,X
              PSHA
              LDA     Kmant_Lmant,X
              STA     0,X
              PULA
              STA     Kmant_Lmant,X
              AIX     #1
              LDA     0,X
              PSHA
              LDA     Kmant_Lmant,X
              STA     0,X
              PULA
              STA     Kmant_Lmant,X
              AIX     #1
              LDA     0,X
              PSHA
              LDA     Kmant_Lmant,X
              STA     0,X
              PULA
              STA     Kmant_Lmant,X
#else
              LDA     #8
              STA     _TEMP0
              LDHX    @L
    _TLK_0:   ;
              LDA     0,X
              PSHA
              LDA     Kmant_Lmant,X
              STA     0,X
              PULA
              STA     Kmant_Lmant,X
              AIX     #1
              DBNZ    _TEMP0,_TLK_0
#endif
              ;
              RTS ! {}, {HX+A+SR}
  }
}

#pragma NO_EXIT
static void D_CLRK (_PARAM_BINARY_D) {
/* K = 0 */
  asm {
              LDA      #(SPECIAL_ZERO)
              STA      Kspec
              RTS ! {}, {HX+A+SR}
  }
}

#pragma NO_EXIT
static void D_CLRK_Unary (_PARAM_UNARY_D) {
/* K = 0 */
  asm {
              LDA      #(SPECIAL_ZERO)
              STA      Kspec
              RTS ! {}, {HX+A+SR}
  }
}

#pragma NO_EXIT
static void D_NANK (_PARAM_BINARY_D) {
/* K = SPECIAL_NAN */
  asm {
              LDA      #(SPECIAL_NAN)
              STA      Kspec
              RTS ! {}, {HX+A+SR}
  }
}

#pragma NO_EXIT
static void D_NANK_Unary (_PARAM_UNARY_D) {
/* K = SPECIAL_NAN */
  asm {
              LDA      #(SPECIAL_NAN)
              STA      Kspec
              RTS ! {}, {HX+A+SR}
  }
}

#pragma NO_EXIT
static void D_INFK (_PARAM_BINARY_D) {
/* K = INV */
  asm {
              LDA      #(SPECIAL_INFINITY)
              STA      Kspec
              RTS ! {}, {HX+A+SR}
  }
}

#pragma NO_EXIT
static void D_INFK_Unary (_PARAM_UNARY_D) {
/* K = INV */
  asm {
              LDA      #(SPECIAL_INFINITY)
              STA      Kspec
              RTS ! {}, {HX+A+SR}
  }
}

#pragma NO_EXIT
static void _DUNPACK_k_to_K (_PARAM_BINARY_D) {
  asm {
               CLR     Kspec
               ;
               LDA     K:0
               LSLA
               ORA     K:1
               ORA     K:2
               ORA     K:3
               ORA     K:4
               ORA     K:5
               ORA     K:6
               ORA     K:7
               BNE     _GETK_5   ; // number != 0
               ;
               LDA     #SPECIAL_ZERO
               STA     Kspec     ; // SPECIAL_ZERO
               RTS ! {}, {HX+A+SR}
    _GETK_5:   ;
               LDA     K:0
               CLR     K:0
               LSLA
               BCC     _GETK_0
               COM     K:0
    _GETK_0:   ;
               LSRA
               LDX     K:1
               LSRA
               RORX
               LSRA
               RORX
               LSRA
               RORX
               LSRA
               RORX
               ;
               STX     Kexp:1
               STA     Kexp:0
               CMP     #0x07
               BNE     GETK_6
               CPX     #0xFF
               BNE     GETK_6
               ;
               DEC     Kspec
               LDA     K:1
               AND     #0x0F
               ORA     K:2
               ORA     K:3
               ORA     K:4
               ORA     K:5
               ORA     K:6
               ORA     K:7
               BNE     GETK_7     // SPECIAL_NAN
               DEC     Kspec      // SPECIAL_INFINITY
    GETK_7:    ;
               RTS ! {}, {HX+A+SR}
    GETK_6:    ;
               LDA     K:1
               AND     #0x0F
               ORA     #0x10
               STA     K:1
               ; // {
               ; //   A contains exponent.
               ; //   Kmant & Ksign are set.
               ; // }
               LDA     Kexp:0
               ORA     Kexp:1
               BNE     _GETK_2  ; // normalized number
               ;
               LDA     K:1
               AND     #0x0F
               STA     K:1
               LDA     Kmant:0
    _GETK_3:   ;
               DEC     Kexp:1
               LSL     Kmant:6
               ROL     Kmant:5
               ROL     Kmant:4
               ROL     Kmant:3
               ROL     Kmant:2
               ROL     Kmant:1
               ROLA
               BIT     #0x10
               BEQ     _GETK_3
               ;
               STA     Kmant:0
               INC     Kexp:1
               BEQ     _GETK_2
               COM     Kexp:0
    _GETK_2:   ; // { assert (K is normalized) }
               ;
               RTS ! {}, {HX+A+SR}
  } /* msa */
}

#pragma NO_EXIT
static void _DUNPACK_k_to_K_Unary (_PARAM_UNARY_D) {
  asm {
               CLR     Kspec
               ;
               LDA     K:0
               LSLA
               ORA     K:1
               ORA     K:2
               ORA     K:3
               ORA     K:4
               ORA     K:5
               ORA     K:6
               ORA     K:7
               BNE     _GETK_5   ; // number != 0
               ;
               LDA     #SPECIAL_ZERO
               STA     Kspec     ; // SPECIAL_ZERO
               RTS ! {}, {HX+A+SR}
    _GETK_5:   ;
               LDA     K:0
               CLR     K:0
               LSLA
               BCC     _GETK_0
               COM     K:0
    _GETK_0:   ;
               LSRA
               LDX     K:1
               LSRA
               RORX
               LSRA
               RORX
               LSRA
               RORX
               LSRA
               RORX
               ;
               STX     Kexp:1
               STA     Kexp:0
               CMP     #0x07
               BNE     GETK_6
               CPX     #0xFF
               BNE     GETK_6
               ;
               DEC     Kspec
               LDA     K:1
               AND     #0x0F
               ORA     K:2
               ORA     K:3
               ORA     K:4
               ORA     K:5
               ORA     K:6
               ORA     K:7
               BNE     GETK_7     // SPECIAL_NAN
               DEC     Kspec      // SPECIAL_INFINITY
    GETK_7:    ;
               RTS ! {}, {HX+A+SR}
    GETK_6:    ;
               LDA     K:1
               AND     #0x0F
               ORA     #0x10
               STA     K:1
               ; // {
               ; //   A contains exponent.
               ; //   Kmant & Ksign are set.
               ; // }
               LDA     Kexp:0
               ORA     Kexp:1
               BNE     _GETK_2  ; // normalized number
               ;
               LDA     K:1
               AND     #0x0F
               STA     K:1
               LDA     Kmant:0
    _GETK_3:   ;
               DEC     Kexp:1
               LSL     Kmant:6
               ROL     Kmant:5
               ROL     Kmant:4
               ROL     Kmant:3
               ROL     Kmant:2
               ROL     Kmant:1
               ROLA
               BIT     #0x10
               BEQ     _GETK_3
               ;
               STA     Kmant:0
               INC     Kexp:1
               BEQ     _GETK_2
               COM     Kexp:0
    _GETK_2:   ; // { assert (K is normalized) }
               ;
               RTS ! {}, {HX+A+SR}
  } /* msa */
}

#pragma NO_EXIT
static void _DUNPACK_j_to_L (_PARAM_BINARY_D) {
  asm {
               CLR     Lspec
               ;
               LDA     L:0
               LSLA
               ORA     L:1
               ORA     L:2
               ORA     L:3
               ORA     L:4
               ORA     L:5
               ORA     L:6
               ORA     L:7
               BNE     _GETL_5   ; // number != 0
               ;
               LDA     #SPECIAL_ZERO
               STA     Lspec     ; // SPECIAL_ZERO
               RTS ! {}, {HX+A+SR}
    _GETL_5:   ;
               LDA     L:0
               CLR     L:0
               LSLA
               BCC     _GETL_0
               COM     L:0
    _GETL_0:   ;
               LSRA
               LDX     L:1
               LSRA
               RORX
               LSRA
               RORX
               LSRA
               RORX
               LSRA
               RORX
               ;
               STX     Lexp:1
               STA     Lexp:0
               CMP     #0x07
               BNE     GETL_6
               CPX     #0xFF
               BNE     GETL_6
               ;
               DEC     Lspec
               LDA     L:1
               AND     #0x0F
               ORA     L:2
               ORA     L:3
               ORA     L:4
               ORA     L:5
               ORA     L:6
               ORA     L:7
               BNE     GETL_7     // SPECIAL_NAN
               DEC     Lspec      // SPECIAL_INFINITY
    GETL_7:    ;
               RTS ! {}, {HX+A+SR}
    GETL_6:    ;
               LDA     L:1
               AND     #0x0F
               ORA     #0x10
               STA     L:1
               ; // {
               ; //   A contains exponent.
               ; //   Lmant & Lsign are set.
               ; // }
               LDA     Lexp:0
               ORA     Lexp:1
               BNE     _GETL_2  ; // normalized number
               ;
               LDA     L:1
               AND     #0x0F
               STA     L:1
               LDA     Lmant:0
    _GETL_3:   ;
               DEC     Lexp:1
               LSL     Lmant:6
               ROL     Lmant:5
               ROL     Lmant:4
               ROL     Lmant:3
               ROL     Lmant:2
               ROL     Lmant:1
               ROLA
               BIT     #0x10
               BEQ     _GETL_3
               ;
               STA     Lmant:0
               INC     Lexp:1
               BEQ     _GETL_2
               COM     Lexp:0
    _GETL_2:   ; // { assert (L is normalized) }
               ;
               RTS ! {}, {HX+A+SR}
  } /* msa */
}

#pragma NO_EXIT
static void _DPACK_K_to_k (_PARAM_BINARY_D) {
  asm {
               TSX
               LDA     Kspec
               BEQ     PUTK_1
               ;
               BMI     PUTK_2
               ; // {
               ; //   number == 0.
               ; // }
               CLRA
               STA     K:0
               STA     K:1
    PUTK_5:    ;
               TSX
               STA     K:2
               STA     K:3
               STA     K:4
               STA     K:5
               STA     K:6
               STA     K:7
               BRA     PUTK_4
    PUTK_2:    ;
               CMP     #(SPECIAL_NAN)
               BEQ     PUTK_3
               LDA     #0xF0
               STA     K:1
               LDA     #0x7F
               ORA     K:0     ; // + or - INF
               STA     K:0
               CLRA
               BRA     PUTK_5
    PUTK_3:    ;
               LDA     #0x7F
               STA     K:0
               LDA     #0xFF
               STA     K:1
               BRA     PUTK_5
    PUTK_1:    ;
               ; // {
               ; //   number not special
               ; // }
               LDA     Kexp:1
               ORA     Kexp:0
               BNE     PUTK_0       ; // normalized
               ; // {
               ; //   non-normalized number.
               ; // }
               LSR     Kmant:0
               ROR     Kmant:1
               ROR     Kmant:2
               ROR     Kmant:3
               ROR     Kmant:4
               ROR     Kmant:5
               ROR     Kmant:6
               LSL     Ksign
               CLR     K:0
               ROR     K:0
               BRA     PUTK_4
    PUTK_0:    ;
               LDA     K:1
               AND     #0x0F
               STA     K:1
               LDA     Kexp:1
               LDX     Kexp:0
               ASLA
               ROLX
               ASLA
               ROLX
               ASLA
               ROLX
               ASLA
               ROLX
               LSLX
               LSL     Ksign
               RORX
               STX     K:0
               ORA     K:1
               STA     K:1
    PUTK_4:
               JSR     D_TKRes
               _EXIT_BINARY_D
               RTS ! {}, {HX+A+SR}
  } /* msa */
}

#pragma NO_EXIT
static void _DPACK_K_to_k_Unary (_PARAM_UNARY_D) {
  asm {
               TSX
               LDA     Kspec
               BEQ     PUTK_1
               ;
               BMI     PUTK_2
               ; // {
               ; //   number == 0.
               ; // }
               CLRA
               STA     K:0
               STA     K:1
    PUTK_5:    ;
               TSX
               STA     K:2
               STA     K:3
               STA     K:4
               STA     K:5
               STA     K:6
               STA     K:7
               BRA     PUTK_4
    PUTK_2:    ;
               CMP     #(SPECIAL_NAN)
               BEQ     PUTK_3
               LDA     #0x80
               STA     K:1
               COMA            ; // 0x7F
               ORA     K:0     ; // + or - INF
               STA     K:0
               CLRA
               BRA     PUTK_5
    PUTK_3:    ;
               LDA     #0x7F
               STA     K:0
               LDA     #0xFF
               STA     K:1
               BRA     PUTK_5
    PUTK_1:    ;
               ; // {
               ; //   number not special
               ; // }
               LDA     Kexp:1
               ORA     Kexp:0
               BNE     PUTK_0       ; // normalized
               ; // {
               ; //   non-normalized number.
               ; // }
               LSR     Kmant:0
               ROR     Kmant:1
               ROR     Kmant:2
               ROR     Kmant:3
               ROR     Kmant:4
               ROR     Kmant:5
               ROR     Kmant:6
               LSL     Ksign
               CLR     K:0
               ROR     K:0
               BRA     PUTK_4
    PUTK_0:    ;
               LDA     K:1
               AND     #0x0F
               STA     K:1
               LDA     Kexp:1
               LDX     Kexp:0
               ASLA
               ROLX
               ASLA
               ROLX
               ASLA
               ROLX
               ASLA
               ROLX
               LSLX
               LSL     Ksign
               RORX
               STX     K:0
               ORA     K:1
               STA     K:1
    PUTK_4:
               _EXIT_UNARY_D
               RTS ! {}, {HX+A+SR}
  } /* msa */
}

#pragma NO_EXIT
static void D_NORMK_Unary (_PARAM_UNARY_D) {
  asm {
              TST     Kspec
              BEQ     NORMK_11
              ;
              RTS ! {}, {HX+A+SR}
    NORMK_11: ;
              LDA     Kmant:0
              ORA     Kmant:1
              ORA     Kmant:2
              ORA     Kmant:3
              ORA     Kmant:4
              ORA     Kmant:5
              ORA     Kmant:6
              BNE     NORMK_10
              JMP     D_CLRK_Unary ! {}, {HX+A+SR}
    NORMK_10: ;
              LDA     Kexp:0
              BMI     NORMK_0    ; // underflow ->denormalize
              CMP     #(0x08)
              BLO     NORMK_1    ; // in range
              ; // {
              ; //   OVERFLOW !
              ; // }
              JMP     D_INFK_Unary ! {}, {HX+A+SR}
    NORMK_1:  ;
              LDA     Kmant:0
              BRA     NORMK_4
    NORMK_3:  ;
              TST     Kexp:1
              BNE     NORMK_6
              TST     Kexp:0
              BEQ     NORMK_2
              DEC     Kexp:0
    NORMK_6:  ;
              DEC     Kexp:1
              LSL     Kmant:6
              ROL     Kmant:5
              ROL     Kmant:4
              ROL     Kmant:3
              ROL     Kmant:2
              ROL     Kmant:1
              ROLA
    NORMK_4:  ;
              BIT     #0x10
              BEQ     NORMK_3
    NORMK_2:  ;
              STA     Kmant:0
              RTS ! {}, {HX+A+SR}
              ;
    NORMK_0:  ; // { 
              ; //   assert (expo < 0);
              ; //   denormalize!
              ; // }
              CMP     #0xFF
              BNE     NORMK_7
              ;
              LDA     Kexp:1
              NEGA
    NORMK_5:  ;
              LSR     Kmant:0
              ROR     Kmant:1
              ROR     Kmant:2
              ROR     Kmant:3
              ROR     Kmant:4
              ROR     Kmant:5
              ROR     Kmant:6
              DBNZA   NORMK_5
              ;
     NORMK_8: ; // { 
              ; //   return-non-normalized number
              ; // }
              CLR     Kexp:1
              CLR     Kexp:0
     NORMK_9: ;
              RTS ! {}, {HX+A+SR}
              ;
     NORMK_7: ;
              JMP     D_CLRK_Unary ! {}, {HX+A+SR}
  }
}

#pragma NO_EXIT
static void D_NORMK (_PARAM_BINARY_D) {
  asm {
              TST     Kspec
              BEQ     NORMK_11
              ;
              RTS ! {}, {HX+A+SR}
    NORMK_11: ;
              LDA     Kmant:0
              ORA     Kmant:1
              ORA     Kmant:2
              ORA     Kmant:3
              ORA     Kmant:4
              ORA     Kmant:5
              ORA     Kmant:6
              BNE     NORMK_10
              JMP     D_CLRK ! {}, {HX+A+SR}
    NORMK_10: ;
              LDA     Kexp:0
              BMI     NORMK_0    ; // underflow ->denormalize
              CMP     #(0x08)
              BLO     NORMK_1    ; // in range
              ; // {
              ; //   OVERFLOW !
              ; // }
              JMP     D_INFK ! {}, {HX+A+SR}
    NORMK_1:  ;
              LDA     Kmant:0
              BRA     NORMK_4
    NORMK_3:  ;
              TST     Kexp:1
              BNE     NORMK_6
              TST     Kexp:0
              BEQ     NORMK_2
              DEC     Kexp:0
    NORMK_6:  ;
              DEC     Kexp:1
              LSL     Kmant:6
              ROL     Kmant:5
              ROL     Kmant:4
              ROL     Kmant:3
              ROL     Kmant:2
              ROL     Kmant:1
              ROLA
    NORMK_4:  ;
              BIT     #0x10
              BEQ     NORMK_3
    NORMK_2:  ;
              STA     Kmant:0
              RTS ! {}, {HX+A+SR}
              ;
    NORMK_0:  ; // { 
              ; //   assert (expo < 0);
              ; //   denormalize!
              ; // }
              CMP     #0xFF
              BNE     NORMK_7
              ;
              LDA     Kexp:1
              NEGA
    NORMK_5:  ;
              LSR     Kmant:0
              ROR     Kmant:1
              ROR     Kmant:2
              ROR     Kmant:3
              ROR     Kmant:4
              ROR     Kmant:5
              ROR     Kmant:6
              DBNZA   NORMK_5
              ;
     NORMK_8: ; // { 
              ; //   return-non-normalized number
              ; // }
              CLR     Kexp:1
              CLR     Kexp:0
     NORMK_9: ;
              RTS ! {}, {HX+A+SR}
              ;
     NORMK_7: ;
              JMP     D_CLRK ! {}, {HX+A+SR}
  }
}


/* IEEE64 operations */

#undef  _TEMP0
#define _TEMP0 res:0

#pragma NO_EXIT
static void _DADD_K_is_K_plus_L (_PARAM_BINARY_D) {
/* K = K + L */
  asm {
              LDA     Kspec
              ORA     Lspec
              BEQ     ADDKL_Normal
              ; // {
              ; //   one number is special
              ; // }
              BPL     ADDKL_Zero     ; // SPECIAL_ZERO
              ; // NAN + x or x + NAN or INF + x or x + INF or INF + INF
              INCA
              BEQ     ADDKL_RET_NAN    ; // return K   : NAN + x or x + NAN -> NAN
              ;
              LDA     Lspec
              EOR     Kspec
              BPL     ADDKL_BothInf  ; // INF + INF case
              LDA     Kspec
              CMP     #(SPECIAL_INFINITY)
              BNE     ADDKL_2        ; // return L   : whatever + (+/-)INF -> (+/-)INF 
              RTS ! {}, {HX+A+SR}    ; // return K   : (+/-)INF + whatever -> (+/-)INF
ADDKL_Zero:                          ; // 0 + x or x + 0 (or 0 + 0) (assuming x is not special)
              LDA     Kspec
              BGT     ADDKL_2        ; // return L // 0 + L
              RTS ! {}, {HX+A+SR}    ; // return K // K + 0       
ADDKL_BothInf:                       ; // INF + INF case. compare signs. If != -> NAN
              LDA     K:0
              EOR     L:0
              BPL     ADDKL_3        ; // return K
ADDKL_RET_NAN:
              JMP     D_NANK ! {}, {HX+A+SR} ; // return NAN : (+/-)INF + (-/+)INF -> NAN

ADDKL_Normal:  ;// L and K are not special (means not 0,NAN,inf)
              LDA     Kexp:1
              SUB     Lexp:1
              TAX
              LDA     Kexp:0
              SBC     Lexp:0
              BCS     ADDKL_0        ; // K:exp < L:exp
              TXA
              BEQ     ADDKL_1        ; // K:exp >= L:exp
              TST     Kexp:0         ; // R2780 negative Kexp not handled!
              BLT     ADDKL_0        ; // K:exp < L:exp
    ADDKL_3:  ;
              RTS ! {}, {HX+A+SR}    ; // return K
    ADDKL_2:  ;
              JSR     D_TLK ! {}, {HX+A+SR}     ; // return L
              RTS ! {}, {HX+A+SR}
    ADDKL_0:  ;
              COMA
              TXA
              BNE     ADDKL_2   ; // return L
              ; // { K:exp < L:exp ==> swap K,L }
              NEGA
              BEQ     ADDKL_2
              ;
              STA     M:0
              JSR     D_XGKL ! {}, {HX+A+SR}
              LDA     M:0
              ;
              ; // NOTE: One may not push-pull registers,
              ; //       since D_XGKL expects ... K, L ... at
              ; //       fixed offset !
              ;
    ADDKL_1:  ; // {
              ; //   assert (K:exp >= L:exp)
              ; //   A == K:exp - L:exp
              ; // }
              CMP     #(D_WIDTH)
              BHS     ADDKL_3   ; // return K
              TSTA
              BEQ     ADDKL_4
              ;
    ADDKL_5:  LSR     Lmant:0
              ROR     Lmant:1
              ROR     Lmant:2
              ROR     Lmant:3
              ROR     Lmant:4
              ROR     Lmant:5
              ROR     Lmant:6
              DBNZA   ADDKL_5
              ;
    ADDKL_4:  ; // {
              ; //   K, L mantissas with exponent == K:exp
              ; // }
              LDA     Ksign
              CMP     Lsign
              BEQ     ADDKL_6     ; // signs are the same
              ; // {
              ; //   assert (Ksign != Lsign)
              ; //   K, L mantissas with exponent == K:exp
              ; // }
              LDHX    @Kmant:6
              LDA     #(D_MBYTES)
              STA     _TEMP0
              CLC
    ADDKL_7:  ;
              LDA     0,X
              SBC     Lmant_Kmant,X
              STA     0,X
              AIX     #-1
              DBNZ    _TEMP0,ADDKL_7
              ;
              BCC     ADDKL_8     ; // |K| was greater than |L|
              ; // {
              ; //   assert (K.mant == K.mant - L.mant);
              ; // }
              COM     Ksign
              COM     Kmant:0
              COM     Kmant:1
              COM     Kmant:2
              COM     Kmant:3
              COM     Kmant:4
              COM     Kmant:5
              NEG     Kmant:6
              BNE     ADDKL_8
              INC     Kmant:5
              BNE     ADDKL_8
              INC     Kmant:4
              BNE     ADDKL_8
              INC     Kmant:3
              BNE     ADDKL_8
              INC     Kmant:2
              BNE     ADDKL_8
              INC     Kmant:1
              BNE     ADDKL_8
              INC     Kmant:0

    ADDKL_8:  ; // {
              ; //   informal: K contains the correct result. 
              ; //             But K.mant may have leading zeros.
              ; // }
              JMP     D_NORMK ! {}, {HX+A+SR}   ; // normalize and return
    ADDKL_6:  ; // {
              ; //   assert (Ksign == Lsign)
              ; //   K, L mantissas with exponent == K:exp
              ; // }
              LDHX    @Kmant:6
              LDA     #(D_MBYTES)
              STA     _TEMP0
              CLC
    ADDKL_10: ;
              LDA     0,X
              ADC     Lmant_Kmant,X
              STA     0,X
              AIX     #-1
              DBNZ    _TEMP0,ADDKL_10
              ;
              LDA     Kmant:0
              BIT     #0xE0
              BEQ     ADDKL_11     ; // no overflow
              LSR     Kmant:0
              ROR     Kmant:1
              ROR     Kmant:2
              ROR     Kmant:3
              ROR     Kmant:4
              ROR     Kmant:5
              ROR     Kmant:6
              INC     Kexp:1
              BNE     ADDKL_11
              INC     Kexp:0
    ADDKL_11: ; // {
              ; //   assert (result in K)
              ; // }
              JMP     D_NORMK ! {}, {HX+A+SR}   ; // normalize and return 
  } /* asm */
}

#undef  _TEMP0
#undef  _TEMP1
#define _TEMP0   res:0
#define _TEMP1   res:1

#pragma NO_EXIT
static void _DMUL_K_is_K_times_L (_PARAM_BINARY_D) {
/* K = K * L */
  asm {
                LDA     Kspec
                ORA     Lspec
                BEQ     MULKL_12
                ; // {
                ; //   one number is special
                ; // }
                INCA
                BEQ     MULKL_RET_NAN    ; // return K   : NAN + x or x + NAN -> NAN
                BMI     MULKL_ONE_INF
                ; // None is NAN or INF, but one is special -> 0*0, 0*x or X*0 cases. All result in 0

                JMP     D_CLRK ! {}, {HX+A+SR} ; // return 0 for 0 * x
    MULKL_ONE_INF:
                LDA     Kspec
                EOR     Lspec
                CBEQA   #(SPECIAL_INFINITY^SPECIAL_ZERO), MULKL_RET_NAN ; // 0 * INF or INF * 0 case                
                BRA     MULKL_INF_RES ; // INF * INF, INF * X or X * INF
    MULKL_RET_NAN:
                JMP     D_NANK ! {}, {HX+A+SR} ; // return NAN for INF * 0
    MULKL_NOT_ZERO:   ; // L != 0 -> L is INF or normal
                TSTA
                BEQ     MULKL_INF_RES   ; // branch if L is normal
                LDA     Kspec  
                BGT     MULKL_RET_NAN ; // SPECIAL_ZERO)    ; return NAN for 0 * INF
    MULKL_INF_RES:             
                LDA      Lsign ; // INF * x or x * INF or INF * x
                EOR      Ksign
                STA      Ksign
                JMP      D_INFK ! {}, {HX+A+SR} ; // return NAN for INF * 0

    MULKL_12:   ;
                LDA      Lsign
                EOR      Ksign
                STA      Lsign
                ; // { sign done }
                LDA      Lexp:1
                ADD      Kexp:1
                STA      Lexp:1
                LDA      Lexp:0
                ADC      Kexp:0
                PSHA
                LDA      Lexp:1
                SUB      #(D_EXPLO + 3)
                STA      Lexp:1
                PULA
                SBC      #(D_EXPHI)
                STA      Lexp:0
                ; // { exp done }
                LDA      #(D_MBYTES)
                STA      _TEMP0
                LDHX     @M
    _MULKL_5:   ;
                CLR      0,X
                LDA      Kmant_M,X
                STA      (D_MBYTES),X
                CLR      (Lmant_M + D_MBYTES),X
                AIX      #1
                DBNZ     _TEMP0,_MULKL_5
                ;
                LDA      #(D_MBITS)
                STA      _TEMP0
                ;
    MULKL_0:    LDHX     @Lmant:(2*D_MBYTES)
                LDA      #(2*D_MBYTES)
                CLC
                ;
    MULKL_1:    AIX      #-1
                ROL      0,X
                DBNZA    MULKL_1
                ;
                BCC      MULKL_2
                LDHX     @Lmant:(2*D_MBYTES)
                LDA      #(2*D_MBYTES)
                STA      _TEMP1
                CLC
                ;
    MULKL_3:    AIX      #-1
                LDA      0,X
                ADC      M_Lmant,X
                STA      0,X
                DBNZ     _TEMP1,MULKL_3
    MULKL_2:    ;
                DBNZ     _TEMP0,MULKL_0
                LDA      #3          ; // take additional 3 bits of result into L
                STA      _TEMP0
    MULKL_6:
                LDA      #(2*D_MBYTES)
                LDHX     @Lmant:(2*D_MBYTES)
                CLC
    MULKL_7:
                AIX      #-1
                ROL      0,X
                DBNZA    MULKL_7
                DBNZ      _TEMP0,MULKL_6
                ;
                ; // (result in L)
                ;
                JSR      D_TLK ! {}, {HX+A+SR}
                JMP      D_NORMK ! {}, {HX+A+SR}
  }
} /** D_MULKL **/

#undef _TEMP0
#undef _TEMP1
#define _TEMP0   res:0
#define _TEMP1   res:1

#pragma NO_EXIT
static void _DDIV_K_is_L_div_K (_PARAM_BINARY_D) {
/* K = K / L */
  asm {
                LDA      Kspec
                ORA      Lspec
                BEQ      DIVLK_12
                ; // {
                ; //   one number is special
                ; // }
                INCA
                BEQ      DIVLK_RET_NAN
                LDA      Kspec
                BGT      DIVLK_RET_NAN       ; // SPECIAL_ZERO: branch for "L / 0" case, return NAN
                LDA      Lspec
                BGT      DIVLK_13            ; // SPECIAL_ZERO: branch for "0 / x" case, return 0
                CMP      #(SPECIAL_INFINITY) ; // INF / (X|INF) case?
                BNE      DIVLK_13            ; // branch for "x / INF" case. return 0
                                             ; // INF / X or INF / INF case
                LDA      Kspec
                CMP      #(SPECIAL_INFINITY) ; // INF / INF or x / INF?
                BNE      DIVLK_16            ; // branch for "INF / x" case. return INF with sign(INF^x)
                                             ; // for "INF/INF" case, return NAN 
    DIVLK_RET_NAN:   ;
                JMP      D_NANK ! {}, {HX+A+SR}
    DIVLK_16:                                ; // INF / x case
                LDA      Lsign
                EOR      Ksign
                STA      Ksign               ; // sign(res) = sign(K) ^ sign(L)
                JMP      D_INFK ! {}, {HX+A+SR}

    DIVLK_13:   ;
                JMP      D_CLRK ! {}, {HX+A+SR}
    DIVLK_12:   ;
                TST      Kmant:0
                BNE      DIVLK_5
                ; // { divisor == 0 }
                JMP      D_INFK ! {}, {HX+A+SR}
    DIVLK_5:    ;
                LDA      Lsign
                EOR      Ksign
                STA      Ksign
                ; // { sign done }
                LDA      Lexp:1
                SUB      Kexp:1
                STA      Kexp:1
                LDA      Lexp:0
                SBC      Kexp:0
                PSHA
                LDA      Kexp:1
                ADD      #(D_EXPLO + 4)
                ;
                ; // NOTE: add more, since the virtual
                ; //       fractional point will be at bit <52>.<51>
                ;
                STA      Kexp:1
                PULA
                ADC      #(D_EXPHI)
                STA      Kexp:0
                ; // { exp done }
                CLR      M:0
                CLR      M:1
                CLR      M:2
                CLR      M:3
                CLR      M:4
                CLR      M:5
                CLR      M:6
                ;
                ; // NOTE: all other bits of M are shifted away.
                ;
                LDA      #(D_WIDTH)
                STA      _TEMP0
                CLC
    DIVLK_0:    ;
                ; // { carry set if > 0xFFFF }
                ;
                BCS      DIVLK_1
                LDHX     @Lmant
                LDA      #(D_MBYTES)
                STA      _TEMP1
    DIVLK_6:    ;
                LDA      Kmant_Lmant,X
                CMP      0,X
                BNE      DIVLK_2
                AIX      #1
                DEC      _TEMP1
                BNE      DIVLK_6
    DIVLK_2:    ;
                BHI      DIVLK_3   ; // carry cleared !
    DIVLK_1:    ;
                LDHX     @Lmant:(D_MBYTES)
                LDA      #(D_MBYTES)
                STA      _TEMP1
                CLC
    DIVLK_4:    ;
                AIX      #-1
                LDA      0,X
                SBC      Kmant_Lmant,X
                STA      0,X
                DBNZ      _TEMP1,DIVLK_4
                SEC
    DIVLK_3:    ; // {
                ; //   Carry is set if q += 1
                ; // }
                ROL      M:6
                ROL      M:5
                ROL      M:4
                ROL      M:3
                ROL      M:2
                ROL      M:1
                ROL      M:0
                LSL      Lmant:6
                ROL      Lmant:5
                ROL      Lmant:4
                ROL      Lmant:3
                ROL      Lmant:2
                ROL      Lmant:1
                ROL      Lmant:0
                DBNZ     _TEMP0,DIVLK_0
                ;
                LDHX     @M
                LDA      #(D_MBYTES)
                STA      _TEMP1
    _DIVLK_8:   ;
                LDA      0,X
                STA      Kmant_M,X
                AIX      #1
                DBNZ     _TEMP1,_DIVLK_8
                ;
                JMP      D_NORMK ! {}, {HX+A+SR}
  }
}

#pragma NO_EXIT
static void _DCMP_k_rel_j (_PARAM_BINARY_L64) {
/* unsigned CC = k <rel> j */
  asm {
                LDHX    @j
                LDA     #8
                STA     res:0
                LDA     k:0
                ORA     j:0
                BPL     _CMP_0     ; // both positive
    _CMP_1:     ;
                LDA     0,X
                CMP     8,X
                BNE     _CMP_3
                AIX     #1
                DBNZ    res:0, _CMP_1
                ;
                BRA     _CMP_3
    _CMP_0:     ;
    _CMP_2:     ;
                LDA     8,X
                CMP     0,X
                BNE     _CMP_3
                AIX     #1
                DBNZ    res:0, _CMP_2
    _CMP_3:     ;
                AIS     #20
                PULA
                PULH
                PULX
                AIS     #8
                _SRET
                JMP     ,X
  }
}

#undef _TEMP0
#define _TEMP0 temp

#pragma NO_EXIT
static void _D_FLOATK (_PARAM_UNARY_D) {
/* K = (double) K */
  asm {
                LDA      K:4
                ORA      K:5
                ORA      K:6
                ORA      K:7
                BNE      _FLOAT_0
                JMP      D_CLRK_Unary ! {}, {HX+A+SR}
    _FLOAT_0:   ;
#ifdef __HCS08__
                LDHX     K:4
                STHX     K:2
                LDHX     K:6
                STHX     K:4
#else
                LDA      K:4
                STA      K:2
                LDA      K:5
                STA      K:3
                LDA      K:6
                STA      K:4
                LDA      K:7
                STA      K:5
#endif
                LDA      #(5*8)
                STA      _TEMP0
                CLRA
    _FLOAT_1:   ;
                DEC      _TEMP0
                LSL      K:5
                ROL      K:4
                ROL      K:3
                ROL      K:2
                ROLA
                BIT      #0x10
                BEQ      _FLOAT_1
    _FLOAT_2:   ;
                STA      K:1
                CLR      K:0
                CLR      K:6
                CLR      K:7
                LDA      _TEMP0
                ADD      #(D_EXPLO)
                STA      Kexp:1
                CLRA
                ADC      #(D_EXPHI)
                STA      Kexp:0
                CLR      Kspec
                RTS ! {}, {HX+A+SR}
  }
}

#pragma NO_EXIT
static void _D_LONGK (_PARAM_UNARY_D) {
/* K = (long) K */
  asm {
                LDA      Kspec
                BEQ      LONGK_12
                ;
                CMP      #(SPECIAL_ZERO)
                BNE      LONGK_13
                ;
    LONGK_0:    ;
                CLR      K:4
    LONGK_14:   ;
                CLR      K:5
                CLR      K:6
                CLR      K:7
                BRA      LONGK_5
    LONGK_13:   ;
                LDA      #0x80
                STA      K:4
                BRA      LONGK_14
    LONGK_12:   ;
                LDA      Kexp:1
                SUB      #(D_EXPLO)
                PSHA
                LDA      Kexp:0
                SBC      #(D_EXPHI)
                PULA
                BCS      LONGK_0    ; // number < 1
                NEGA
                ADD      #(40)
                BCC      LONGK_0    ; // number too large
                ;
    LONGK_2:    ;
                LSR      K:1
                ROR      K:2
                ROR      K:3
                ROR      K:4
                ROR      K:5
                DBNZA    LONGK_2
                ;
#ifdef __HCS08__
                LDHX     K:4
                STHX     K:6
                LDHX     K:2
                STHX     K:4
#else
                LDA      K:5
                STA      K:7
                LDA      K:4
                STA      K:6
                LDA      K:3
                STA      K:5
                LDA      K:2
                STA      K:4
#endif
                ;
                TST      Ksign
                BEQ      LONGK_5     ; // number positive
                LDHX     @K:4
                JSR      _NEG_L_HX
                ;
    LONGK_5:
                _EXIT_UNARY_D4
                RTS ! {}, {HX+A+SR}
                ;
  }
}

#pragma NO_EXIT
static void _D_FTODK (_PARAM_UNARY_D) {
/* K = (double) K */
  asm {
                CLR      Kspec
                CLR      Ksign
                ASL      K:5
                LDA      K:4
                ROLA
                BCC      _FTOD_0
                COM      Ksign
    _FTOD_0:    ;
                TSTA
                SEC
                BNE      _FTOD_1
                INCA
                CLC
    _FTOD_1:    ;
                ROR      K:5
                CLR      K:1
                CLR      K:2
                CLR      K:3
                CLR      K:4
                ADD      #(D_EXPLO - 127 + 33)
                STA      Kexp:1
                CLRA
                ADC      #(D_EXPHI)
                STA      Kexp:0
                JMP      D_NORMK_Unary ! {}, {HX+A+SR}
  }
}

#undef  _TEMP0
#define _TEMP0 temp

#pragma NO_EXIT
static void _D_DTOFK (_PARAM_UNARY_D) {
/* K = (float) K */
  asm {
                LDA      Kspec
                BMI      _DTOF_2    ; // set MAX
                BNE      _DTOF_3    ; // set ZERO
                ;
    _DTOF_12:   ;
                LDA      #3
                STA      _TEMP0
    _DTOF_0:    ;
                ROL      K:4
                ROL      K:3
                ROL      K:2
                ROL      K:1
                DBNZ     _TEMP0, _DTOF_0
                ;
                LDA      Kexp:1
                SUB      #(D_EXPLO + 4 - 127)
                PSHA
                LDA      Kexp:0
                SBC      #(D_EXPHI)
                BCS      _DTOF_1       ; // underflow --> denormalize
                PULA
                BNE      _DTOF_2       ; // overflow  --> max!
                TSTA
                BEQ      _DTOF_6       ; // slight underflow
    _DTOF_7:    ;
                ; // {
                ; //   A == exponent, is O.K.
                ; // }
                PSHA
                LDA      K:3
                STA      K:7
                LDA      K:2
                STA      K:6
                LDA      K:1
                ASLA
                STA      K:5
                LSL      Ksign
                PULA
                RORA
                STA      K:4
                ROR      K:5
                BRA      _DTOF_8
                ;
    _DTOF_1:    ;
                CMP      #0xFF
                PULA
                BNE      _DTOF_3    ; // --> set zero
                CMP      #(-23)
                BLS      _DTOF_3
                NEGA
    _DTOF_6:    ;
                INCA
    _DTOF_4:    ;
                LSR      K:1
                ROR      K:2
                ROR      K:3
                DBNZA    _DTOF_4
                ;
                BRA      _DTOF_7    ; // set value A == exp == 0
                ;
    _DTOF_2:    ; // {
                ; //   set max value for float.
                ; // }
                LDA      Ksign
                ORA      #0x7F
                STA      K:4
                LDA      #0x80
                STA      K:5
                CLR      K:6
                CLR      K:7
                BRA      _DTOF_8
                ;
    _DTOF_3:    ; // {
                ; //   set val == 0.
                ; // }
                CLR      K:4
                CLR      K:5
                CLR      K:6
                CLR      K:7
    _DTOF_8:
                _EXIT_UNARY_D4
                RTS ! {}, {HX+A+SR}
  }
}


#pragma NO_EXIT
static void _DADD_Common(void) {
  asm {
                JSR _DUNPACK_k_to_K;
                JSR _DUNPACK_j_to_L;
                JSR _DADD_K_is_K_plus_L;
                _SRET
                JSR _DPACK_K_to_k;     /* returns itself */
  }
}

#pragma NO_EXIT
static void _DSUB_Common(void) {
  asm {
                TSX
                LDA  22,X
                EOR  #0x80
                STA  22,X
          is_zero:
                _SRET
                JMP _DADD_Common ! {}, {HX+A+SR}
  }
}

#pragma NO_EXIT
static void _DMUL_Common(void) {
  asm {
                JSR _DUNPACK_k_to_K;
                JSR _DUNPACK_j_to_L;
                JSR _DMUL_K_is_K_times_L;
                _SRET
                JSR _DPACK_K_to_k;    /* returns itself */
  }
}


#pragma NO_EXIT
static void _DDIV_Common(void) {
  asm {
                JSR _DUNPACK_k_to_K;
                JSR _DUNPACK_j_to_L;
                JSR _DDIV_K_is_L_div_K;
                _SRET
                JSR _DPACK_K_to_k;    /* returns itself */
  }
}


/* IEEE64 interface routines */

#pragma NO_EXIT
void _DADD(void) {
/* DESC:    adds two floating point (IEEE64) values
   IN:      &f1: on stack, &f2 in HX
   OUT:     result on top of stack
   WRITTEN: none */
  asm {
    _ENTER_BINARY_D();
    JMP _DADD_Common
  }
}

#pragma NO_EXIT
void _DADD_RC(void) {           /* RC means right operand will follow in code stream */
/* DESC:    IEEE64 addition f1 + f2 (f2 constant)
   IN:      H:X @f1, 0,RA f2
   OUT:     result on top of stack
   WRITTEN: none */
  asm {
    _ENTER_BINARY_D_RC();
    _SRET
    JMP _DADD_Common
  }
}

#pragma NO_EXIT
void _DSUB(void) {
/* DESC:    subtracts two floating point (IEEE64) values (f1 - f2)
   IN:      &f2: on stack, &f1 in HX
   OUT:     result on top of stack
   WRITTEN: none */
  asm {
    _ENTER_BINARY_D();
    JMP _DSUB_Common
  }
}

#pragma NO_EXIT
void _DSUB_RC(void) {           /* RC means right operand will follow in code stream */
/* DESC:    IEEE64 subtraction f1 - f2 (f2 constant)
   IN:      H:X @f1, 0,RA f2
   OUT:     result on top of stack
   WRITTEN: none */
  asm {
    _ENTER_BINARY_D_RC();
    _SRET
    JMP _DSUB_Common
  }
}

#pragma NO_EXIT
void _DSUB_LC(void) {           /* LC means left operand will follow in code stream */
/* DESC:    IEEE64 subtraction f1 - f2 (f1 constant)
   IN:      H:X @f2, 0,RA f1
   OUT:     result on top of stack
   WRITTEN: none */
  asm {
    _ENTER_BINARY_D_LC();
    _SRET
    JMP _DSUB_Common
  }
}

#pragma NO_EXIT
void _DMUL(void) {
/* DESC:    multiplies two floating point (IEEE64) values (f1 * f2)
   IN:      &f1: on stack, &f2 in HX
   OUT:     result on top of stack
   WRITTEN: none */
  asm {
    _ENTER_BINARY_D();
    JMP _DMUL_Common
  }
}

#pragma NO_EXIT
void _DMUL_RC(void) {           /* RC means right operand will follow in code stream */
/* DESC:    IEEE64 multiplication f1 * f2 (f2 constant)
   IN:      H:X @f1, 0,RA f2
   OUT:     result on top of stack
   WRITTEN: none */
  asm {
    _ENTER_BINARY_D_RC();
    _SRET
    JMP _DMUL_Common
  }
}

#pragma NO_EXIT
void _DDIV(void) {
/* DESC:    divides two floating point (IEEE64) values (f1 / f2)
   IN:      &f2: on stack, &f1 in HX
   OUT:     result on top of stack
   WRITTEN: none */
  asm {
    _ENTER_BINARY_D();
    JMP _DDIV_Common
  }
}

#pragma NO_EXIT
void _DDIV_LC(void) {           /* LC means left operand will follow in code stream */
/* DESC:    IEEE64 division f1 / f2 (f1 constant)
   IN:      H:X @f2, 0,RA f1
   OUT:     result on top of stack
   WRITTEN: none */
  asm {
    _ENTER_BINARY_D_RC();
    _SRET
    JMP _DDIV_Common
  }
}

#pragma NO_EXIT
void _DDIV_RC(void) {           /* RC means right operand will follow in code stream */
/* DESC:    IEEE64 division f1 / f2 (f2 constant)
   IN:      H:X @f1, 0,RA f2
   OUT:     result on top of stack
   WRITTEN: none */
  asm {
    _ENTER_BINARY_D_LC();
    _SRET
    JMP _DDIV_Common
  }
}

#pragma NO_EXIT
void _DCMP(void) {
/* DESC:    Compares two IEEE64 values (f1 cmp f2)
   IN:      Stack: &f2, HX: &f1
   OUT:     CCR
   WRITTEN: H,X */
  _ENTER_BINARY_L64();
  asm JSR _DCMP_k_rel_j;
}

#pragma NO_EXIT
void _DCMP_RC(void) {
/* DESC:    IEEE64 comparison f1 cmp f2 (f2 constant)
   IN:      H:X @f1, 0,RA f2
   OUT:     CCR
   WRITTEN: H,X */
  _ENTER_BINARY_L64_RC();
  asm _SRET;
  asm JSR _DCMP_k_rel_j;
}

static const double plus1d=+1.0;
static const double minus1d=-1.0;

#pragma NO_EXIT
void _DINC(void) {
/* DESC:    increments a IEEE64 value by 1.0 (f++)
   IN:      H:X contains &f
   OUT:     TOS: f+1.0
   WRITTEN: none */

   asm {
     AIS #-2
     PSHH             ; // save H&X for frame optimization
     PSHX
     PSHA
#ifdef __HCS08__
     LDHX  6,SP       ; // move return address
     STHX  4,SP
     LDHX  @plus1d    ; // store address of constant
     STHX  6,SP
#else
     LDA   6,SP       ; // move return address
     STA   4,SP
     LDA   7,SP
     STA   5,SP
     LDA  @plus1d     ; // store address of constant
     STA   7,SP
     LDA  @plus1d:MSB
     STA   6,SP
#endif
     PULA
     PULX
     PULH
     JMP _DADD
   }
}

#pragma NO_EXIT
void _DDEC(void) {
/* DESC:    subtracts 1.0 from a IEEE64 value (f--)
   IN:      H:X contains &f
   OUT:     TOS: f-1.0
   WRITTEN: none */

   asm {
     AIS #-2
     PSHH             ; // save H&X for frame optimization
     PSHX
     PSHA
#ifdef __HCS08__
     LDHX  6,SP       ; // move return address 
     STHX  4,SP
     LDHX  @minus1d   ; // store address of constant 
     STHX  6,SP
#else
     LDA   6,SP       ; // move return address 
     STA   4,SP
     LDA   7,SP
     STA   5,SP
     LDA  @minus1d    ; // store address of constant 
     STA   7,SP
     LDA  @minus1d:MSB
     STA   6,SP
#endif
     PULA
     PULX
     PULH
     JMP _DADD
   }
}

void _DNEG(_PARAM_UNARY_L64_INLINE) {
/* DESC:     negates a IEEE64 floating point value (f1 => -f1)
   IN:       HX: &f1
   OUT:      result is on stack (TOS, 64bit)
   WRITTEN:  none */

  _ENTER_UNARY_L64();
  asm {
    LDA   k:7
    ORA   k:6
    ORA   k:5
    ORA   k:4
    ORA   k:3
    ORA   k:2
    ORA   k:1
    ORA   k:0
    BEQ   isZero
    LDA   k:0
    EOR   #0x80
    STA   k:0
  isZero:
    _EXIT_UNARY_L64
    RTS
  }
}

void _DABS(_PARAM_UNARY_L64_INLINE) {
/* DESC:     absolute value of a IEEE64 value (-f1 => f1, f1 => f1)
   IN:       HX: &f1
   OUT:      result is on stack (TOS, 64bit)
   WRITTEN:  none */

  _ENTER_UNARY_L64();
  asm {
    LDA   k
    AND   #0x7F  ; // clear sign bit
    STA   k
  finish:
    _EXIT_UNARY_L64
    RTS
  }
}

void _DSFLOAT(_PARAM_UNARY_L64_INLINE) {
/* DESC:    converts a 32bit signed integral value into a IEEE64 value (f = L)
   IN:      H:X contains &L
   OUT:     TOS: converted IEEE64 value
   WRITTEN: none */
  asm {
                _ENTER_UNARY_D4()
                TST   k:4
                BMI   negative
                JSR   _D_FLOATK
                BRA   done
  negative:
                LDHX  @k:4
                JSR   _NEG_L_HX
                JSR   _D_FLOATK
                COM   k:0                      ; // invert sign
  done:
                JSR   _DPACK_K_to_k_Unary;    /* returns itself */
  }
}

#pragma NO_EXIT
void _DUFLOAT(void) {
/* DESC:    converts a 32bit unsigned integral value into a IEEE64 value (f = uL)
   IN:      H:X contains &uL
   OUT:     TOS: converted IEEE64 value
   WRITTEN: none */
  asm {
    _ENTER_UNARY_D4()
    JSR _D_FLOATK;
    JSR _DPACK_K_to_k_Unary;      /* returns itself */
  }
}

#pragma NO_EXIT
void _DSTRUNC(void) {
/* DESC:    returns the integral part of a floating point (IEEE64) value
   IN:      HX: address of floating point value
   OUT:     result is on stack (TOS, 32bit)
   WRITTEN: none */
  asm {
    _ENTER_UNARY_D();
    JSR _DUNPACK_k_to_K_Unary;
    JSR _D_LONGK;     /* returns itself */
  }
}

#pragma NO_EXIT
void _DUTRUNC(void) {
/* DESC:    returns the integral part of a floating point (IEEE64) value (range [0 .. 2^32[)
   IN:      HX: address of floating point value
   OUT:     result is on stack (TOS, 32bit)
   WRITTEN: none */
  asm JMP _DSTRUNC;  /* NYI */
}


#pragma NO_EXIT
void _DSHORT(void) {
/* DESC:    converts a floating point IEEE64 to a floating point IEEE32 value
   IN:      HX: address of IEEE64 value
   OUT:     result is on stack (TOS, 32bit)
   WRITTEN: none */
  asm {
    _ENTER_UNARY_D();
    JSR _DUNPACK_k_to_K_Unary;
    JSR _D_DTOFK;     /* returns itself */
  }
}

void _DLONG(void) {
/* DESC:    converts a floating point IEEE64 to a floating point IEEE32 value
   IN:      H:X contains &f
   OUT:     TOS: converted IEEE64 value
   WRITTEN: none */
  asm {
    _ENTER_UNARY_D4();
    JSR _D_FTODK;
    JSR _DPACK_K_to_k_Unary  /* returns itself */
  }
}

#endif /* __NO_FLOAT__ */
